      program predict_starting_model

!.... ENTER A SPECTRAL TYPE AND LUMINOSITY CLASS TO DETERMINE THE 
!.... TEFF AND LOG G OF A PLANE-PARALLEL MODEL TO SERVE AS THE 
!.... STARTING MODEL

      use var_types

      implicit none

!------------------------------ CONSTANTS ------------------------------

!.... TABLES FROM SECTION 15.3.1 OF ASTROPHYSICAL QUANTITIES
!.... THE SPECTRAL TYPES TAKEN FROM TABLE 15.8

      character(len=2), parameter :: st_g(8) =   ! GIANTS
     &   [ "B0", "B5", "A0", "G0", "G5", "K0", "K5", "M0" ]
      character(len=2), parameter :: st_ms(20) = ! MAIN SEQUENCE
     &   [ "O3", "O5", "O6", "O8", "B0", "B3", "B5", "B8", "A0", "A5", 
     &     "F0", "F5", "G0", "G5", "K0", "K5", "M0", "M2", "M5", "M8" ]
      character(len=2), parameter :: st_sg(15) = ! SUPERGIANTS
     &   [ "O5", "O6", "O8", "B0", "B5", "A0", "A5", "F0", "F5", "G0", 
     &     "G5", "K0", "K5", "M0", "M2" ]

!.... LOG G IN CGS UNITS

      real(re_type), parameter :: lg_g_g(8) =   ! GIANTS
     &   [ 3.3, 3.5, 3.6, 3.0, 2.5, 2.1, 1.7, 1.3 ]
      real(re_type), parameter :: lg_g_ms(20) = ! MAIN SEQUENCE
     &   [ 4.1, 4.0, 4.0, 3.9, 3.9, 3.9, 4.0, 4.0, 4.1, 4.3, 4.3, 4.3, 
     &     4.4, 4.5, 4.5, 4.5, 4.6, 4.6, 4.9, 4.9 ]
      real(re_type), parameter :: lg_g_sg(15) = ! SUPERGIANTS
     &   [ 3.3, 3.2, 3.2, 2.8, 2.4, 2.1, 2.0, 1.7, 1.4, 1.3, 1.1, 0.9, 
     &     0.3, 0.1, 0.0 ]

!.... THESE TEFF ARE INTERPOLATED FROM TABLE 15.7 TO THE SPECTRAL TYPES
!.... IN TABLE 15.8

      real(re_type), parameter :: teff_g(8) =   ! GIANTS
     &   [ 30000.0, 14400.0, 9900.0, 5700.0, 5050.0,
     &      4660.0,  4050.0, 3690.0 ]
      real(re_type), parameter :: teff_ms(20) = ! MAIN SEQUENCE
     &   [ 50000.0, 42000.0, 39000.0, 35000.0, 30000.0,
     &     22000.0, 15200.0, 11400.0,  9790.0,  8180.0,
     &      7300.0,  6650.0,  5940.0,  5560.0,  5150.0,
     &      4410.0,  3840.0,  3520.0,  3170.0,  3000.0 ]
      real(re_type), parameter :: teff_sg(15) = ! SUPERGIANTS
     &   [ 50000.0, 40000.0, 35000.0, 22000.0, 13600.0,
     &      9980.0,  8610.0,  7460.0,  6370.0,  5370.0,
     &      4930.0,  4550.0,  3990.0,  3620.0,  3370.0 ]

!------------------------------ VARIABLES ------------------------------

      character(len=3) :: lum_class
      character(len=2) :: spec_type

      integer(in_type) :: i
      integer(in_type) :: subtype
      integer(in_type) :: subtype_g(8)
      integer(in_type) :: subtype_ms(20)
      integer(in_type) :: subtype_sg(15)

      real(re_type) :: frac
      real(re_type) :: model_g
      real(re_type) :: model_t

!------------------------------ EXECUTION ------------------------------

!.... SET UP THE SPECTRAL SUBTYPES AS INTEGERS

      do i = 1, 8
         read(st_g(i)(2:2), '(i1)') subtype_g(i)
      end do

      do i = 1, 20
         read(st_ms(i)(2:2), '(i1)') subtype_ms(i)
      end do

      do i = 1, 15
         read(st_sg(i)(2:2), '(i1)') subtype_sg(i)
      end do

      write(*, '(a)', advance="no") "ENTER SPECTRAL TYPE, E.G. G2: "
      read(*, '(a)') spec_type
      write(*, '(2a)', advance="no") "ENTER LUMINOSITY CLASS: ",
     &   "I, III OR V: "
      read(*, '(a)') lum_class
      write(*, '(4a)') "spectral type = ", spec_type,
     &   " luminosity class = ", lum_class

      read(spec_type(2:2), '(i1)') subtype ! MAKE AN INTEGER

      if(lum_class .eq. "V  ") then ! MAIN SEQUENCE
         write(*, '(a)') "MAIN SEQUENCE"
         i = 1

         do

            if(spec_type(1:1) .eq. st_ms(i)(1:1)) then ! FOUND TYPE

               do ! IDENTIFY SUBTYPE

                  if(i .eq. 20) then
                     model_g = lg_g_ms(i)
                     model_t = teff_ms(i)
                     exit

                  else if(subtype_ms(i+1) .ge. subtype) then ! BRACKET
                     frac = real(subtype - subtype_ms(i)) / 
     &                      real(subtype_ms(i+1) - subtype_ms(i))
                     model_t = teff_ms(i) +
     &                         frac * (teff_ms(i+1) - teff_ms(i))
                     model_g = lg_g_ms(i) +
     &                         frac * (lg_g_ms(i+1) - lg_g_ms(i))
                     exit

                  else if(subtype_ms(i+1) .le.  subtype_ms(i)) then 
                     frac = real(subtype - subtype_ms(i)) /
     &                      real(abs(subtype_ms(i+1) - subtype_ms(i)))
                     model_t = teff_ms(i) +
     &                         frac * (teff_ms(i+1) - teff_ms(i))
                     model_g = lg_g_ms(i) +
     &                         frac * (lg_g_ms(i+1) - lg_g_ms(i))
                     exit

                  else
                     i = i + 1
                  end if

               end do ! SUBTYPE

               write(*, '(a, f8.0)') "INTERPOLATED TEFF = ", model_t
               write(*, '(a, f8.1)') "INTERPOLATED LOG G = ", model_g
               exit

            else
               i = i + 1

               if(i .gt. 20) then
                  stop "ERROR - MAIN SEQUENCE SUBTYPE"
               end if

            end if ! SPECTRAL TYPE

         end do

      else if(lum_class .eq. "III") then ! GIANT
         write(*, '(a)') "GIANT"
         i = 1

         do

            if(spec_type(1:1) .eq. "F") then ! BRACKET

!.... FOR GIANTS THE TABLES HAVE NO F TYPES

               i = 3
               frac = real(10 + subtype) / 20.0
               model_t = teff_g(i) + frac * (teff_g(i+1) - teff_g(i))
               model_g = lg_g_g(i) + frac * (lg_g_g(i+1) - lg_g_g(i))

               write(*, '(a, f8.0)') "INTERPOLATED TEFF = ", model_t
               write(*, '(a, f8.1)') "INTERPOLATED LOG G = ", model_g
               exit

            else if(spec_type(1:1) .eq. st_g(i)(1:1)) then ! FOUND TYPE

               do ! IDENTIFY SUBTYPE

                  if(i .eq. 8) then ! AT M0
                     model_g = lg_g_g(i)
                     model_t = teff_g(i)
                     exit

!.... FOR GIANTS THE TABLES HAVE NO F TYPES

                  else if(spec_type(1:1) .eq. "A") then ! BRACKET
                     frac = real(subtype) / 20.0
                     model_t = teff_g(i) +
     &                         frac * (teff_g(i+1) - teff_g(i))
                     model_g = lg_g_g(i) +
     &                         frac * (lg_g_g(i+1) - lg_g_g(i))
                     exit

                  else if(subtype_g(i+1) .ge. subtype) then ! BRACKET SUBTYPE
                     frac = real(subtype - subtype_g(i)) /
     &                      real(subtype_g(i+1) - subtype_g(i))
                     model_t = teff_g(i) +
     &                         frac * (teff_g(i+1) - teff_g(i))
                     model_g = lg_g_g(i) +
     &                         frac * (lg_g_g(i+1) - lg_g_g(i))
                     exit

                  else if(subtype_g(i+1) .le. subtype_g(i)) then ! BRACKET
                     frac = real(subtype - subtype_g(i)) / 
     &                      real(abs(subtype_g(i+1) - subtype_g(i)))
                     model_t = teff_g(i) +
     &                         frac * (teff_g(i+1) - teff_g(i))
                     model_g = lg_g_g(i) +
     &                         frac * (lg_g_g(i+1) - lg_g_g(i))
                     exit

                  else
                     i = i + 1
                  end if

               end do ! SUBTYPE

               write(*, '(a, f8.0)') "INTERPOLATED TEFF = ", model_t
               write(*, '(a, f8.1)') "INTERPOLATED LOG G = ", model_g
               exit

            else
               i = i + 1

               if(i .gt. 8) then
                  stop "ERROR - CANNOT IDENTIFY TYPE"
               end if

            end if ! SPECTRAL TYPE

         end do

      else if(lum_class .eq. "I  ") then ! SUPERGIANT
         write(*, '(a)') "SUPERGIANT"
         i = 1

         do
            if(spec_type(1:1) .eq. st_sg(i)(1:1)) then ! FOUND TYPE

               do ! IDENTIFY SUBTYPE

                  if(i .eq. 15) then ! AT END
                     model_g = lg_g_sg(i)
                     exit

                  else if(subtype_sg(i+1) .ge. subtype) then ! BRACKET SUBTYPE
                     frac = real(subtype - subtype_sg(i)) /
     &                      real(subtype_sg(i+1) - subtype_sg(i))
                     model_t = teff_sg(i) +
     &                         frac * (teff_sg(i+1) - teff_sg(i))
                     model_g = lg_g_sg(i) +
     &                         frac * (lg_g_sg(i+1) - lg_g_sg(i))
                     exit

                  else if(subtype_sg(i+1) .le. subtype_sg(i)) then
                     frac =  real(subtype - subtype_sg(i)) /
     &                       real(abs(subtype_sg(i+1) - subtype_sg(i)))
                     model_t = teff_sg(i) +
     &                         frac * (teff_sg(i+1) - teff_sg(i))
                     model_g = lg_g_sg(i) +
     &                         frac * (lg_g_sg(i+1) - lg_g_sg(i))
                     exit
                  
                  else
                     i = i + 1
                  end if

               end do ! SUBTYPE

               write(*, '(a, f8.0)') "INTERPOLATED TEFF = ", model_t
               write(*, '(a, f8.1)') "INTERPOLATED LOG G = ", model_g
               exit

            else
               i = i + 1

               if(i .gt. 15) then
                  stop "ERROR - CANNOT IDENTIFY TYPE"
               end if

            end if ! SPECTRAL TYPE

         end do

      else
         stop "ERROR - CANNOT RECOGNIZE LUMINOSITY CLASS"
      end if

      end program predict_starting_model
