      subroutine josh ! BOB'S TRANSFER ROUTINE

!.... 2019 APR - REPLACED forall BY do concurrent
!.... 2014 MAY - CHANGED module_atlas_dimensions TO module_code_dimensions
!.... 2014 APR - RENAMED rhox TO rhodz FOR RHO * DZ
!.... 2011 JUN - MADE IF_INT AND IF_SFLUX ARRAYS DIMENSION MAX_ITER
!.... 2007 DEC - CHANGED jmins TO jmins_nu
!....          - CHANGED abtot TO abtot_nu, alpha TO alpha_nu
!.... 2007 JUN - REPLACED ifsurf NUMBER BY LOGICALS if_sflux AND if_int
!.... 2007 MAR - CHANGED nrhox TO ndepth
!.... 2007 JAN - CHANGED maxd TO max_d

!.... ALWAYS SOLVE INTEGRAL EQUATION FOR SOURCE FUNCTION, IFSCAT = 1
!.... DEFAULT IS TO ALWAYS CALCULATE J AND H
!.... if_int(iter) = .true. CALCULATE SURFACE SPECIFIC INTENSITY

      use abtot_vars                 ! abtot_nu, alpha_nu
      use atmosphere_parameters, only: ndepth
      use code_dimensions,       only: max_d, max_mu
      use freq_vars,             only: bnu
      use if_vars,               only: if_int
      use intensity_vars,        only: n_mu, surf_int, surf_mu
      use iter_vars,             only: iter
      use rad_vars                   ! hnu, jmins_nu, jnu, knu, snu,
                                     ! taunu
      use rhodz_var                  ! rhodz
      use total_opacity              ! a_cont, a_line, s_cont, s_line,
                                     ! sigma_c, sigma_l
      use var_types

      implicit none

!.... BOB HAS THIS COMMON, BUT HE NEVER USES THE VARIABLES. COMMENT OUT

!!!!  common / prd /    prddop, prdpow, itprd, nitprd, sigprd, nuprd,
!!!! &                  linprd
!!!!  real(re_type)  prddop, prdpow, sigprd(max_d)
!!!!  integer(in_type)           itprd, nitprd, nuprd, linprd

!--------------------------- INTERFACE BLOCK ---------------------------

      interface

         subroutine deriv(x, f, dfdx)
         use var_types
         real(re_type), intent(out) :: dfdx(:)
         real(re_type), intent(in)  :: f(:)
         real(re_type), intent(in)  :: x(:)
         end subroutine deriv

         subroutine integ(x, f, fint, start)
         use var_types
         real(re_type), intent(in)  :: f(:)
         real(re_type), intent(out) :: fint(:)
         real(re_type), intent(in)  :: start
         real(re_type), intent(in)  :: x(:)
         end subroutine integ

         function map_cs(x_old, f_old, x_new, f_new) result(map_1)
         use var_types
         integer(in_type)           :: map_1
         real(re_type), intent(out) :: f_new(:)
         real(re_type), intent(in)  :: f_old(:)
         real(re_type), intent(in)  :: x_new(:)
         real(re_type), intent(in)  :: x_old(:)
         end function map_cs

         function map1(x_old, f_old, x_new, f_new) result(map_1)
         use var_types
         integer(in_type)           :: map_1
         real(re_type), intent(out) :: f_new(:)
         real(re_type), intent(in)  :: f_old(:)
         real(re_type), intent(in)  :: x_new(:)
         real(re_type), intent(in)  :: x_old(:)
         end function map1

      end interface

!--------------------------- josh CONSTANTS ----------------------------

      integer(in_type), parameter :: nxtau = 51

!!!!  real(re_type), paramater :: ch(nxtau) = [
      real(r4_type), parameter :: ch(nxtau) = [
     & 7.15528131E-07, 1.49142693E-06, 1.52106577E-06, 2.98150826E-06,
     & 5.33941056E-06, 9.13329677E-06, 1.61715943E-05, 2.97035986E-05,
     & 5.33166603E-05, 9.11154202E-05, 1.61084638E-04, 2.95118050E-04,
     & 5.27450291E-04, 8.67939554E-04, 1.61498412E-03, 2.50720908E-03,
     & 3.20994272E-03, 5.61912498E-03, 8.60872678E-03, 1.04706492E-02,
     & 1.33110350E-02, 1.62635669E-02, 1.90288834E-02, 2.18877215E-02,
     & 2.36015432E-02, 2.10819542E-02, 1.80345085E-02, 1.64786074E-02,
     & 1.49382707E-02, 1.19676525E-02, 9.90213640E-03, 8.17766134E-03,
     & 6.11252524E-03, 4.84035723E-03, 3.06078210E-03, 2.40512565E-03,
     & 2.01712688E-03, 1.33288081E-03, 7.83530239E-04, 4.31428343E-04,
     & 1.76504589E-04, 4.75738016E-05, 1.65963702E-05, 5.41117970E-06,
     & 2.08043571E-06, 7.11612643E-07, 8.08788982E-08, 1.95130507E-08,
     & 4.33638281E-09, 8.87765583E-10, 3.90236420E-11 ]

!!!!  real(re_type), parameter :: ck(nxtau) = [
      real(r4_type), parameter :: ck(nxtau) = [
     & 3.57771910E-07, 7.45730404E-07, 7.60575176E-07, 1.49091113E-06,
     & 2.67016185E-06, 4.56793896E-06, 8.08956065E-06, 1.48632944E-05,
     & 2.66928291E-05, 4.56529851E-05, 8.08134864E-05, 1.48363324E-04,
     & 2.66052346E-04, 4.39771306E-04, 8.25088180E-04, 1.29440730E-03,
     & 1.67680858E-03, 2.98973685E-03, 4.68314718E-03, 5.84855257E-03,
     & 7.64854718E-03, 9.63155832E-03, 1.16419578E-02, 1.38551742E-02,
     & 1.54840983E-02, 1.42877987E-02, 1.25930300E-02, 1.17983138E-02,
     & 1.09717194E-02, 8.98320694E-03, 7.59950886E-03, 6.38808031E-03,
     & 4.86854184E-03, 3.91568616E-03, 2.51398841E-03, 2.00142385E-03,
     & 1.70069211E-03, 1.14058319E-03, 6.80292083E-04, 3.80097074E-04,
     & 1.57705377E-04, 4.31706540E-05, 1.51795348E-05, 4.98576401E-06,
     & 1.92979223E-06, 6.63957223E-07, 7.65236692E-08, 1.84933668E-08,
     & 4.12596224E-09, 8.47334369E-10, 3.81791959E-11 ]

      real(re_type), parameter :: third = 1.0d0/3.0d0
      real(re_type), parameter :: real_tiny = tiny(1.0)

      real(re_type), parameter :: xtau8(nxtau) = [
     &    0.0,      0.0000032, 0.0000056, 0.00001, 0.000018,
     &    0.000032, 0.000056,  0.0001,    0.00018, 0.00032,
     &    0.00056,  0.001,     0.0018,    0.0032,  0.0056,
     &    0.01,     0.016,     0.025,     0.042,   0.065,
     &    0.096,    0.139,     0.196,     0.273,   0.375,
     &    0.5,      0.63,      0.78,      0.95,    1.15,
     &    1.35,     1.6,       1.85,      2.15,    2.45,
     &    2.75,     3.15,      3.65,      4.25,    5.0,
     &    6.0,      7.0,       8.0,       9.0,    10.0,
     &   11.5,     13.0,      14.5,      16.0,    18.0,
     &   20.0 ]

!--------------------------- josh VARIABLES ----------------------------

      integer(in_type) :: i_mu
      integer(in_type) :: j
      integer(in_type) :: k
      integer(in_type) :: l
      integer(in_type) :: m
      integer(in_type) :: maxj
      integer(in_type) :: maxj1
      integer(in_type) :: mdummy

      logical :: if_err
      logical :: if_neg

      real(re_type)       :: a(max_d)
      real(re_type)       :: b(max_d)
      real(re_type)       :: b2ct(max_d)
      real(re_type)       :: b2ct1(max_d)
      real(re_type)       :: c(max_d)

!!!!  real(re_type), save :: coef_j(nxtau, nxtau)
      real(r4_type), save :: coef_j(nxtau, nxtau)

!!!!  real(re_type), save :: coef_h(nxtau, nxtau)
      real(r4_type), save :: coef_h(nxtau, nxtau)

      real(re_type)       :: ctwo(max_d)

!!!!  real(re_type)       :: delxs
      real(r4_type)       :: delxs

!!!!  real(re_type)       :: diag(nxtau)
      real(r4_type)       :: diag(nxtau)

      real(re_type)       :: d
      real(re_type)       :: ddddd
      real(re_type)       :: error
      real(re_type)       :: extau(nxtau, max_mu)
      real(re_type)       :: new
      real(re_type)       :: old
      real(re_type)       :: snew
      real(re_type)       :: snubar(max_d)
      real(re_type)       :: sum_mu
      real(re_type)       :: tangle

!!!!  real(re_type)       :: xalpha(nxtau)
      real(r4_type)       :: xalpha(nxtau)
      real(re_type)       :: xalpha8(nxtau)

!!!!  real(re_type)       :: xh(nxtau)
      real(r4_type)       :: xh(nxtau)

      real(re_type)       :: xh8(nxtau)

!!!!  real(re_type)       :: xjs(nxtau)
      real(r4_type)       :: xjs(nxtau)

      real(re_type)       :: xjs8(nxtau)

!!!!  real(re_type)       :: xs(nxtau)
      real(r4_type)       :: xs(nxtau)
      real(re_type)       :: xs8(nxtau)

!!!!  real(re_type)       :: xsbar(nxtau)
      real(r4_type)       :: xsbar(nxtau)

      real(re_type)       :: xsbar8(nxtau)

!--------------------------- INITIALIZATION ----------------------------

      data coef_h(1:51, 1) /
     &  7.15528120e-07,-7.63679981e-07,-7.15500186e-07,-7.15459313e-07,
     & -7.15399161e-07,-7.15312983e-07,-7.15165254e-07,-7.14858701e-07,
     & -7.14382582e-07,-7.13607800e-07,-7.12377050e-07,-7.10300825e-07,
     & -7.06860448e-07,-7.01420388e-07,-6.93063611e-07,-6.79532139e-07,
     & -6.63270842e-07,-6.41769994e-07,-6.06941637e-07,-5.67254987e-07,
     & -5.22282879e-07,-4.70473751e-07,-4.14388755e-07,-3.53840218e-07,
     & -2.91560381e-07,-2.33727889e-07,-1.88154634e-07,-1.48244615e-07,
     & -1.14444272e-07,-8.53987435e-08,-6.43470333e-08,-4.56539380e-08,
     & -3.26942276e-08,-2.21150989e-08,-1.50875635e-08,-1.03652019e-08,
     & -6.33939388e-09,-3.46859459e-09,-1.70493170e-09,-7.13016572e-10,
     & -2.27726935e-10,-7.40657549e-11,-2.44269562e-11,-8.14546806e-12,
     & -2.74070206e-12,-5.42231753e-13,-1.08693370e-13,-2.20183137e-14,
     & -4.49889022e-15,-5.47402692e-16,-6.72957970e-17/

      data coef_h(1:51, 2) /
     &  1.49142688e-06,
     & -3.90281313e-07,-1.61431531e-06,-1.49134752e-06,-1.49120973e-06,
     & -1.49097554e-06,-1.49063560e-06,-1.49006119e-06,-1.48907404e-06,
     & -1.48745761e-06,-1.48489081e-06,-1.48056166e-06,-1.47338907e-06,
     & -1.46204838e-06,-1.44462816e-06,-1.41642180e-06,-1.38252570e-06,
     & -1.33770838e-06,-1.26511114e-06,-1.18238754e-06,-1.08864711e-06,
     & -9.80655730e-07,-8.63751857e-07,-7.37544289e-07,-6.07728094e-07,
     & -4.87182021e-07,-3.92189148e-07,-3.09000744e-07,-2.38547359e-07,
     & -1.78004914e-07,-1.34124776e-07,-9.51609333e-08,-6.81477476e-08,
     & -4.60966417e-08,-3.14484678e-08,-2.16051921e-08,-1.32138108e-08,
     & -7.22992635e-09,-3.55375351e-09,-1.48620912e-09,-4.74673171e-10,
     & -1.54382378e-10,-5.09154535e-11,-1.69783820e-11,-5.71270866e-12,
     & -1.13022573e-12,-2.26560028e-13,-4.58948851e-14,-9.37746872e-15,
     & -1.14100396e-15,-1.40271087e-16/

      data coef_h(1:51, 3) /
     &  1.52106578e-06, 1.68212841e-06,
     &  4.09944249e-07,-1.75645270e-06,-1.52097046e-06,-1.52073553e-06,
     & -1.52034786e-06,-1.51978995e-06,-1.51877481e-06,-1.51712012e-06,
     & -1.51449942e-06,-1.51008118e-06,-1.50276284e-06,-1.49119345e-06,
     & -1.47342353e-06,-1.44465254e-06,-1.41007905e-06,-1.36436695e-06,
     & -1.29032132e-06,-1.20594808e-06,-1.11033885e-06,-1.00019498e-06,
     & -8.80961299e-07,-7.52238763e-07,-6.19835885e-07,-4.96887979e-07,
     & -4.00002484e-07,-3.15156696e-07,-2.43299681e-07,-1.81551082e-07,
     & -1.36796756e-07,-9.70566781e-08,-6.95053428e-08,-4.70149442e-08,
     & -3.20749578e-08,-2.20355911e-08,-1.34770434e-08,-7.37395328e-09,
     & -3.62454737e-09,-1.51581560e-09,-4.84129008e-10,-1.57457780e-10,
     & -5.19297218e-11,-1.73166017e-11,-5.82650912e-12,-1.15274044e-12,
     & -2.31073221e-13,-4.68091342e-14,-9.56427242e-15,-1.16373329e-15,
     & -1.43065350e-16/

      data coef_h(1:51, 4) /
     &  2.98150821e-06, 2.98161961e-06, 3.02985925e-06,
     &  5.50707149e-07,-3.40191479e-06,-2.98116452e-06,-2.98048320e-06,
     & -2.97927252e-06,-2.97725646e-06,-2.97401344e-06,-2.96886903e-06,
     & -2.96020082e-06,-2.94584763e-06,-2.92316158e-06,-2.88832120e-06,
     & -2.83191590e-06,-2.76413769e-06,-2.67452550e-06,-2.52937199e-06,
     & -2.36397499e-06,-2.17655345e-06,-1.96064093e-06,-1.72691069e-06,
     & -1.47458038e-06,-1.21503614e-06,-9.74026455e-07,-7.84106027e-07,
     & -6.17786634e-07,-4.76928629e-07,-3.55885746e-07,-2.68155970e-07,
     & -1.90255412e-07,-1.36247871e-07,-9.21610475e-08,-6.28749313e-08,
     & -4.31952603e-08,-2.64183673e-08,-1.44547867e-08,-7.10501576e-09,
     & -2.97137592e-09,-9.49013288e-10,-3.08656397e-10,-1.01795161e-10,
     & -3.39448415e-11,-1.14214051e-11,-2.25965748e-12,-4.52960874e-13,
     & -9.17575200e-14,-1.87483472e-14,-2.28120596e-15,-2.80443572e-16/

      data coef_h(1:51, 5) /
     &  5.33941059e-06, 5.33959050e-06, 5.33971838e-06, 5.46288397e-06,
     &  9.45594076e-07,-5.99346507e-06,-5.33838396e-06,-5.33622077e-06,
     & -5.33261638e-06,-5.32678301e-06,-5.31753886e-06,-5.30199406e-06,
     & -5.27626139e-06,-5.23560527e-06,-5.17318158e-06,-5.07213438e-06,
     & -4.95072325e-06,-4.79020869e-06,-4.53021687e-06,-4.23397287e-06,
     & -3.89828536e-06,-3.51157189e-06,-3.09294861e-06,-2.64101390e-06,
     & -2.17616046e-06,-1.74450433e-06,-1.40435137e-06,-1.10646897e-06,
     & -8.54188854e-07,-6.37398304e-07,-4.80272397e-07,-3.40750911e-07,
     & -2.44022354e-07,-1.65062027e-07,-1.12610064e-07,-7.73634273e-08,
     & -4.73157247e-08,-2.58887527e-08,-1.27251939e-08,-5.32177962e-09,
     & -1.69969699e-09,-5.52808151e-10,-1.82316613e-10,-6.07956998e-11,
     & -2.04558995e-11,-4.04707848e-12,-8.11259273e-13,-1.64338996e-13,
     & -3.35785494e-14,-4.08567126e-15,-5.02278278e-16/

      data coef_h(1:51, 6) /
     &  9.13329675e-06,
     &  9.13358176e-06, 9.13380306e-06, 9.13420159e-06, 9.37031962e-06,
     &  1.47104366e-06,-1.03630349e-05,-9.13025211e-06,-9.12398505e-06,
     & -9.11390737e-06,-9.09802697e-06,-9.07135096e-06,-9.02725015e-06,
     & -8.95762338e-06,-8.85075952e-06,-8.67781716e-06,-8.47005078e-06,
     & -8.19539043e-06,-7.75053819e-06,-7.24367751e-06,-6.66934449e-06,
     & -6.00772113e-06,-5.29151194e-06,-4.51831779e-06,-3.72302663e-06,
     & -2.98453429e-06,-2.40259068e-06,-1.89296597e-06,-1.46135960e-06,
     & -1.09047012e-06,-8.21656389e-07,-5.82960842e-07,-4.17476248e-07,
     & -2.82389890e-07,-1.92654440e-07,-1.32354098e-07,-8.09481724e-08,
     & -4.42906997e-08,-2.17703641e-08,-9.10454075e-09,-2.90785360e-09,
     & -9.45747879e-10,-3.11908427e-10,-1.04009658e-10,-3.49960749e-11,
     & -6.92376506e-12,-1.38790687e-12,-2.81152041e-13,-5.74463594e-14,
     & -6.98978740e-15,-8.59300205e-16/

      data coef_h(1:51, 7) /
     &  1.61715949e-05, 1.61720720e-05,
     &  1.61724330e-05, 1.61730963e-05, 1.61743243e-05, 1.65968833e-05,
     &  3.13002612e-06,-1.85271744e-05,-1.61620847e-05,-1.61440014e-05,
     & -1.61156455e-05,-1.60681414e-05,-1.59898028e-05,-1.58662617e-05,
     & -1.56767807e-05,-1.53702665e-05,-1.50021225e-05,-1.45155180e-05,
     & -1.37274731e-05,-1.28296443e-05,-1.18123413e-05,-1.06404566e-05,
     & -9.37191392e-06,-8.00246045e-06,-6.59388527e-06,-5.28591985e-06,
     & -4.25522842e-06,-3.35262610e-06,-2.58820551e-06,-1.93132270e-06,
     & -1.45522738e-06,-1.03247504e-06,-7.39386642e-07,-5.00136599e-07,
     & -3.41207244e-07,-2.34410137e-07,-1.43365879e-07,-7.84424320e-08,
     & -3.85570653e-08,-1.61248655e-08,-5.15003730e-09,-1.67499319e-09,
     & -5.52413963e-10,-1.84209108e-10,-6.19807297e-11,-1.22625154e-11,
     & -2.45808833e-12,-4.97941514e-13,-1.01741831e-13,-1.23794385e-14,
     & -1.52188503e-15/

      data coef_h(1:51, 8) /
     &  2.97035986e-05, 2.97044194e-05, 2.97050368e-05,
     &  2.97061753e-05, 2.97082586e-05, 2.97119611e-05, 3.03723299e-05,
     &  5.59251004e-06,-3.39127760e-05,-2.96749188e-05,-2.96219260e-05,
     & -2.95338145e-05,-2.93890504e-05,-2.91612567e-05,-2.88123315e-05,
     & -2.82483380e-05,-2.75712512e-05,-2.66765293e-05,-2.52278235e-05,
     & -2.35775061e-05,-2.17077264e-05,-1.95539426e-05,-1.72226003e-05,
     & -1.47058714e-05,-1.21172992e-05,-9.71365644e-06,-7.81957905e-06,
     & -6.16090061e-06,-4.75616197e-06,-3.54904619e-06,-2.67415660e-06,
     & -1.89729441e-06,-1.35870797e-06,-9.19057204e-07,-6.27005937e-07,
     & -4.30754070e-07,-2.63450092e-07,-1.44146186e-07,-7.08525779e-08,
     & -2.96310732e-08,-9.46370616e-09,-3.07796454e-09,-1.01511437e-09,
     & -3.38501999e-10,-1.13895526e-10,-2.25335364e-11,-4.51696914e-12,
     & -9.15014236e-13,-1.86960114e-13,-2.27483682e-14,-2.79660454e-15/

      data coef_h(1:51, 9) /
     &  5.33166603e-05, 5.33180333e-05, 5.33190646e-05, 5.33209602e-05,
     &  5.33244220e-05, 5.33305318e-05, 5.33411700e-05, 5.45903715e-05,
     &  9.45344212e-06,-5.98750523e-05,-5.32379559e-05,-5.30766673e-05,
     & -5.28137644e-05,-5.24019279e-05,-5.17726607e-05,-5.07570460e-05,
     & -4.95388096e-05,-4.79297869e-05,-4.53254263e-05,-4.23593245e-05,
     & -3.89992680e-05,-3.51292236e-05,-3.09404208e-05,-2.64187551e-05,
     & -2.17681883e-05,-1.74499821e-05,-1.40472898e-05,-1.10675313e-05,
     & -8.54399479e-06,-6.37549656e-06,-4.80383043e-06,-3.40827054e-06,
     & -2.44075547e-06,-1.65097143e-06,-1.12633544e-06,-7.73792885e-07,
     & -4.73252449e-07,-2.58938632e-07,-1.27276602e-07,-5.32279265e-08,
     & -1.70001466e-08,-5.52910028e-09,-1.82349841e-09,-6.08066807e-10,
     & -2.04595667e-10,-4.04779745e-11,-8.11402352e-12,-1.64367808e-12,
     & -3.35844075e-13,-4.08638019e-14,-5.02365045e-15/

      data coef_h(1:51, 10) /
     &  9.11154221e-05,
     &  9.11176038e-05, 9.11192420e-05, 9.11222497e-05, 9.11277327e-05,
     &  9.11373748e-05, 9.11540492e-05, 9.11851448e-05, 9.35975830e-05,
     &  1.47063950e-05,-1.03462574e-04,-9.08852267e-05,-9.04268484e-05,
     & -8.97143077e-05,-8.86303398e-05,-8.68853564e-05,-8.47952605e-05,
     & -8.20369956e-05,-7.75751291e-05,-7.24955321e-05,-6.67426582e-05,
     & -6.01176981e-05,-5.29478997e-05,-4.52090066e-05,-3.72500018e-05,
     & -2.98601473e-05,-2.40372258e-05,-1.89381774e-05,-1.46199094e-05,
     & -1.09092379e-05,-8.21988044e-06,-5.83189075e-06,-4.17635691e-06,
     & -2.82495145e-06,-1.92724820e-06,-1.32401640e-06,-8.09767079e-07,
     & -4.43060175e-07,-2.17777566e-07,-9.10757716e-08,-2.90880576e-08,
     & -9.46053242e-09,-3.12008024e-09,-1.04042571e-09,-3.50070669e-10,
     & -6.92592007e-11,-1.38833572e-11,-2.81238403e-12,-5.74639184e-13,
     & -6.99191232e-14,-8.59560278e-15/

      data coef_h(1:51, 11) /
     &  1.61084635e-04, 1.61088204e-04,
     &  1.61090883e-04, 1.61095798e-04, 1.61104749e-04, 1.61120458e-04,
     &  1.61147526e-04, 1.61197628e-04, 1.61290476e-04, 1.65663482e-04,
     &  3.12833434e-05,-1.84767463e-04,-1.60383520e-04,-1.59095632e-04,
     & -1.57151752e-04,-1.54037373e-04,-1.50316849e-04,-1.45414273e-04,
     & -1.37492118e-04,-1.28479528e-04,-1.18276734e-04,-1.06530685e-04,
     & -9.38212441e-05,-8.01050512e-05,-6.60003185e-05,-5.29053047e-05,
     & -4.25875346e-05,-3.35527875e-05,-2.59017163e-05,-1.93273550e-05,
     & -1.45626021e-05,-1.03318579e-05,-7.39883160e-06,-5.00464369e-06,
     & -3.41426413e-06,-2.34558186e-06,-1.43454739e-06,-7.84901321e-07,
     & -3.85800856e-07,-1.61343209e-07,-5.15300229e-08,-1.67594408e-08,
     & -5.52724107e-09,-1.84311601e-09,-6.20149585e-10,-1.22692260e-10,
     & -2.45942378e-11,-4.98210441e-12,-1.01796509e-12,-1.23860554e-13,
     & -1.52269488e-14/

      data coef_h(1:51, 12) /
     &  2.95118060e-04, 2.95124067e-04, 2.95128573e-04,
     &  2.95136840e-04, 2.95151884e-04, 2.95178258e-04, 2.95223609e-04,
     &  2.95307219e-04, 2.95460872e-04, 2.95735401e-04, 3.02765668e-04,
     &  5.58676964e-05,-3.37573852e-04,-2.93054973e-04,-2.89397464e-04,
     & -2.83590510e-04,-2.76688687e-04,-2.67619889e-04,-2.52994639e-04,
     & -2.36378122e-04,-2.17582125e-04,-1.95954621e-04,-1.72562085e-04,
     & -1.47323476e-04,-1.21375266e-04,-9.72882826e-05,-7.83117813e-05,
     & -6.16962878e-05,-4.76263106e-05,-3.55369461e-05,-2.67755473e-05,
     & -1.89963282e-05,-1.36034154e-05,-9.20135573e-06,-6.27727001e-06,
     & -4.31241145e-06,-2.63742438e-06,-1.44303115e-06,-7.09283125e-07,
     & -2.96621802e-07,-9.47346057e-08,-3.08109284e-08,-1.01613469e-08,
     & -3.38839182e-09,-1.14008132e-09,-2.25556130e-10,-4.52136250e-11,
     & -9.15898950e-12,-1.87139993e-12,-2.27701364e-13,-2.79926878e-14/

      data coef_h(1:51, 13) /
     &  5.27450267e-04, 5.27460048e-04, 5.27467385e-04, 5.27480841e-04,
     &  5.27505322e-04, 5.27548208e-04, 5.27621863e-04, 5.27757350e-04,
     &  5.28005233e-04, 5.28444089e-04, 5.29212981e-04, 5.42982886e-04,
     &  9.43257257e-05,-5.94289096e-04,-5.21949862e-04,-5.11218873e-04,
     & -4.98593747e-04,-4.82097985e-04,-4.55597512e-04,-4.25563821e-04,
     & -3.91641331e-04,-3.52647471e-04,-3.10500866e-04,-2.65051281e-04,
     & -2.18341644e-04,-1.74994619e-04,-1.40851146e-04,-1.10959921e-04,
     & -8.56508809e-05,-6.39065263e-05,-4.81490961e-05,-3.41589441e-05,
     & -2.44608124e-05,-1.65448705e-05,-1.12868617e-05,-7.75380763e-06,
     & -4.74205493e-06,-2.59450211e-06,-1.27523489e-06,-5.33293310e-07,
     & -1.70319442e-07,-5.53929789e-08,-1.82682443e-08,-6.09165942e-09,
     & -2.04962735e-09,-4.05499382e-10,-8.12834463e-11,-1.64656199e-11,
     & -3.36430424e-12,-4.09347595e-13,-5.03233504e-14/

      data coef_h(1:51, 14) /
     &  8.67939526e-04,
     &  8.67954213e-04, 8.67965230e-04, 8.67985431e-04, 8.68022175e-04,
     &  8.68086519e-04, 8.68196952e-04, 8.68399838e-04, 8.68770152e-04,
     &  8.69422777e-04, 8.70555855e-04, 8.72685043e-04, 9.00316828e-04,
     &  1.18726820e-04,-1.05068040e-03,-8.51640282e-04,-8.30180005e-04,
     & -8.02328563e-04,-7.57838031e-04,-7.07603820e-04,-6.50991942e-04,
     & -5.86013815e-04,-5.15855799e-04,-4.40257134e-04,-3.62606802e-04,
     & -2.90576808e-04,-2.33857067e-04,-1.84211669e-04,-1.42183646e-04,
     & -1.06079989e-04,-7.99196360e-05,-5.66953416e-05,-4.05972023e-05,
     & -2.74581721e-05,-1.87312937e-05,-1.28676222e-05,-7.86932554e-06,
     & -4.30539149e-06,-2.11610499e-06,-8.84915842e-07,-2.82610874e-07,
     & -9.19117308e-08,-3.03114320e-08,-1.01074120e-08,-3.40075163e-09,
     & -6.72798403e-10,-1.34862967e-10,-2.73190325e-11,-5.58186980e-12,
     & -6.79162392e-13,-8.34926889e-14/

      data coef_h(1:51, 15) /
     &  1.61498414e-03, 1.61500857e-03,
     &  1.61502689e-03, 1.61506049e-03, 1.61512159e-03, 1.61522856e-03,
     &  1.61541206e-03, 1.61574893e-03, 1.61636286e-03, 1.61744182e-03,
     &  1.61930543e-03, 1.62277036e-03, 1.62924770e-03, 1.68333458e-03,
     &  3.60899758e-04,-1.82730765e-03,-1.58410667e-03,-1.52899488e-03,
     & -1.44234855e-03,-1.34546950e-03,-1.23689296e-03,-1.11271628e-03,
     & -9.78971597e-04,-8.35109934e-04,-6.87538020e-04,-5.50779439e-04,
     & -4.43161682e-04,-3.49011624e-04,-2.69337615e-04,-2.00915634e-04,
     & -1.51349791e-04,-1.07355641e-04,-7.68658545e-05,-5.19840976e-05,
     & -3.54597507e-05,-2.43579442e-05,-1.48953901e-05,-8.14890300e-06,
     & -4.00495346e-06,-1.67469957e-06,-5.34808728e-07,-1.73924801e-07,
     & -5.73564345e-08,-1.91250993e-08,-6.43470743e-09,-1.27299579e-09,
     & -2.55167484e-10,-5.16880700e-11,-1.05608383e-11,-1.28494760e-12,
     & -1.57962724e-13/

      data coef_h(1:51, 16) /
     &  2.50720905e-03, 2.50724320e-03, 2.50726881e-03,
     &  2.50731578e-03, 2.50740118e-03, 2.50755068e-03, 2.50780708e-03,
     &  2.50827754e-03, 2.50913426e-03, 2.51063770e-03, 2.51322757e-03,
     &  2.51801809e-03, 2.52687709e-03, 2.54289614e-03, 2.63774332e-03,
     &  1.76260415e-04,-2.78126196e-03,-2.45704320e-03,-2.31262955e-03,
     & -2.15394575e-03,-1.97774832e-03,-1.77739883e-03,-1.56245308e-03,
     & -1.33188374e-03,-1.09584644e-03,-8.77427902e-04,-7.05726344e-04,
     & -5.55621605e-04,-4.28668973e-04,-3.19696027e-04,-2.40783539e-04,
     & -1.70762699e-04,-1.22247776e-04,-8.26646695e-05,-5.63817215e-05,
     & -3.87261829e-05,-2.36795672e-05,-1.29532571e-05,-6.36557648e-06,
     & -2.66157673e-06,-8.49889370e-07,-2.76373703e-07,-9.11370404e-08,
     & -3.03877430e-08,-1.02237148e-08,-2.02250217e-09,-4.05390165e-10,
     & -8.21157965e-11,-1.67774240e-11,-2.04127682e-12,-2.50935807e-13/

      data coef_h(1:51, 17) /
     &  3.20994276e-03, 3.20998251e-03, 3.21001232e-03, 3.21006698e-03,
     &  3.21016637e-03, 3.21034034e-03, 3.21063867e-03, 3.21118592e-03,
     &  3.21218201e-03, 3.21392850e-03, 3.21693250e-03, 3.22247328e-03,
     &  3.23266196e-03, 3.25087011e-03, 3.28333413e-03, 3.52880121e-03,
     &  2.90599435e-04,-3.99201557e-03,-3.07880403e-03,-2.86239707e-03,
     & -2.62426711e-03,-2.35540482e-03,-2.06836350e-03,-1.76152461e-03,
     & -1.44821000e-03,-1.15882208e-03,-9.31623746e-04,-7.33184814e-04,
     & -5.65473991e-04,-4.21599458e-04,-3.17461291e-04,-2.25092147e-04,
     & -1.61113538e-04,-1.08927603e-04,-7.42843836e-05,-5.10170507e-05,
     & -3.11911486e-05,-1.70601847e-05,-8.38286988e-06,-3.50466057e-06,
     & -1.11897755e-06,-3.63847665e-07,-1.19974679e-07,-4.00009634e-08,
     & -1.34574290e-08,-2.66207243e-09,-5.33563710e-10,-1.08075026e-10,
     & -2.20806555e-11,-2.68642989e-12,-3.30236900e-13/

      data coef_h(1:51, 18) /
     &  5.61912502e-03,
     &  5.61918736e-03, 5.61923412e-03, 5.61931984e-03, 5.61947572e-03,
     &  5.61974854e-03, 5.62021633e-03, 5.62107432e-03, 5.62263546e-03,
     &  5.62537113e-03, 5.63007179e-03, 5.63872596e-03, 5.65458361e-03,
     &  5.68273038e-03, 5.73223829e-03, 5.82772200e-03, 6.17039808e-03,
     &  1.43639190e-03,-6.68103918e-03,-5.37711410e-03,-4.91201049e-03,
     & -4.39582996e-03,-3.85110759e-03,-3.27335460e-03,-2.68668446e-03,
     & -2.14696473e-03,-1.72437761e-03,-1.35598680e-03,-1.04510471e-03,
     & -7.78728323e-04,-5.86104634e-04,-4.15382293e-04,-2.97210854e-04,
     & -2.00873337e-04,-1.36950183e-04,-9.40333957e-05,-5.74765590e-05,
     & -3.14294076e-05,-1.54399049e-05,-6.45358275e-06,-2.06005782e-06,
     & -6.69737788e-07,-2.20809596e-07,-7.36127593e-08,-2.47632234e-08,
     & -4.89800939e-09,-9.81635331e-10,-1.98820044e-10,-4.06183901e-11,
     & -4.94151282e-12,-6.07419294e-13/

      data coef_h(1:51, 19) /
     &  8.60872637e-03, 8.60881218e-03,
     &  8.60887654e-03, 8.60899453e-03, 8.60920907e-03, 8.60958456e-03,
     &  8.61022836e-03, 8.61140900e-03, 8.61355681e-03, 8.61731911e-03,
     &  8.62377964e-03, 8.63565978e-03, 8.65738046e-03, 8.69577446e-03,
     &  8.76277550e-03, 8.88981793e-03, 9.07314195e-03, 9.60580772e-03,
     &  5.68967448e-04,-1.02680807e-02,-8.28490183e-03,-7.37632354e-03,
     & -6.43680808e-03,-5.45363991e-03,-4.46443150e-03,-3.56017279e-03,
     & -2.85517842e-03,-2.24242398e-03,-1.72651510e-03,-1.28528029e-03,
     & -9.66674652e-04,-6.84627687e-04,-4.89594200e-04,-3.30726916e-04,
     & -2.25387410e-04,-1.54703712e-04,-9.45251600e-05,-5.16691993e-05,
     & -2.53739882e-05,-1.06022757e-05,-3.38323341e-06,-1.09963342e-06,
     & -3.62473002e-07,-1.20820939e-07,-4.06387087e-08,-8.03682131e-09,
     & -1.61050138e-09,-3.26157577e-10,-6.66275604e-11,-8.10497456e-12,
     & -9.96203776e-13/

      data coef_h(1:51, 20) /
     &  1.04706488e-02, 1.04707426e-02, 1.04708129e-02,
     &  1.04709419e-02, 1.04711764e-02, 1.04715868e-02, 1.04722905e-02,
     &  1.04735808e-02, 1.04759277e-02, 1.04800379e-02, 1.04870926e-02,
     &  1.05000549e-02, 1.05237187e-02, 1.05654328e-02, 1.06378618e-02,
     &  1.07738115e-02, 1.09663337e-02, 1.12726017e-02, 1.26399636e-02,
     &  1.00257118e-03,-1.32448573e-02,-1.02003884e-02,-8.84301595e-03,
     & -7.45329555e-03,-6.07603131e-03,-4.82977802e-03,-3.86459789e-03,
     & -3.02952045e-03,-2.32888278e-03,-1.73132600e-03,-1.30078168e-03,
     & -9.20311543e-04,-6.57610360e-04,-4.43884672e-04,-3.02318135e-04,
     & -2.07403662e-04,-1.26655650e-04,-6.91945397e-05,-3.39630183e-05,
     & -1.41841204e-05,-4.52398956e-06,-1.46986400e-06,-4.84372815e-07,
     & -1.61415718e-07,-5.42826409e-08,-1.07326218e-08,-2.15032417e-09,
     & -4.35418135e-10,-8.89365256e-11,-1.08173327e-11,-1.32944308e-12/

      data coef_h(1:51, 21) /
     &  1.33110352e-02, 1.33111436e-02, 1.33112249e-02, 1.33113740e-02,
     &  1.33116450e-02, 1.33121192e-02, 1.33129323e-02, 1.33144233e-02,
     &  1.33171350e-02, 1.33218831e-02, 1.33300305e-02, 1.33449935e-02,
     &  1.33722851e-02, 1.34203163e-02, 1.35034721e-02, 1.36586923e-02,
     &  1.38764431e-02, 1.42174069e-02, 1.49161828e-02, 1.68840513e-02,
     &  1.68049676e-03,-1.75585958e-02,-1.31968210e-02,-1.10288702e-02,
     & -8.93226417e-03,-7.06568073e-03,-5.63477235e-03,-4.40515439e-03,
     & -3.37877289e-03,-2.50692076e-03,-1.88069912e-03,-1.32869378e-03,
     & -9.48354161e-04,-6.39452497e-04,-4.35142460e-04,-2.98317864e-04,
     & -1.82035567e-04,-9.93745215e-05,-4.87418148e-05,-2.03423910e-05,
     & -6.48375458e-06,-2.10552910e-06,-6.93572266e-07,-2.31057278e-07,
     & -7.76821776e-08,-1.53542835e-08,-3.07552806e-09,-6.22636771e-10,
     & -1.27155755e-10,-1.54631131e-11,-1.90012484e-12/

      data coef_h(1:51, 22) /
     &  1.62635669e-02,
     &  1.62636881e-02, 1.62637789e-02, 1.62639456e-02, 1.62642486e-02,
     &  1.62647788e-02, 1.62656879e-02, 1.62673547e-02, 1.62703861e-02,
     &  1.62756934e-02, 1.62847988e-02, 1.63015153e-02, 1.63319866e-02,
     &  1.63855544e-02, 1.64781138e-02, 1.66502535e-02, 1.68902874e-02,
     &  1.72624926e-02, 1.80097009e-02, 1.91301740e-02, 2.20507234e-02,
     &  1.60598467e-03,-2.29301685e-02,-1.64041170e-02,-1.31520191e-02,
     & -1.03269818e-02,-8.19508155e-03,-6.38158993e-03,-4.87910449e-03,
     & -3.61017455e-03,-2.70275023e-03,-1.90566618e-03,-1.35806528e-03,
     & -9.14371740e-04,-6.21497638e-04,-4.25669633e-04,-2.59477519e-04,
     & -1.41505510e-04,-6.93399826e-05,-2.89124421e-05,-9.20687979e-06,
     & -2.98779009e-06,-9.83669747e-07,-3.27560755e-07,-1.10088386e-07,
     & -2.17503608e-08,-4.35523904e-09,-8.81474870e-10,-1.79975890e-10,
     & -2.18811141e-11,-2.68824340e-12/

      data coef_h(1:51, 23) /
     &  1.90288838e-02, 1.90290141e-02,
     &  1.90291117e-02, 1.90292909e-02, 1.90296165e-02, 1.90301864e-02,
     &  1.90311635e-02, 1.90329549e-02, 1.90362129e-02, 1.90419163e-02,
     &  1.90517000e-02, 1.90696574e-02, 1.91023759e-02, 1.91598480e-02,
     &  1.92590133e-02, 1.94429574e-02, 1.96983785e-02, 2.00918628e-02,
     &  2.08715613e-02, 2.20119512e-02, 2.37371486e-02, 2.83114149e-02,
     &  2.46456465e-03,-2.93002107e-02,-1.96758212e-02,-1.52741478e-02,
     & -1.20287271e-02,-9.31144019e-03,-7.08564280e-03,-5.22193919e-03,
     & -3.89775767e-03,-2.74046750e-03,-1.94871813e-03,-1.30935747e-03,
     & -8.88519151e-04,-6.07746487e-04,-3.69934554e-04,-2.01457063e-04,
     & -9.85867634e-05,-4.10554077e-05,-1.30573022e-05,-4.23335105e-06,
     & -1.39273145e-06,-4.63507528e-07,-1.55703706e-07,-3.07449815e-08,
     & -6.15350944e-09,-1.24497731e-09,-2.54117234e-10,-3.08848439e-11,
     & -3.79339169e-12/

      data coef_h(1:51, 24) /
     &  2.18877223e-02, 2.18878607e-02, 2.18879644e-02,
     &  2.18881547e-02, 2.18885006e-02, 2.18891059e-02, 2.18901436e-02,
     &  2.18920464e-02, 2.18955065e-02, 2.19015636e-02, 2.19119529e-02,
     &  2.19310185e-02, 2.19657448e-02, 2.20267078e-02, 2.21317887e-02,
     &  2.23263409e-02, 2.25956922e-02, 2.30087684e-02, 2.38202866e-02,
     &  2.49891583e-02, 2.67122164e-02, 2.94459970e-02, 3.60952709e-02,
     &  2.92743075e-03,-3.73647840e-02,-2.38681375e-02,-1.85748279e-02,
     & -1.42447859e-02,-1.07616185e-02,-7.88376564e-03,-5.85888497e-03,
     & -4.10243568e-03,-2.90808105e-03,-1.94826632e-03,-1.31904346e-03,
     & -9.00545180e-04,-5.47062546e-04,-2.97330725e-04,-1.45238774e-04,
     & -6.03779764e-05,-1.91696902e-05,-6.20711878e-06,-2.04005630e-06,
     & -6.78401904e-07,-2.27744155e-07,-4.49348897e-08,-8.98804938e-09,
     & -1.81755780e-09,-3.70836723e-10,-4.50505020e-11,-5.53125955e-12/

      data coef_h(1:51, 25) /
     &  2.36015431e-02, 2.36016814e-02, 2.36017851e-02, 2.36019752e-02,
     &  2.36023209e-02, 2.36029258e-02, 2.36039630e-02, 2.36058645e-02,
     &  2.36093224e-02, 2.36153754e-02, 2.36257567e-02, 2.36448050e-02,
     &  2.36794912e-02, 2.37403567e-02, 2.38451879e-02, 2.40390035e-02,
     &  2.43067409e-02, 2.47159813e-02, 2.55150365e-02, 2.66539649e-02,
     &  2.83052241e-02, 3.08509749e-02, 3.47923899e-02, 4.46119530e-02,
     &  3.20503910e-03,-4.47187614e-02,-2.99247790e-02,-2.25420281e-02,
     & -1.68110438e-02,-1.21916112e-02,-8.99598614e-03,-6.25826949e-03,
     & -4.41481615e-03,-2.94457279e-03,-1.98669550e-03,-1.35260584e-03,
     & -8.19247514e-04,-4.43981117e-04,-2.16298007e-04,-8.96923905e-05,
     & -2.84065921e-05,-9.18119435e-06,-3.01326406e-06,-1.00090450e-06,
     & -3.35701359e-07,-6.61622460e-08,-1.32225530e-08,-2.67198497e-09,
     & -5.44850998e-10,-6.61487149e-11,-8.11753486e-12/

      data coef_h(1:51, 26) /
     &  2.10819534e-02,
     &  2.10820692e-02, 2.10821561e-02, 2.10823154e-02, 2.10826050e-02,
     &  2.10831118e-02, 2.10839807e-02, 2.10855737e-02, 2.10884705e-02,
     &  2.10935411e-02, 2.11022372e-02, 2.11181919e-02, 2.11472395e-02,
     &  2.11981948e-02, 2.12859096e-02, 2.14479200e-02, 2.16713789e-02,
     &  2.20121631e-02, 2.26748079e-02, 2.36128573e-02, 2.49588564e-02,
     &  2.69994181e-02, 3.00695593e-02, 3.50546660e-02, 4.82036877e-02,
     & -7.97111783e-04,-4.87438566e-02,-3.09328678e-02,-2.26544845e-02,
     & -1.62047269e-02,-1.18463299e-02,-8.17351975e-03,-5.73132332e-03,
     & -3.80196984e-03,-2.55452966e-03,-1.73345966e-03,-1.04625341e-03,
     & -5.65090551e-04,-2.74450106e-04,-1.13476006e-04,-3.58373195e-05,
     & -1.15586552e-05,-3.78744381e-06,-1.25645208e-06,-4.20972316e-07,
     & -8.28645553e-08,-1.65443178e-08,-3.34060400e-09,-6.80748360e-10,
     & -8.25891906e-11,-1.01292468e-11/

      data coef_h(1:51, 27) /
     &  1.80345087e-02, 1.80346025e-02,
     &  1.80346728e-02, 1.80348018e-02, 1.80350362e-02, 1.80354466e-02,
     &  1.80361500e-02, 1.80374397e-02, 1.80397849e-02, 1.80438900e-02,
     &  1.80509298e-02, 1.80638448e-02, 1.80873553e-02, 1.81285880e-02,
     &  1.81995382e-02, 1.83304906e-02, 1.85109119e-02, 1.87856137e-02,
     &  1.93181999e-02, 2.00685569e-02, 2.11377479e-02, 2.27412677e-02,
     &  2.51123942e-02, 2.88515368e-02, 3.50495187e-02, 5.16632021e-02,
     &  3.00500586e-03,-5.10502016e-02,-3.11834652e-02,-2.18472301e-02,
     & -1.57558003e-02,-1.07467788e-02,-7.47494351e-03,-4.92345064e-03,
     & -3.29037961e-03,-2.22343494e-03,-1.33610350e-03,-7.18622045e-04,
     & -3.47694325e-04,-1.43252461e-04,-4.50864884e-05,-1.45053799e-05,
     & -4.74387966e-06,-1.57134883e-06,-5.25827462e-07,-1.03351743e-07,
     & -2.06108848e-08,-4.15785767e-09,-8.46641867e-10,-1.02630428e-10,
     & -1.25787831e-11/

      data coef_h(1:51, 28) /
     &  1.64786075e-02, 1.64786895e-02, 1.64787510e-02,
     &  1.64788638e-02, 1.64790690e-02, 1.64794279e-02, 1.64800433e-02,
     &  1.64811715e-02, 1.64832231e-02, 1.64868141e-02, 1.64929723e-02,
     &  1.65042692e-02, 1.65248325e-02, 1.65608906e-02, 1.66229198e-02,
     &  1.67373508e-02, 1.68948904e-02, 1.71344875e-02, 1.75980955e-02,
     &  1.82492013e-02, 1.91727468e-02, 2.05483911e-02, 2.25612944e-02,
     &  2.56821454e-02, 3.07104584e-02, 3.88090757e-02, 5.60828824e-02,
     &  1.38016993e-03,-5.70339351e-02,-3.20806211e-02,-2.27479574e-02,
     & -1.52901387e-02,-1.05291477e-02,-6.87599889e-03,-4.56641458e-03,
     & -3.07072472e-03,-1.83601908e-03,-9.82842417e-04,-4.73524685e-04,
     & -1.94334826e-04,-6.09350428e-05,-1.95508802e-05,-6.38070901e-06,
     & -2.11006760e-06,-7.05164855e-07,-1.38381964e-07,-2.75628064e-08,
     & -5.55478372e-09,-1.13017307e-09,-1.36879731e-10,-1.67645870e-11/

      data coef_h(1:51, 29) /
     &  1.49382707e-02, 1.49383419e-02, 1.49383954e-02, 1.49384934e-02,
     &  1.49386716e-02, 1.49389835e-02, 1.49395182e-02, 1.49404985e-02,
     &  1.49422810e-02, 1.49454010e-02, 1.49507513e-02, 1.49605658e-02,
     &  1.49784293e-02, 1.50097490e-02, 1.50636142e-02, 1.51629417e-02,
     &  1.52995974e-02, 1.55072323e-02, 1.59083061e-02, 1.64700506e-02,
     &  1.72637549e-02, 1.84392393e-02, 2.01445276e-02, 2.27531418e-02,
     &  2.68624659e-02, 3.32277631e-02, 4.20505558e-02, 6.32993021e-02,
     &  4.77638112e-03,-6.19856622e-02,-3.71436880e-02,-2.42903955e-02,
     & -1.64402688e-02,-1.05882308e-02,-6.96335960e-03,-4.64839913e-03,
     & -2.75889880e-03,-1.46685643e-03,-7.02504829e-04,-2.86746026e-04,
     & -8.94499336e-05,-2.85938507e-05,-9.30594132e-06,-3.07068166e-06,
     & -1.02437882e-06,-2.00603757e-07,-3.98910777e-08,-8.02885662e-09,
     & -1.63180516e-09,-1.97406377e-10,-2.41552084e-11/

      data coef_h(1:51, 30) /
     &  1.19676523e-02,
     &  1.19677075e-02, 1.19677488e-02, 1.19678247e-02, 1.19679626e-02,
     &  1.19682039e-02, 1.19686175e-02, 1.19693759e-02, 1.19707550e-02,
     &  1.19731688e-02, 1.19773080e-02, 1.19849008e-02, 1.19987195e-02,
     &  1.20229452e-02, 1.20646023e-02, 1.21413936e-02, 1.22469917e-02,
     &  1.24073222e-02, 1.27166280e-02, 1.31489715e-02, 1.37581127e-02,
     &  1.46565416e-02, 1.59520156e-02, 1.79155963e-02, 2.09638003e-02,
     &  2.55738576e-02, 3.17370854e-02, 4.13539855e-02, 6.47330583e-02,
     & -3.13002439e-03,-6.64300268e-02,-3.38972397e-02,-2.25383908e-02,
     & -1.42927434e-02,-9.29930308e-03,-6.15889710e-03,-3.62688409e-03,
     & -1.91468013e-03,-9.11336795e-04,-3.69930636e-04,-1.14801546e-04,
     & -3.65621381e-05,-1.18661438e-05,-3.90696388e-06,-1.30108568e-06,
     & -2.54265702e-07,-5.04812466e-08,-1.01473767e-08,-2.06022904e-09,
     & -2.48953995e-10,-3.04351150e-11/

      data coef_h(1:51, 31) /
     &  9.90213654e-03, 9.90218064e-03,
     &  9.90221371e-03, 9.90227434e-03, 9.90238459e-03, 9.90257751e-03,
     &  9.90290825e-03, 9.90351464e-03, 9.90461728e-03, 9.90654721e-03,
     &  9.90985662e-03, 9.91592702e-03, 9.92697457e-03, 9.94634027e-03,
     &  9.97963508e-03, 1.00409936e-02, 1.01253323e-02, 1.02533023e-02,
     &  1.04998993e-02, 1.08439750e-02, 1.13275447e-02, 1.20382071e-02,
     &  1.30576130e-02, 1.45908932e-02, 1.69430759e-02, 2.04359677e-02,
     &  2.49835860e-02, 3.18044908e-02, 4.24883980e-02, 7.17810122e-02,
     &  8.67332704e-03,-6.89895852e-02,-3.75877207e-02,-2.31267135e-02,
     & -1.47569239e-02,-9.64269831e-03,-5.60654549e-03,-2.92702753e-03,
     & -1.38022587e-03,-5.55694562e-04,-1.71160116e-04,-5.42248495e-05,
     & -1.75297441e-05,-5.75424343e-06,-1.91162598e-06,-3.72520712e-07,
     & -7.37975875e-08,-1.48084838e-08,-3.00232238e-09,-3.62241875e-10,
     & -4.42306309e-11/

      data coef_h(1:51, 32) /
     &  8.17766125e-03, 8.17769672e-03, 8.17772332e-03,
     &  8.17777209e-03, 8.17786076e-03, 8.17801593e-03, 8.17828196e-03,
     &  8.17876969e-03, 8.17965656e-03, 8.18120883e-03, 8.18387062e-03,
     &  8.18875298e-03, 8.19763809e-03, 8.21321213e-03, 8.23998502e-03,
     &  8.28931430e-03, 8.35709735e-03, 8.45989986e-03, 8.65784069e-03,
     &  8.93367940e-03, 9.32067034e-03, 9.88797957e-03, 1.06988496e-02,
     &  1.19121548e-02, 1.37590627e-02, 1.64702835e-02, 1.99453256e-02,
     &  2.50435890e-02, 3.27774721e-02, 4.59347375e-02, 7.42013248e-02,
     & -3.84440691e-03,-7.61316789e-02,-3.61005009e-02,-2.25876634e-02,
     & -1.45415779e-02,-8.33864216e-03,-4.30235527e-03,-2.00920297e-03,
     & -8.02224334e-04,-2.45245798e-04,-7.72927354e-05,-2.48915407e-05,
     & -8.14674445e-06,-2.70011013e-06,-5.24735849e-07,-1.03734098e-07,
     & -2.07811051e-08,-4.20755327e-09,-5.06922737e-10,-6.18247845e-11/

      data coef_h(1:51, 33) /
     &  6.11252521e-03, 6.11255098e-03, 6.11257031e-03, 6.11260575e-03,
     &  6.11267017e-03, 6.11278293e-03, 6.11297622e-03, 6.11333061e-03,
     &  6.11397502e-03, 6.11510290e-03, 6.11703692e-03, 6.12058433e-03,
     &  6.12703979e-03, 6.13835434e-03, 6.15780263e-03, 6.19362889e-03,
     &  6.24284199e-03, 6.31744633e-03, 6.46097742e-03, 6.66074206e-03,
     &  6.94051524e-03, 7.34963242e-03, 7.93233789e-03, 8.79983611e-03,
     &  1.01105399e-02, 1.20139653e-02, 1.44190263e-02, 1.78794221e-02,
     &  2.29776191e-02, 3.12457639e-02, 4.32092822e-02, 8.03682628e-02,
     &  7.96504037e-03,-7.77435659e-02,-3.82082769e-02,-2.39372959e-02,
     & -1.33928966e-02,-6.77602270e-03,-3.11640595e-03,-1.22867876e-03,
     & -3.71486061e-04,-1.16205869e-04,-3.72204288e-05,-1.21315871e-05,
     & -4.00773967e-06,-7.75921754e-07,-1.52950062e-07,-3.05712348e-08,
     & -6.17843471e-09,-7.42915499e-10,-9.04651545e-11/

      data coef_h(1:51, 34) /
     &  4.84035718e-03,
     &  4.84037714e-03, 4.84039212e-03, 4.84041957e-03, 4.84046948e-03,
     &  4.84055683e-03, 4.84070657e-03, 4.84098112e-03, 4.84148033e-03,
     &  4.84235409e-03, 4.84385235e-03, 4.84660042e-03, 4.85160115e-03,
     &  4.86036555e-03, 4.87542922e-03, 4.90317432e-03, 4.94127798e-03,
     &  4.99902191e-03, 5.11005101e-03, 5.26444026e-03, 5.48039520e-03,
     &  5.79563422e-03, 6.24351713e-03, 6.90795579e-03, 7.90675700e-03,
     &  9.34685632e-03, 1.11498855e-02, 1.37134602e-02, 1.74284831e-02,
     &  2.33075143e-02, 3.15464164e-02, 4.71303961e-02, 8.31860584e-02,
     & -1.39809080e-03,-8.27387920e-02,-4.26914313e-02,-2.30114305e-02,
     & -1.13291684e-02,-5.10872289e-03,-1.98326793e-03,-5.91887189e-04,
     & -1.83572375e-04,-5.84402378e-05,-1.89608675e-05,-6.24146835e-06,
     & -1.20342928e-06,-2.36485698e-07,-4.71534551e-08,-9.51109020e-09,
     & -1.14126675e-09,-1.38742936e-10/

      data coef_h(1:51, 35) /
     &  3.06078216e-03, 3.06079452e-03,
     &  3.06080379e-03, 3.06082079e-03, 3.06085169e-03, 3.06090577e-03,
     &  3.06099849e-03, 3.06116847e-03, 3.06147755e-03, 3.06201853e-03,
     &  3.06294614e-03, 3.06464754e-03, 3.06774352e-03, 3.07316939e-03,
     &  3.08249429e-03, 3.09966707e-03, 3.12324631e-03, 3.15896846e-03,
     &  3.22761787e-03, 3.32299780e-03, 3.45626012e-03, 3.65047653e-03,
     &  3.92579238e-03, 4.33292826e-03, 4.94215751e-03, 5.81499105e-03,
     &  6.89908541e-03, 8.42487188e-03, 1.06056083e-02, 1.39880458e-02,
     &  1.85952310e-02, 2.69222098e-02, 3.97570231e-02, 8.17551485e-02,
     & -2.09131453e-03,-8.11066235e-02,-3.25775813e-02,-1.56223141e-02,
     & -6.89366661e-03,-2.63175306e-03,-7.74659828e-04,-2.38122772e-04,
     & -7.53309275e-05,-2.43267140e-05,-7.97870666e-06,-1.53201107e-06,
     & -3.00117305e-07,-5.96957156e-08,-1.20174531e-08,-1.43902739e-09,
     & -1.74654400e-10/

      data coef_h(1:51, 36) /
     &  2.40512571e-03, 2.40513525e-03, 2.40514241e-03,
     &  2.40515553e-03, 2.40517939e-03, 2.40522114e-03, 2.40529272e-03,
     &  2.40542396e-03, 2.40566258e-03, 2.40608023e-03, 2.40679639e-03,
     &  2.40810991e-03, 2.41050005e-03, 2.41468874e-03, 2.42188700e-03,
     &  2.43514190e-03, 2.45333866e-03, 2.48089982e-03, 2.53384359e-03,
     &  2.60735450e-03, 2.70997012e-03, 2.85933438e-03, 3.07069863e-03,
     &  3.38249757e-03, 3.84744668e-03, 4.51040248e-03, 5.32899596e-03,
     &  6.47276667e-03, 8.09200038e-03, 1.05707708e-02, 1.38891885e-02,
     &  1.97365176e-02, 2.84560234e-02, 4.53047976e-02, 8.93548007e-02,
     &  1.14633644e-02,-8.50321604e-02,-2.84309792e-02,-1.22811274e-02,
     & -4.58861788e-03,-1.32715400e-03,-4.03446003e-04,-1.26653331e-04,
     & -4.06694747e-05,-1.32809492e-05,-2.53758515e-06,-4.95287716e-07,
     & -9.82372640e-08,-1.97315428e-08,-2.35709284e-09,-2.85539273e-10/

      data coef_h(1:51, 37) /
     &  2.01712685e-03, 2.01713472e-03, 2.01714062e-03, 2.01715144e-03,
     &  2.01717110e-03, 2.01720552e-03, 2.01726451e-03, 2.01737268e-03,
     &  2.01756936e-03, 2.01791361e-03, 2.01850388e-03, 2.01958652e-03,
     &  2.02155648e-03, 2.02500871e-03, 2.03094102e-03, 2.04186369e-03,
     &  2.05685634e-03, 2.07955939e-03, 2.12315372e-03, 2.18364614e-03,
     &  2.26801810e-03, 2.39068326e-03, 2.56398272e-03, 2.81904874e-03,
     &  3.19818241e-03, 3.73643021e-03, 4.39752935e-03, 5.31530603e-03,
     &  6.60386497e-03, 8.55456517e-03, 1.11290500e-02, 1.55742879e-02,
     &  2.20122110e-02, 3.39250844e-02, 5.36878777e-02, 9.95979074e-02,
     &  7.38783044e-03,-9.50072179e-02,-2.79356771e-02,-1.01851683e-02,
     & -2.87000689e-03,-8.58470434e-04,-2.66568783e-04,-8.49241131e-05,
     & -2.75671564e-05,-5.23211420e-06,-1.01618882e-06,-2.00793527e-07,
     & -4.02097603e-08,-4.78825897e-09,-5.78618369e-10/

      data coef_h(1:51, 38) /
     &  1.33288084e-03,
     &  1.33288594e-03, 1.33288976e-03, 1.33289677e-03, 1.33290952e-03,
     &  1.33293182e-03, 1.33297006e-03, 1.33304017e-03, 1.33316764e-03,
     &  1.33339075e-03, 1.33377332e-03, 1.33447498e-03, 1.33575171e-03,
     &  1.33798900e-03, 1.34183333e-03, 1.34891082e-03, 1.35862388e-03,
     &  1.37332860e-03, 1.40155268e-03, 1.44069144e-03, 1.49523153e-03,
     &  1.57442635e-03, 1.68611825e-03, 1.85011482e-03, 2.09306390e-03,
     &  2.43641543e-03, 2.85584007e-03, 3.43430215e-03, 4.23973960e-03,
     &  5.44579212e-03, 7.01592436e-03, 9.67657467e-03, 1.34312915e-02,
     &  2.01127759e-02, 3.05514315e-02, 4.72151598e-02, 1.06374254e-01,
     &  6.17009581e-03,-9.96689690e-02,-2.29826081e-02,-6.35395069e-03,
     & -1.86016685e-03,-5.69117121e-04,-1.79391630e-04,-5.77685069e-05,
     & -1.08675594e-05,-2.09713305e-06,-4.12348604e-07,-8.22550044e-08,
     & -9.75547377e-09,-1.17513816e-09/

      data coef_h(1:51, 39) /
     &  7.83530263e-04, 7.83533205e-04,
     &  7.83535411e-04, 7.83539456e-04, 7.83546811e-04, 7.83559683e-04,
     &  7.83581748e-04, 7.83622204e-04, 7.83695764e-04, 7.83824512e-04,
     &  7.84045274e-04, 7.84450168e-04, 7.85186888e-04, 7.86477851e-04,
     &  7.88695987e-04, 7.92779208e-04, 7.98382125e-04, 8.06862583e-04,
     &  8.23133638e-04, 8.45683544e-04, 8.77081467e-04, 9.22621229e-04,
     &  9.86747741e-04, 1.08070159e-03, 1.21947081e-03, 1.41480537e-03,
     &  1.65228363e-03, 1.97796366e-03, 2.42825567e-03, 3.09645031e-03,
     &  3.95688231e-03, 5.39401252e-03, 7.38417265e-03, 1.08343660e-02,
     &  1.60328684e-02, 2.39734579e-02, 4.17194172e-02, 1.11167234e-01,
     &  8.12147389e-03,-1.03787203e-01,-1.21664585e-02,-3.86675215e-03,
     & -1.18915115e-03,-3.73896986e-04,-1.19964593e-04,-2.24513332e-05,
     & -4.31383088e-06,-8.45232000e-07,-1.68121328e-07,-1.98775513e-08,
     & -2.38852128e-09/

      data coef_h(1:51, 40) /
     &  4.31428330e-04, 4.31429921e-04, 4.31431114e-04,
     &  4.31433301e-04, 4.31437277e-04, 4.31444237e-04, 4.31456167e-04,
     &  4.31478041e-04, 4.31517814e-04, 4.31587425e-04, 4.31706786e-04,
     &  4.31925702e-04, 4.32324019e-04, 4.33021975e-04, 4.34221142e-04,
     &  4.36428406e-04, 4.39456738e-04, 4.44039416e-04, 4.52828828e-04,
     &  4.65003230e-04, 4.81941729e-04, 5.06483608e-04, 5.40992089e-04,
     &  5.91451062e-04, 6.65773945e-04, 7.70012232e-04, 8.96197259e-04,
     &  1.06838152e-03, 1.30498580e-03, 1.65337178e-03, 2.09788586e-03,
     &  2.83167264e-03, 3.83295521e-03, 5.53558430e-03, 8.03725864e-03,
     &  1.17425654e-02, 1.96937155e-02, 3.83110605e-02, 1.16965943e-01,
     &  1.70596981e-02,-1.04676812e-01,-1.49791483e-02,-4.61490244e-03,
     & -1.41336984e-03,-4.44019379e-04,-8.12146998e-05,-1.53549240e-05,
     & -2.97292628e-06,-5.85942629e-07,-6.86335825e-08,-8.18846903e-09/

      data coef_h(1:51, 41) /
     &  1.76504596e-04, 1.76505236e-04, 1.76505716e-04, 1.76506595e-04,
     &  1.76508194e-04, 1.76510993e-04, 1.76515791e-04, 1.76524587e-04,
     &  1.76540581e-04, 1.76568575e-04, 1.76616575e-04, 1.76704609e-04,
     &  1.76864784e-04, 1.77145446e-04, 1.77627631e-04, 1.78515097e-04,
     &  1.79732531e-04, 1.81574490e-04, 1.85106158e-04, 1.89995492e-04,
     &  1.96793481e-04, 2.06633635e-04, 2.20451953e-04, 2.40621411e-04,
     &  2.70257059e-04, 3.11687506e-04, 3.61652140e-04, 4.29533409e-04,
     &  5.22318426e-04, 6.58041955e-04, 8.29891879e-04, 1.11086753e-03,
     &  1.48978818e-03, 2.12459025e-03, 3.04004297e-03, 4.36637098e-03,
     &  7.12331778e-03, 1.32969150e-02, 2.85096991e-02, 1.11666537e-01,
     & -4.21396883e-03,-1.10284816e-01,-1.64598789e-02,-5.04576505e-03,
     & -1.54500498e-03,-2.74412953e-04,-5.08407627e-05,-9.70061455e-06,
     & -1.89099014e-06,-2.19067018e-07,-2.59202504e-08/

      data coef_h(1:51, 42) /
     &  4.75737999e-05,
     &  4.75739692e-05, 4.75740961e-05, 4.75743288e-05, 4.75747518e-05,
     &  4.75754922e-05, 4.75767614e-05, 4.75790884e-05, 4.75833196e-05,
     &  4.75907252e-05, 4.76034231e-05, 4.76267115e-05, 4.76690835e-05,
     &  4.77433262e-05, 4.78708713e-05, 4.81055985e-05, 4.84275557e-05,
     &  4.89145775e-05, 4.98480482e-05, 5.11396923e-05, 5.29342760e-05,
     &  5.55293882e-05, 5.91687194e-05, 6.44709356e-05, 7.22419364e-05,
     &  8.30697594e-05, 9.60776780e-05, 1.13671693e-04, 1.37592294e-04,
     &  1.72353366e-04, 2.16035135e-04, 2.86792225e-04, 3.81147735e-04,
     &  5.37033212e-04, 7.58041404e-04, 1.07204934e-03, 1.70711650e-03,
     &  3.06896932e-03, 6.23597647e-03, 1.46455423e-02, 1.10928995e-01,
     & 3.32637354e-04,-1.09275276e-01,-1.61840066e-02,-4.96436691e-03,
     & -8.49428119e-04,-1.53251063e-04,-2.87008566e-05,-5.51873468e-06,
     & -6.30811811e-07,-7.39041184e-08/

      data coef_h(1:51, 43) /
     &  1.65963707e-05, 1.65964292e-05,
     &  1.65964730e-05, 1.65965535e-05, 1.65966997e-05, 1.65969557e-05,
     &  1.65973944e-05, 1.65981988e-05, 1.65996615e-05, 1.66022215e-05,
     &  1.66066110e-05, 1.66146615e-05, 1.66293089e-05, 1.66549730e-05,
     &  1.66990618e-05, 1.67801970e-05, 1.68914772e-05, 1.70597947e-05,
     &  1.73823576e-05, 1.78285817e-05, 1.84483554e-05, 1.93441974e-05,
     &  2.05997413e-05, 2.24274540e-05, 2.51031596e-05, 2.88259537e-05,
     &  3.32908189e-05, 3.93183762e-05, 4.74950377e-05, 5.93453073e-05,
     &  7.41919140e-05, 9.81549891e-05, 1.29978323e-04, 1.82300609e-04,
     &  2.56075790e-04, 3.60299807e-04, 5.69642755e-04, 1.01491888e-03,
     &  2.04637231e-03, 4.98400307e-03, 1.62482170e-02, 1.09495330e-01,
     &  4.87780468e-04,-1.07070311e-01,-1.85042259e-02,-2.99110492e-03,
     & -5.16537201e-04,-9.40606513e-05,-1.77389579e-05,-1.99116462e-06,
     & -2.30294703e-07/

      data coef_h(1:51, 44) /
     &  5.41117987e-06, 5.41119878e-06, 5.41121296e-06,
     &  5.41123897e-06, 5.41128625e-06, 5.41136899e-06, 5.41151083e-06,
     &  5.41177089e-06, 5.41224376e-06, 5.41307138e-06, 5.41449045e-06,
     &  5.41709305e-06, 5.42182829e-06, 5.43012499e-06, 5.44437766e-06,
     &  5.47060549e-06, 5.50657593e-06, 5.56097877e-06, 5.66522125e-06,
     &  5.80939544e-06, 6.00958333e-06, 6.29882216e-06, 6.70396889e-06,
     &  7.29329526e-06, 8.15515016e-06, 9.35265992e-06, 1.07866473e-05,
     &  1.27191268e-05, 1.53351846e-05, 1.91171055e-05, 2.38419814e-05,
     &  3.14425618e-05, 4.14968487e-05, 5.79511291e-05, 8.10274507e-05,
     &  1.13439313e-04, 1.78053573e-04, 3.14022833e-04, 6.24374131e-04,
     &  1.49124696e-03, 4.87119183e-03, 1.58675957e-02, 1.08112412e-01,
     & -5.64721193e-03,-1.07343941e-01,-1.38176932e-03,-6.67356397e-04,
     & -1.37885383e-04,-2.72730427e-05,-3.15302305e-06,-3.70214196e-07/

      data coef_h(1:51, 45) /
     &  2.08043565e-06, 2.08044287e-06, 2.08044828e-06, 2.08045820e-06,
     &  2.08047624e-06, 2.08050781e-06, 2.08056193e-06, 2.08066115e-06,
     &  2.08084156e-06, 2.08115733e-06, 2.08169875e-06, 2.08269172e-06,
     &  2.08449834e-06, 2.08766372e-06, 2.09310134e-06, 2.10310735e-06,
     &  2.11682952e-06, 2.13758185e-06, 2.17734083e-06, 2.23231940e-06,
     &  2.30863775e-06, 2.41886513e-06, 2.57318772e-06, 2.79751407e-06,
     &  3.12527806e-06, 3.58015346e-06, 4.12411732e-06, 4.85605468e-06,
     &  5.84511401e-06, 7.27185995e-06, 9.05002713e-06, 1.19022183e-05,
     &  1.56626318e-05, 2.17926082e-05, 3.03508050e-05, 4.23131053e-05,
     &  6.60120571e-05, 1.15442162e-04, 2.26886841e-04, 5.32684722e-04,
     &  1.69371530e-03, 5.52770775e-03, 1.80892239e-02, 1.16234750e-01,
     &  2.76168404e-02,-9.53568850e-02,-2.93943754e-03,-8.38438697e-04,
     & -1.65138689e-04,-1.86290604e-05,-2.14192535e-06/

      data coef_h(1:51, 46) /
     &  7.11612643e-07,
     &  7.11615095e-07, 7.11616934e-07, 7.11620305e-07, 7.11626435e-07,
     &  7.11637161e-07, 7.11655550e-07, 7.11689265e-07, 7.11750569e-07,
     &  7.11857863e-07, 7.12041833e-07, 7.12379237e-07, 7.12993109e-07,
     &  7.14068666e-07, 7.15916272e-07, 7.19316041e-07, 7.23978257e-07,
     &  7.31028596e-07, 7.44534757e-07, 7.63207949e-07, 7.89123296e-07,
     &  8.26541566e-07, 8.78906601e-07, 9.54982163e-07, 1.06605075e-06,
     &  1.22003998e-06, 1.40397899e-06, 1.65116303e-06, 1.98467729e-06,
     &  2.46491486e-06, 3.06223933e-06, 4.01807902e-06, 5.27484361e-06,
     &  7.31698881e-06, 1.01576386e-05, 1.41127191e-05, 2.19093204e-05,
     &  3.80573906e-05, 7.41150532e-05, 1.71687225e-04, 5.33923293e-04,
     &  1.69603350e-03, 5.53819650e-03, 1.83971155e-02, 1.02940053e-01,
     & -6.16317670e-03,-1.00741788e-01,-3.73960779e-03,-9.74975681e-04,
     & -1.09756316e-04,-1.23812238e-05/

      data coef_h(1:51, 47) /
     &  8.08788948e-08, 8.08791690e-08,
     &  8.08793747e-08, 8.08797519e-08, 8.08804375e-08, 8.08816375e-08,
     &  8.08836946e-08, 8.08874661e-08, 8.08943238e-08, 8.09063263e-08,
     &  8.09269060e-08, 8.09646492e-08, 8.10333184e-08, 8.11536300e-08,
     &  8.13602954e-08, 8.17405538e-08, 8.22619620e-08, 8.30503349e-08,
     &  8.45602162e-08, 8.66469097e-08, 8.95413560e-08, 9.37174735e-08,
     &  9.95558916e-08, 1.08026390e-07, 1.20370367e-07, 1.37443819e-07,
     &  1.57782504e-07, 1.85030552e-07, 2.21662610e-07, 2.74182728e-07,
     &  3.39193538e-07, 4.42631225e-07, 5.77739664e-07, 7.95587506e-07,
     &  1.09593305e-06, 1.51015281e-06, 2.31679844e-06, 3.95882473e-06,
     &  7.53797472e-06, 1.68839094e-05, 4.95306923e-05, 1.44789965e-04,
     &  4.15382842e-04, 1.09942732e-03, 1.31208811e-03, 1.02365061e-01,
     &  4.70799162e-04,-9.82089711e-02,-5.38213464e-03,-6.63214122e-04,
     & -7.19592889e-05/

      data coef_h(1:51, 48) /
     &  1.95130503e-08, 1.95131164e-08, 1.95131659e-08,
     &  1.95132568e-08, 1.95134219e-08, 1.95137110e-08, 1.95142064e-08,
     &  1.95151148e-08, 1.95167666e-08, 1.95196575e-08, 1.95246144e-08,
     &  1.95337053e-08, 1.95502450e-08, 1.95792233e-08, 1.96290007e-08,
     &  1.97205891e-08, 1.98461735e-08, 2.00360556e-08, 2.03997073e-08,
     &  2.09022661e-08, 2.15993323e-08, 2.26049996e-08, 2.40108548e-08,
     &  2.60502724e-08, 2.90218592e-08, 3.31312256e-08, 3.80255045e-08,
     &  4.45810213e-08, 5.33920847e-08, 6.60213059e-08, 8.16498384e-08,
     &  1.06509044e-07, 1.38970511e-07, 1.91297283e-07, 2.63424155e-07,
     &  3.62884506e-07, 5.56570399e-07, 9.50953805e-07, 1.81143275e-06,
     &  4.06430324e-06, 1.19959897e-05, 3.56109619e-05, 1.06274847e-04,
     &  3.17923256e-04, 9.41771985e-04, 3.54841289e-03, 9.95277362e-02,
     & -4.11990696e-03,-9.78057270e-02, 3.72452157e-03, 5.71822715e-05/

      data coef_h(1:51, 49) /
     &  4.33638262e-09, 4.33639724e-09, 4.33640820e-09, 4.33642830e-09,
     &  4.33646485e-09, 4.33652880e-09, 4.33663844e-09, 4.33683946e-09,
     &  4.33720496e-09, 4.33784467e-09, 4.33894153e-09, 4.34095316e-09,
     &  4.34461307e-09, 4.35102535e-09, 4.36203991e-09, 4.38230594e-09,
     &  4.41009364e-09, 4.45210675e-09, 4.53256276e-09, 4.64374017e-09,
     &  4.79792631e-09, 5.02033171e-09, 5.33116143e-09, 5.78191679e-09,
     &  6.43840166e-09, 7.34571416e-09, 8.42561144e-09, 9.87098040e-09,
     &  1.18119837e-08, 1.45912712e-08, 1.80267803e-08, 2.34843331e-08,
     &  3.06004650e-08, 4.20522909e-08, 5.78080613e-08, 7.94929896e-08,
     &  1.21621788e-07, 2.07132769e-07, 3.92943235e-07, 8.76821801e-07,
     &  2.56764760e-06, 7.55935688e-06, 2.23853323e-05, 6.66856982e-05,
     &  1.99641258e-04, 1.02807083e-03, 4.04552113e-03, 1.02726581e-01,
     &  1.79886089e-02,-8.60501553e-02, 2.60616271e-03/

      data coef_h(1:51, 50) /
     &  8.87765567e-10,
     &  8.87768549e-10, 8.87770786e-10, 8.87774887e-10, 8.87782344e-10,
     &  8.87795393e-10, 8.87817763e-10, 8.87858776e-10, 8.87933351e-10,
     &  8.88063871e-10, 8.88287666e-10, 8.88698102e-10, 8.89444837e-10,
     &  8.90753137e-10, 8.93000426e-10, 8.97135224e-10, 9.02804526e-10,
     &  9.11375875e-10, 9.27789361e-10, 9.50468423e-10, 9.81917510e-10,
     &  1.02727458e-09, 1.09065231e-09, 1.18253624e-09, 1.31630906e-09,
     &  1.50110859e-09, 1.72094560e-09, 2.01501193e-09, 2.40965166e-09,
     &  2.97428145e-09, 3.67162075e-09, 4.77827845e-09, 6.21961604e-09,
     &  8.53612265e-09, 1.17186469e-08, 1.60922937e-08, 2.45737478e-08,
     &  4.17468094e-08, 7.89451660e-08, 1.75412258e-07, 5.10501814e-07,
     &  1.49282957e-06, 4.38903313e-06, 1.29827929e-05, 3.86624380e-05,
     &  2.01282979e-04, 1.05423514e-03, 4.64098366e-03, 9.04415425e-02,
     & -1.89214643e-03,-1.71716873e-01/

      data coef_h(1:51, 51) /
     &  3.90236429e-11, 3.90237704e-11,
     &  3.90238661e-11, 3.90240415e-11, 3.90243603e-11, 3.90249183e-11,
     &  3.90258750e-11, 3.90276288e-11, 3.90308179e-11, 3.90363993e-11,
     &  3.90459694e-11, 3.90635206e-11, 3.90954520e-11, 3.91513949e-11,
     &  3.92474829e-11, 3.94242567e-11, 3.96665942e-11, 4.00328934e-11,
     &  4.07340360e-11, 4.17022052e-11, 4.30435973e-11, 4.49758834e-11,
     &  4.76714763e-11, 5.15708862e-11, 5.72310912e-11, 6.50204295e-11,
     &  7.42462624e-11, 8.65270039e-11, 1.02913995e-10, 1.26201174e-10,
     &  1.54746928e-10, 1.99652158e-10, 2.57554840e-10, 3.49544435e-10,
     &  4.74280527e-10, 6.43363594e-10, 9.65652637e-10, 1.60290389e-09,
     &  2.94005367e-09, 6.25723494e-09, 1.70097397e-08, 4.56909069e-08,
     &  1.20421289e-07, 3.07324137e-07, 7.38632443e-07, 2.00511821e-06,
     & -4.66809104e-06,-1.85535655e-04,-4.04881136e-03, 8.50154847e-02,
     &  1.69140724e-01/

      data coef_j(1:51, 1) /
     & 9.79744820e-06, 8.96296860e-06, 8.35934549e-06, 7.89335126e-06,
     & 7.44161783e-06, 7.01338932e-06, 6.60406268e-06, 6.18398056e-06,
     & 5.76053476e-06, 5.34734829e-06, 4.94621727e-06, 4.53113519e-06,
     & 4.11083148e-06, 3.69997631e-06, 3.30117385e-06, 2.88937415e-06,
     & 2.55730770e-06, 2.24433006e-06, 1.88506336e-06, 1.58859300e-06,
     & 1.33086897e-06, 1.09505641e-06, 8.86720269e-07, 6.98736456e-07,
     & 5.33933743e-07, 4.00542516e-07, 3.06250782e-07, 2.30467716e-07,
     & 1.70827079e-07, 1.22755304e-07, 8.97411915e-08, 6.17573521e-08,
     & 4.31436590e-08, 2.84810947e-08, 1.90445828e-08, 1.28647073e-08,
     & 7.72163607e-09, 4.14498085e-09, 2.00041016e-09, 8.21655030e-10,
     & 2.57654521e-10, 8.26321572e-11, 2.69513765e-11, 8.90661828e-12,
     & 2.97449037e-12, 5.83155356e-13, 1.16051514e-13, 2.33699420e-14,
     & 4.75155459e-15, 5.75016867e-16, 7.03774182e-17/

      data coef_j(1:51, 2) /
     & 1.84495605e-05,
     & 2.04340585e-05, 1.83893753e-05, 1.67455934e-05, 1.56681842e-05,
     & 1.47030431e-05, 1.38123877e-05, 1.29157909e-05, 1.20215185e-05,
     & 1.11539807e-05, 1.03144214e-05, 9.44721432e-06, 8.57000031e-06,
     & 7.71299640e-06, 6.88139393e-06, 6.02284144e-06, 5.33058911e-06,
     & 4.67816358e-06, 3.92926961e-06, 3.31128764e-06, 2.77407705e-06,
     & 2.28254302e-06, 1.84828352e-06, 1.45644794e-06, 1.11293201e-06,
     & 8.34890799e-07, 6.38348903e-07, 4.80386613e-07, 3.56071674e-07,
     & 2.55870903e-07, 1.87056331e-07, 1.28726866e-07, 8.99285240e-08,
     & 5.93659104e-08, 3.96964700e-08, 2.68151550e-08, 1.60949529e-08,
     & 8.63978399e-09, 4.16964794e-09, 1.71265479e-09, 5.37054134e-10,
     & 1.72238158e-10, 5.61773464e-11, 1.85649208e-11, 6.20001617e-12,
     & 1.21552673e-12, 2.41897316e-13, 4.87122141e-14, 9.90412136e-15,
     & 1.19856284e-15, 1.46694406e-16/

      data coef_j(1:51, 3) /
     & 1.73787847e-05, 1.87918674e-05,
     & 2.08999830e-05, 1.83387653e-05, 1.62807705e-05, 1.51671131e-05,
     & 1.41811027e-05, 1.32240507e-05, 1.22888417e-05, 1.13916825e-05,
     & 1.05286937e-05, 9.64030881e-06, 8.74342375e-06, 7.86815095e-06,
     & 7.01932516e-06, 6.14328580e-06, 5.43706123e-06, 4.77153058e-06,
     & 4.00764123e-06, 3.37730951e-06, 2.82937491e-06, 2.32803523e-06,
     & 1.88511642e-06, 1.48546973e-06, 1.13510730e-06, 8.51525287e-07,
     & 6.51067057e-07, 4.89957364e-07, 3.63165556e-07, 2.60968439e-07,
     & 1.90782877e-07, 1.31291340e-07, 9.17200485e-08, 6.05485677e-08,
     & 4.04872778e-08, 2.73493468e-08, 1.64155829e-08, 8.81189759e-09,
     & 4.25271133e-09, 1.74677236e-09, 5.47752673e-10, 1.75669266e-10,
     & 5.72964369e-11, 1.89347458e-11, 6.32352422e-12, 1.23974071e-12,
     & 2.46716040e-13, 4.96825863e-14, 1.01014163e-14, 1.22243878e-15,
     & 1.49616625e-16/

      data coef_j(1:51, 4) /
     & 3.25551939e-05, 3.36923300e-05, 3.51339037e-05,
     & 3.88807453e-05, 3.39978572e-05, 3.01736404e-05, 2.80557516e-05,
     & 2.60587444e-05, 2.41623827e-05, 2.23709199e-05, 2.06616637e-05,
     & 1.89100581e-05, 1.71462550e-05, 1.54274129e-05, 1.37618090e-05,
     & 1.20435652e-05, 1.06587206e-05, 9.35383616e-06, 7.85622517e-06,
     & 6.62051755e-06, 5.54637186e-06, 4.56358554e-06, 3.69533249e-06,
     & 2.91191152e-06, 2.22510520e-06, 1.66920873e-06, 1.27625797e-06,
     & 9.60441144e-07, 7.11896566e-07, 5.11564087e-07, 3.73982533e-07,
     & 2.57364048e-07, 1.79794324e-07, 1.18690371e-07, 7.93652009e-08,
     & 5.36115600e-08, 3.21786438e-08, 1.72735185e-08, 8.33637456e-09,
     & 3.42410898e-09, 1.07373157e-09, 3.44355456e-10, 1.12315261e-10,
     & 3.71168073e-11, 1.23956785e-11, 2.43019968e-12, 4.83624689e-13,
     & 9.73902009e-14, 1.98012828e-14, 2.39628331e-15, 2.93285700e-16/

      data coef_j(1:51, 5) /
     & 5.51439894e-05, 5.61737699e-05, 5.70971222e-05, 5.97158543e-05,
     & 6.64856520e-05, 5.78678247e-05, 5.11761113e-05, 4.71765037e-05,
     & 4.35390905e-05, 4.02099897e-05, 3.70855896e-05, 3.39125099e-05,
     & 3.07335362e-05, 2.76442550e-05, 2.46552200e-05, 2.15743729e-05,
     & 1.90924626e-05, 1.67544242e-05, 1.40714918e-05, 1.18579707e-05,
     & 9.93395930e-06, 8.17364957e-06, 6.61851880e-06, 5.21535125e-06,
     & 3.98524025e-06, 2.98960335e-06, 2.28581302e-06, 1.72017429e-06,
     & 1.27502343e-06, 9.16222511e-07, 6.69810519e-07, 4.60944116e-07,
     & 3.22015062e-07, 2.12576632e-07, 1.42144487e-07, 9.60192338e-08,
     & 5.76324937e-08, 3.09371550e-08, 1.49305800e-08, 6.13263252e-09,
     & 1.92306956e-09, 6.16745755e-10, 2.01158282e-10, 6.64767407e-11,
     & 2.22008383e-11, 4.35252215e-12, 8.66178627e-13, 1.74427211e-13,
     & 3.54643725e-14, 4.29177646e-15, 5.25278713e-16/

      data coef_j(1:51, 6) /
     & 8.91694801e-05,
     & 9.01232630e-05, 9.09107328e-05, 9.25454757e-05, 9.72160435e-05,
     & 1.08952038e-04, 9.40609395e-05, 8.21386656e-05, 7.52804540e-05,
     & 6.92140241e-05, 6.36798786e-05, 5.81457739e-05, 5.26491551e-05,
     & 4.73328076e-05, 4.22021470e-05, 3.69215245e-05, 3.26707758e-05,
     & 2.86680627e-05, 2.40761116e-05, 2.02882014e-05, 1.69960157e-05,
     & 1.39841085e-05, 1.13233607e-05, 8.92267193e-06, 6.81810177e-06,
     & 5.11470759e-06, 3.91063057e-06, 2.94291447e-06, 2.18133635e-06,
     & 1.56749018e-06, 1.14592288e-06, 7.88590043e-07, 5.50907755e-07,
     & 3.63678755e-07, 2.43182435e-07, 1.64270749e-07, 9.85982526e-08,
     & 5.29275745e-08, 2.55433660e-08, 1.04917571e-08, 3.29000168e-09,
     & 1.05513294e-09, 3.44142921e-10, 1.13728830e-10, 3.79813302e-11,
     & 7.44632040e-12, 1.48186329e-12, 2.98411029e-13, 6.06726381e-14,
     & 7.34239348e-15, 8.98649491e-16/

      data coef_j(1:51, 7) /
     & 1.48734554e-04, 1.49668443e-04,
     & 1.50406412e-04, 1.51853966e-04, 1.54848007e-04, 1.63105701e-04,
     & 1.83690606e-04, 1.57030317e-04, 1.35729830e-04, 1.23968355e-04,
     & 1.13534508e-04, 1.03387927e-04, 9.34668387e-05, 8.39522065e-05,
     & 7.48117928e-05, 6.54283214e-05, 5.78852350e-05, 5.07874030e-05,
     & 4.26485228e-05, 3.59366977e-05, 3.01041834e-05, 2.47687422e-05,
     & 2.00556632e-05, 1.58034172e-05, 1.20757841e-05, 9.05877784e-06,
     & 6.92617657e-06, 5.21222091e-06, 3.86337217e-06, 2.77618035e-06,
     & 2.02953927e-06, 1.39666621e-06, 9.75707553e-07, 6.44107267e-07,
     & 4.30697222e-07, 2.90937554e-07, 1.74625805e-07, 9.37391181e-08,
     & 4.52393852e-08, 1.85817455e-08, 5.82685384e-09, 1.86872322e-09,
     & 6.09503895e-10, 2.01422551e-10, 6.72678568e-11, 1.31880025e-11,
     & 2.62449257e-12, 5.28508532e-13, 1.07455822e-13, 1.30039311e-14,
     & 1.59157564e-15/

      data coef_j(1:51, 8) /
     & 2.56079901e-04, 2.57031747e-04, 2.57766555e-04,
     & 2.59163661e-04, 2.61888194e-04, 2.67327121e-04, 2.81742190e-04,
     & 3.19354058e-04, 2.70603482e-04, 2.32425980e-04, 2.11306977e-04,
     & 1.91402222e-04, 1.72512615e-04, 1.54685828e-04, 1.37706011e-04,
     & 1.20357383e-04, 1.06446541e-04, 9.33741396e-05, 7.83973100e-05,
     & 6.60530846e-05, 5.53292123e-05, 4.55210463e-05, 3.68579859e-05,
     & 2.90425875e-05, 2.21917680e-05, 1.66471758e-05, 1.27280184e-05,
     & 9.57827123e-06, 7.09951281e-06, 5.10161560e-06, 3.72954698e-06,
     & 2.56655106e-06, 1.79298202e-06, 1.18362336e-06, 7.91455799e-07,
     & 5.34630571e-07, 3.20894151e-07, 1.72255713e-07, 8.31321299e-08,
     & 3.41458713e-08, 1.07074315e-08, 3.43396440e-09, 1.12002317e-09,
     & 3.70133495e-10, 1.23611166e-10, 2.42342125e-11, 4.82275355e-12,
     & 9.71184154e-13, 1.97460132e-13, 2.38959340e-14, 2.92466775e-15/

      data coef_j(1:51, 9) /
     & 4.28478665e-04, 4.29422059e-04, 4.30140918e-04, 4.31484994e-04,
     & 4.34020658e-04, 4.38774835e-04, 4.48007289e-04, 4.74192413e-04,
     & 5.41887598e-04, 4.55714167e-04, 3.88809223e-04, 3.48834385e-04,
     & 3.12498533e-04, 2.79274664e-04, 2.48145828e-04, 2.16625539e-04,
     & 1.91470492e-04, 1.67889720e-04, 1.40916781e-04, 1.18707073e-04,
     & 9.94231585e-05, 8.17917639e-05, 6.62222043e-05, 5.21781154e-05,
     & 3.98685750e-05, 2.99067370e-05, 2.28655892e-05, 1.72069282e-05,
     & 1.27538260e-05, 9.16465428e-06, 6.69979937e-06, 4.61055441e-06,
     & 3.22090046e-06, 2.12624430e-06, 1.42175560e-06, 9.60397422e-07,
     & 5.76444952e-07, 3.09434371e-07, 1.49335410e-07, 6.13382136e-08,
     & 1.92343401e-08, 6.16860678e-09, 2.01195275e-09, 6.64888375e-10,
     & 2.22048435e-10, 4.35329925e-11, 8.66332015e-12, 1.74457896e-12,
     & 3.54705773e-13, 4.29252289e-14, 5.25369629e-15/

      data coef_j(1:51, 10) /
     & 6.81346346e-04,
     & 6.82255817e-04, 6.82943987e-04, 6.84219429e-04, 6.86585612e-04,
     & 6.90881586e-04, 6.98754285e-04, 7.15098144e-04, 7.61797141e-04,
     & 8.79148726e-04, 7.30251735e-04, 6.11067411e-04, 5.42551059e-04,
     & 4.82001904e-04, 4.26857355e-04, 3.71876578e-04, 3.28350939e-04,
     & 2.87718988e-04, 2.41367155e-04, 2.03264175e-04, 1.70210809e-04,
     & 1.40006824e-04, 1.13344596e-04, 8.93004808e-05, 6.82295010e-05,
     & 5.11791622e-05, 3.91286652e-05, 2.94446879e-05, 2.18241298e-05,
     & 1.56821832e-05, 1.14643071e-05, 7.88923736e-06, 5.51132513e-06,
     & 3.63822025e-06, 2.43275575e-06, 1.64332221e-06, 9.86342258e-07,
     & 5.29464043e-07, 2.55522410e-07, 1.04953205e-07, 3.29109410e-08,
     & 1.05547740e-08, 3.44253802e-09, 1.13765089e-09, 3.79933350e-10,
     & 7.44864964e-11, 1.48232305e-11, 2.98503002e-12, 6.06912361e-13,
     & 7.34463078e-14, 8.98921998e-15/

      data coef_j(1:51, 11) /
     & 1.11484772e-03, 1.11575700e-03,
     & 1.11644237e-03, 1.11770657e-03, 1.12003104e-03, 1.12418195e-03,
     & 1.13155813e-03, 1.14602725e-03, 1.17595605e-03, 1.25851233e-03,
     & 1.46433575e-03, 1.19777795e-03, 9.84898023e-04, 8.67486801e-04,
     & 7.63496933e-04, 6.62669548e-04, 5.84007289e-04, 5.11122829e-04,
     & 4.28377739e-04, 3.60559179e-04, 3.01823315e-04, 2.48203963e-04,
     & 2.00902454e-04, 1.58263961e-04, 1.20908862e-04, 9.06877167e-05,
     & 6.93314047e-05, 5.21706171e-05, 3.86672514e-05, 2.77844799e-05,
     & 2.03112075e-05, 1.39770538e-05, 9.76407481e-06, 6.44553426e-06,
     & 4.30987268e-06, 2.91128982e-06, 1.74737828e-06, 9.37977549e-07,
     & 4.52670223e-07, 1.85928420e-07, 5.83025562e-08, 1.86979588e-08,
     & 6.09849176e-09, 2.01535459e-09, 6.73052396e-10, 1.31952557e-10,
     & 2.62592423e-11, 5.28794933e-12, 1.07513735e-12, 1.30108980e-13,
     & 1.59242422e-14/

      data coef_j(1:51, 12) /
     & 1.87645118e-03, 1.87738833e-03, 1.87809318e-03,
     & 1.87938984e-03, 1.88176229e-03, 1.88596108e-03, 1.89330275e-03,
     & 1.90726205e-03, 1.93448596e-03, 1.98883781e-03, 2.13292357e-03,
     & 2.50895650e-03, 2.02160072e-03, 1.64022652e-03, 1.42967764e-03,
     & 1.23180309e-03, 1.08166934e-03, 9.44527709e-04, 7.90233024e-04,
     & 6.64466714e-04, 5.55869126e-04, 4.56912546e-04, 3.69718850e-04,
     & 2.91182454e-04, 2.22414806e-04, 1.66800680e-04, 1.27509362e-04,
     & 9.59420089e-05, 7.11054588e-05, 5.10907704e-05, 3.73475053e-05,
     & 2.56997018e-05, 1.79528489e-05, 1.18509126e-05, 7.92410065e-06,
     & 5.35260374e-06, 3.21262703e-06, 1.72448625e-06, 8.32230538e-07,
     & 3.41823776e-07, 1.07186229e-07, 3.43749330e-08, 1.12115909e-08,
     & 3.70504943e-09, 1.23734148e-09, 2.42580742e-10, 4.82746345e-11,
     & 9.72126355e-12, 1.97650655e-12, 2.39188535e-13, 2.92745938e-14/

      data coef_j(1:51, 13) /
     & 3.05596719e-03, 3.05690144e-03, 3.05760323e-03, 3.05889229e-03,
     & 3.06124422e-03, 3.06538575e-03, 3.07256282e-03, 3.08598245e-03,
     & 3.11130067e-03, 3.15877520e-03, 3.25098448e-03, 3.51262193e-03,
     & 4.18929515e-03, 3.32802567e-03, 2.66019874e-03, 2.26255826e-03,
     & 1.97241133e-03, 1.71470116e-03, 1.42979273e-03, 1.19998887e-03,
     & 1.00267078e-03, 8.23483583e-04, 6.65943148e-04, 5.24251321e-04,
     & 4.00308105e-04, 3.00140478e-04, 2.29403448e-04, 1.72588821e-04,
     & 1.27898063e-04, 9.18898504e-05, 6.71676646e-05, 4.62170248e-05,
     & 3.22840871e-05, 2.13103008e-05, 1.42486670e-05, 9.62450663e-06,
     & 5.77646456e-06, 3.10063265e-06, 1.49631817e-06, 6.14572206e-07,
     & 1.92708225e-07, 6.18011032e-08, 2.01565561e-08, 6.66099210e-09,
     & 2.22449327e-09, 4.36107753e-10, 8.67867310e-11, 1.74765026e-11,
     & 3.55326822e-12, 4.29999393e-13, 5.26279611e-14/

      data coef_j(1:51, 14) /
     & 4.58905399e-03,
     & 4.58994482e-03, 4.59061355e-03, 4.59184086e-03, 4.59407675e-03,
     & 4.59800330e-03, 4.60477562e-03, 4.61732873e-03, 4.64062203e-03,
     & 4.68292875e-03, 4.76051134e-03, 4.92176307e-03, 5.38442458e-03,
     & 6.54922376e-03, 5.03719066e-03, 3.84919968e-03, 3.34188460e-03,
     & 2.88688751e-03, 2.39518284e-03, 2.00456361e-03, 1.67194936e-03,
     & 1.37143155e-03, 1.10808371e-03, 8.71743749e-04, 6.65317431e-04,
     & 4.98659080e-04, 3.81044905e-04, 2.86621113e-04, 2.12371138e-04,
     & 1.52561656e-04, 1.11506092e-04, 7.67190760e-05, 5.35872980e-05,
     & 3.53701589e-05, 2.36483834e-05, 1.59731092e-05, 9.58640571e-06,
     & 5.14549598e-06, 2.48305034e-06, 1.01981189e-06, 3.19766809e-07,
     & 1.02546081e-07, 3.34450069e-08, 1.10521721e-08, 3.69092105e-09,
     & 7.23588135e-10, 1.43994616e-10, 2.89963727e-11, 5.89541038e-12,
     & 7.13428630e-13, 8.73165412e-14/

      data coef_j(1:51, 15) /
     & 7.63344239e-03, 7.63436095e-03,
     & 7.63505020e-03, 7.63631460e-03, 7.63861604e-03, 7.64265143e-03,
     & 7.64959272e-03, 7.66239614e-03, 7.68593738e-03, 7.72797518e-03,
     & 7.80267307e-03, 7.94918588e-03, 8.25219168e-03, 9.08595967e-03,
     & 1.11718179e-02, 8.50948584e-03, 6.67067668e-03, 5.66624412e-03,
     & 4.63749691e-03, 3.85370081e-03, 3.20032500e-03, 2.61728999e-03,
     & 2.11034644e-03, 1.65770792e-03, 1.26373000e-03, 9.46397409e-04,
     & 7.22788157e-04, 5.43449520e-04, 4.02532513e-04, 2.89087221e-04,
     & 2.11247909e-04, 1.45315957e-04, 1.01486491e-04, 6.69768382e-05,
     & 4.47757942e-05, 3.02408867e-05, 1.81477045e-05, 9.73991532e-06,
     & 4.69979281e-06, 1.93010363e-06, 6.05148839e-07, 1.94054898e-07,
     & 6.32876542e-08, 2.09132360e-08, 6.98388223e-09, 1.36911508e-09,
     & 2.72448318e-10, 5.48621737e-11, 1.11541473e-11, 1.34978709e-12,
     & 1.65198131e-13/

      data coef_j(1:51, 16) /
     & 1.06716479e-02, 1.06724962e-02, 1.06731327e-02,
     & 1.06743000e-02, 1.06764237e-02, 1.06801444e-02, 1.06865355e-02,
     & 1.06982944e-02, 1.07198145e-02, 1.07579188e-02, 1.08245983e-02,
     & 1.09515714e-02, 1.11999257e-02, 1.16984658e-02, 1.30537716e-02,
     & 1.64947744e-02, 1.24602153e-02, 9.56746194e-03, 7.64827637e-03,
     & 6.27825452e-03, 5.17676322e-03, 4.21361197e-03, 3.38651398e-03,
     & 2.65388823e-03, 2.01962478e-03, 1.51059058e-03, 1.15272726e-03,
     & 8.66154710e-04, 6.41233882e-04, 4.60319778e-04, 3.36269473e-04,
     & 2.31249862e-04, 1.61465725e-04, 1.06538851e-04, 7.12127109e-05,
     & 4.80898278e-05, 2.88550145e-05, 1.54845115e-05, 7.47082608e-06,
     & 3.06775966e-06, 9.61734204e-07, 3.08377280e-07, 1.00565701e-07,
     & 3.32300398e-08, 1.10965833e-08, 2.17526303e-09, 4.32852492e-10,
     & 8.71597423e-11, 1.77202052e-11, 2.14430323e-12, 2.62432019e-13/

      data coef_j(1:51, 17) /
     & 1.24208536e-02, 1.24215406e-02, 1.24220560e-02, 1.24230012e-02,
     & 1.24247202e-02, 1.24277306e-02, 1.24328975e-02, 1.24423905e-02,
     & 1.24597184e-02, 1.24902550e-02, 1.25432456e-02, 1.26425720e-02,
     & 1.28308312e-02, 1.31867664e-02, 1.38836281e-02, 1.61371087e-02,
     & 2.09342998e-02, 1.50640284e-02, 1.04396460e-02, 8.52366488e-03,
     & 6.96923411e-03, 5.63914478e-03, 4.51383247e-03, 3.52682352e-03,
     & 2.67803436e-03, 1.99989867e-03, 1.52453504e-03, 1.14460280e-03,
     & 8.46833440e-04, 6.07587365e-04, 4.43675885e-04, 3.05000337e-04,
     & 2.12901324e-04, 1.40441165e-04, 9.38549059e-05, 6.33698772e-05,
     & 3.80168928e-05, 2.03976823e-05, 9.83979455e-06, 4.03995811e-06,
     & 1.26634031e-06, 4.06007089e-07, 1.32393704e-07, 4.37443103e-08,
     & 1.46069081e-08, 2.86322161e-09, 5.69721947e-10, 1.14715681e-10,
     & 2.33218140e-11, 2.82205447e-12, 3.45369833e-13/

      data coef_j(1:51, 18) /
     & 1.94810935e-02,
     & 1.94818503e-02, 1.94824179e-02, 1.94834588e-02, 1.94853517e-02,
     & 1.94886657e-02, 1.94943513e-02, 1.95047890e-02, 1.95238140e-02,
     & 1.95572554e-02, 1.96150267e-02, 1.97224258e-02, 1.99228219e-02,
     & 2.02906364e-02, 2.09790973e-02, 2.24766315e-02, 2.59060675e-02,
     & 3.37894606e-02, 2.36650522e-02, 1.67865205e-02, 1.34852146e-02,
     & 1.07554640e-02, 8.52894897e-03, 6.62031125e-03, 5.00327821e-03,
     & 3.72392321e-03, 2.83262704e-03, 2.12313739e-03, 1.56873156e-03,
     & 1.12430016e-03, 8.20332823e-04, 5.63506593e-04, 3.93125304e-04,
     & 2.59191151e-04, 1.73143499e-04, 1.16866620e-04, 7.00863842e-05,
     & 3.75917115e-05, 1.81285895e-05, 7.44097441e-06, 2.33174953e-06,
     & 7.47439487e-07, 2.43692177e-07, 8.05085341e-08, 2.68803496e-08,
     & 5.26840946e-09, 1.04820661e-09, 2.11044554e-10, 4.29029307e-11,
     & 5.19111983e-12, 6.35267562e-13/

      data coef_j(1:51, 19) /
     & 2.68144794e-02, 2.68152340e-02,
     & 2.68158000e-02, 2.68168377e-02, 2.68187248e-02, 2.68220282e-02,
     & 2.68276939e-02, 2.68380900e-02, 2.68570222e-02, 2.68902478e-02,
     & 2.69474868e-02, 2.70533594e-02, 2.72490323e-02, 2.76017887e-02,
     & 2.82397420e-02, 2.95368461e-02, 3.16415686e-02, 3.66967999e-02,
     & 4.98810707e-02, 3.43114781e-02, 2.40016204e-02, 1.87161429e-02,
     & 1.45996143e-02, 1.12081149e-02, 8.40567489e-03, 6.22335553e-03,
     & 4.71782027e-03, 3.52695811e-03, 2.60069160e-03, 1.86076446e-03,
     & 1.35601978e-03, 9.30420718e-04, 6.48541490e-04, 4.27250986e-04,
     & 2.85234445e-04, 1.92429775e-04, 1.15342167e-04, 6.18339132e-05,
     & 2.98055973e-05, 1.22285626e-05, 3.83040581e-06, 1.22745361e-06,
     & 4.00099709e-07, 1.32156222e-07, 4.41179135e-08, 8.64532737e-09,
     & 1.71983935e-09, 3.46231251e-10, 7.03783355e-11, 8.51470572e-12,
     & 1.04191002e-12/

      data coef_j(1:51, 20) /
     & 2.93105845e-02, 2.93111926e-02, 2.93116486e-02,
     & 2.93124848e-02, 2.93140052e-02, 2.93166665e-02, 2.93212301e-02,
     & 2.93296012e-02, 2.93448369e-02, 2.93715472e-02, 2.94174786e-02,
     & 2.95021568e-02, 2.96577012e-02, 2.99349550e-02, 3.04259264e-02,
     & 3.13820917e-02, 3.28196439e-02, 3.52993354e-02, 4.37293513e-02,
     & 6.16813834e-02, 4.10710605e-02, 2.73272861e-02, 2.08412122e-02,
     & 1.57078159e-02, 1.16349995e-02, 8.54306269e-03, 6.44256291e-03,
     & 4.79727031e-03, 3.52654009e-03, 2.51682379e-03, 1.83075442e-03,
     & 1.25402197e-03, 8.72988704e-04, 5.74439671e-04, 3.83149531e-04,
     & 2.58298944e-04, 1.54704606e-04, 8.28739739e-05, 3.99203092e-05,
     & 1.63679751e-05, 5.12384714e-06, 1.64119506e-06, 5.34777682e-07,
     & 1.76593238e-07, 5.89393441e-08, 1.15466876e-08, 2.29654360e-09,
     & 4.62254965e-10, 9.39497631e-11, 1.13648268e-11, 1.39050368e-12/

      data coef_j(1:51, 21) /
     & 3.38719415e-02, 3.38724916e-02, 3.38729041e-02, 3.38736604e-02,
     & 3.38750357e-02, 3.38774427e-02, 3.38815700e-02, 3.38891395e-02,
     & 3.39029120e-02, 3.39270440e-02, 3.39685030e-02, 3.40448073e-02,
     & 3.41845344e-02, 3.44321947e-02, 3.48663543e-02, 3.56957363e-02,
     & 3.69034406e-02, 3.89082216e-02, 4.34786937e-02, 5.48837934e-02,
     & 7.92311401e-02, 5.11201902e-02, 3.29130965e-02, 2.41709942e-02,
     & 1.75449024e-02, 1.27168996e-02, 9.51480609e-03, 7.04353074e-03,
     & 5.15471899e-03, 3.66546818e-03, 2.65931636e-03, 1.81718603e-03,
     & 1.26276077e-03, 8.29549046e-04, 5.52603530e-04, 3.72158022e-04,
     & 2.22659715e-04, 1.19153630e-04, 5.73421433e-05, 2.34905250e-05,
     & 7.34723417e-06, 2.35189262e-06, 7.65991588e-07, 2.52849277e-07,
     & 8.43647141e-08, 1.65217439e-08, 3.28511655e-09, 6.61088474e-10,
     & 1.34336273e-10, 1.62470060e-11, 1.98752421e-12/

      data coef_j(1:51, 22) /
     & 3.78705822e-02,
     & 3.78710789e-02, 3.78714514e-02, 3.78721343e-02, 3.78733761e-02,
     & 3.78755493e-02, 3.78792755e-02, 3.78861088e-02, 3.78985392e-02,
     & 3.79203122e-02, 3.79576958e-02, 3.80264251e-02, 3.81520308e-02,
     & 3.83738644e-02, 3.87602803e-02, 3.94896661e-02, 4.05309718e-02,
     & 4.22050142e-02, 4.58079989e-02, 5.18680175e-02, 6.69949177e-02,
     & 9.99650283e-02, 6.22151021e-02, 3.79264661e-02, 2.68360813e-02,
     & 1.90636001e-02, 1.40947336e-02, 1.03444628e-02, 7.52188726e-03,
     & 5.32118372e-03, 3.84637814e-03, 2.61953759e-03, 1.81578530e-03,
     & 1.19014956e-03, 7.91436695e-04, 5.32265138e-04, 3.17985329e-04,
     & 1.69927045e-04, 8.16723406e-05, 3.34177509e-05, 1.04402136e-05,
     & 3.33917358e-06, 1.08684559e-06, 3.58580423e-07, 1.19593740e-07,
     & 2.34095416e-08, 4.65290141e-09, 9.36054907e-10, 1.90163622e-10,
     & 2.29927543e-11, 2.81213504e-12/

      data coef_j(1:51, 23) /
     & 4.07045732e-02, 4.07050138e-02,
     & 4.07053442e-02, 4.07059501e-02, 4.07070517e-02, 4.07089796e-02,
     & 4.07122849e-02, 4.07183460e-02, 4.07293704e-02, 4.07486759e-02,
     & 4.07818096e-02, 4.08426810e-02, 4.09537773e-02, 4.11495151e-02,
     & 4.14890357e-02, 4.21248963e-02, 4.30212543e-02, 4.44338295e-02,
     & 4.73542676e-02, 5.19486346e-02, 5.96595030e-02, 8.00440021e-02,
     & 1.23805652e-01, 7.36099127e-02, 4.22094370e-02, 2.93020252e-02,
     & 2.12632345e-02, 1.53999160e-02, 1.10899988e-02, 7.78581980e-03,
     & 5.59792888e-03, 3.79408528e-03, 2.62063726e-03, 1.71219569e-03,
     & 1.13580719e-03, 7.62385553e-04, 4.54536615e-04, 2.42425104e-04,
     & 1.16311835e-04, 4.75131868e-05, 1.48204245e-05, 4.73468735e-06,
     & 1.53971465e-06, 5.07644565e-07, 1.69215319e-07, 3.31006873e-08,
     & 6.57573986e-09, 1.32234067e-09, 2.68548744e-10, 3.24584914e-11,
     & 3.96867843e-12/

      data coef_j(1:51, 24) /
     & 4.32342833e-02, 4.32346787e-02, 4.32349753e-02,
     & 4.32355189e-02, 4.32365074e-02, 4.32382374e-02, 4.32412033e-02,
     & 4.32466417e-02, 4.32565325e-02, 4.32738504e-02, 4.33035646e-02,
     & 4.33581273e-02, 4.34576211e-02, 4.36326376e-02, 4.39353696e-02,
     & 4.44994366e-02, 4.52881489e-02, 4.65156962e-02, 4.89934064e-02,
     & 5.27270383e-02, 5.86246340e-02, 6.89792766e-02, 9.55178919e-02,
     & 1.53805931e-01, 8.72665990e-02, 4.81287151e-02, 3.42518696e-02,
     & 2.42867642e-02, 1.72256496e-02, 1.19534085e-02, 8.52602534e-03,
     & 5.73789848e-03, 3.94296355e-03, 2.56435298e-03, 1.69519482e-03,
     & 1.13475697e-03, 6.74616532e-04, 3.58827677e-04, 1.71739540e-04,
     & 6.99968308e-05, 2.17862780e-05, 6.94913635e-06, 2.25715388e-06,
     & 7.43486392e-07, 2.47641764e-07, 4.83984298e-08, 9.60806708e-09,
     & 1.93104607e-09, 3.91989428e-10, 4.73549560e-11, 5.78776151e-12/

      data coef_j(1:51, 25) /
     & 4.32087632e-02, 4.32091029e-02, 4.32093576e-02, 4.32098246e-02,
     & 4.32106738e-02, 4.32121598e-02, 4.32147075e-02, 4.32193789e-02,
     & 4.32278743e-02, 4.32427472e-02, 4.32682614e-02, 4.33150958e-02,
     & 4.34004438e-02, 4.35504092e-02, 4.38093044e-02, 4.42899816e-02,
     & 4.49583505e-02, 4.59898809e-02, 4.80394281e-02, 5.10451508e-02,
     & 5.55906118e-02, 6.30744680e-02, 7.57836932e-02, 1.10123496e-01,
     & 1.85653774e-01, 1.03646665e-01, 5.94714146e-02, 4.05637117e-02,
     & 2.79429568e-02, 1.89929637e-02, 1.33657202e-02, 8.89196126e-03,
     & 6.06091784e-03, 3.91386910e-03, 2.57362900e-03, 1.71569689e-03,
     & 1.01565207e-03, 5.38061069e-04, 2.56602824e-04, 1.04242060e-04,
     & 3.23436807e-05, 1.02933846e-05, 3.33770653e-06, 1.09794325e-06,
     & 3.65311742e-07, 7.13048076e-08, 1.41415068e-08, 2.83995095e-09,
     & 5.76120933e-10, 6.95510376e-11, 8.49584676e-12/

      data coef_j(1:51, 26) /
     & 3.61991027e-02,
     & 3.61993546e-02, 3.61995435e-02, 3.61998898e-02, 3.62005195e-02,
     & 3.62016216e-02, 3.62035109e-02, 3.62069750e-02, 3.62132746e-02,
     & 3.62243025e-02, 3.62432184e-02, 3.62779330e-02, 3.63411687e-02,
     & 3.64521995e-02, 3.66436361e-02, 3.69982490e-02, 3.74895600e-02,
     & 3.82437785e-02, 3.97276968e-02, 4.18684883e-02, 4.50254746e-02,
     & 5.00133841e-02, 5.80103385e-02, 7.21340556e-02, 1.13025502e-01,
     & 1.99418938e-01, 1.09586908e-01, 5.94793914e-02, 3.95984015e-02,
     & 2.61425920e-02, 1.80656790e-02, 1.18401639e-02, 7.98800065e-03,
     & 5.11311185e-03, 3.34058255e-03, 2.21597730e-03, 1.30517128e-03,
     & 6.88176910e-04, 3.26823531e-04, 1.32263645e-04, 4.08905223e-05,
     & 1.29798657e-05, 4.20064215e-06, 1.37970996e-06, 4.58502450e-07,
     & 8.93659154e-08, 1.77037640e-08, 3.55218722e-09, 7.20086542e-10,
     & 8.68632550e-11, 1.06039398e-11/

      data coef_j(1:51, 27) /
     & 2.93072216e-02, 2.93074062e-02,
     & 2.93075447e-02, 2.93077986e-02, 2.93082602e-02, 2.93090679e-02,
     & 2.93104528e-02, 2.93129919e-02, 2.93176092e-02, 2.93256917e-02,
     & 2.93395542e-02, 2.93649910e-02, 2.94113137e-02, 2.94926082e-02,
     & 2.96326546e-02, 2.98916735e-02, 3.02496816e-02, 3.07973230e-02,
     & 3.18679302e-02, 3.33963795e-02, 3.56156824e-02, 3.90380654e-02,
     & 4.43148129e-02, 5.31920594e-02, 6.91640591e-02, 1.16575775e-01,
     & 2.09934433e-01, 1.12013093e-01, 5.82642687e-02, 3.70523334e-02,
     & 2.49076025e-02, 1.59750451e-02, 1.06254412e-02, 6.72152469e-03,
     & 4.35445309e-03, 2.87016476e-03, 1.67963537e-03, 8.80394242e-04,
     & 4.15953076e-04, 1.67551113e-04, 5.15743081e-05, 1.63204537e-05,
     & 5.26947577e-06, 1.72764130e-06, 5.73294937e-07, 1.11549698e-07,
     & 2.20694579e-08, 4.42352504e-09, 8.95959064e-10, 1.07979712e-10,
     & 1.31720886e-11/

      data coef_j(1:51, 28) /
     & 2.56383409e-02, 2.56384908e-02, 2.56386032e-02,
     & 2.56388093e-02, 2.56391840e-02, 2.56398398e-02, 2.56409641e-02,
     & 2.56430254e-02, 2.56467737e-02, 2.56533350e-02, 2.56645878e-02,
     & 2.56852340e-02, 2.57228259e-02, 2.57887774e-02, 2.59023307e-02,
     & 2.61121441e-02, 2.64017019e-02, 2.68436483e-02, 2.77041846e-02,
     & 2.89248481e-02, 3.06808077e-02, 3.33507344e-02, 3.73782881e-02,
     & 4.39205671e-02, 5.52858594e-02, 7.56826005e-02, 1.25832006e-01,
     & 2.30642137e-01, 1.18332498e-01, 5.71643469e-02, 3.76047112e-02,
     & 2.34557191e-02, 1.53226318e-02, 9.55330997e-03, 6.12663729e-03,
     & 4.00811123e-03, 2.32818065e-03, 1.21213576e-03, 5.69369511e-04,
     & 2.28164697e-04, 6.98955884e-05, 2.20433677e-05, 7.09934098e-06,
     & 2.32303675e-06, 7.69668743e-07, 1.49486186e-07, 2.95334964e-08,
     & 5.91300771e-09, 1.19656261e-09, 1.44067978e-10, 1.75607129e-11/

      data coef_j(1:51, 29) /
     & 2.22761665e-02, 2.22762878e-02, 2.22763789e-02, 2.22765457e-02,
     & 2.22768492e-02, 2.22773802e-02, 2.22782905e-02, 2.22799595e-02,
     & 2.22829944e-02, 2.22883068e-02, 2.22974173e-02, 2.23141315e-02,
     & 2.23445596e-02, 2.23979289e-02, 2.24897765e-02, 2.26593445e-02,
     & 2.28930634e-02, 2.32491203e-02, 2.39401206e-02, 2.49151104e-02,
     & 2.63070342e-02, 2.83995587e-02, 3.15019911e-02, 3.64043040e-02,
     & 4.45264732e-02, 5.81378958e-02, 7.88629990e-02, 1.36137259e-01,
     & 2.57211631e-01, 1.30192772e-01, 6.63608831e-02, 3.93311909e-02,
     & 2.48502333e-02, 1.51148435e-02, 9.53662310e-03, 6.16693160e-03,
     & 3.54233209e-03, 1.82616568e-03, 8.50690362e-04, 3.38428996e-04,
     & 1.02988259e-04, 3.23302532e-05, 1.03769100e-05, 3.38662021e-06,
     & 1.11972018e-06, 2.16945941e-07, 4.27817069e-08, 8.55290917e-09,
     & 1.72871725e-09, 2.07874878e-10, 2.53124541e-11/

      data coef_j(1:51, 30) /
     & 1.72345593e-02,
     & 1.72346481e-02, 1.72347146e-02, 1.72348367e-02, 1.72350586e-02,
     & 1.72354470e-02, 1.72361127e-02, 1.72373334e-02, 1.72395531e-02,
     & 1.72434383e-02, 1.72501011e-02, 1.72623239e-02, 1.72845732e-02,
     & 1.73235899e-02, 1.73907151e-02, 1.75145682e-02, 1.76851231e-02,
     & 1.79446101e-02, 1.84470178e-02, 1.91532719e-02, 2.01562308e-02,
     & 2.16523609e-02, 2.38450945e-02, 2.72488293e-02, 3.27267114e-02,
     & 4.14736368e-02, 5.40800824e-02, 7.55433816e-02, 1.36071502e-01,
     & 2.66937747e-01, 1.30878244e-01, 5.70704857e-02, 3.54810703e-02,
     & 2.10071223e-02, 1.30176391e-02, 8.31203265e-03, 4.71755468e-03,
     & 2.40684877e-03, 1.11154500e-03, 4.38918146e-04, 1.32672849e-04,
     & 4.14559690e-05, 1.32607288e-05, 4.31651874e-06, 1.42423313e-06,
     & 2.75284873e-07, 5.41870348e-08, 1.08174709e-08, 2.18388654e-09,
     & 2.62281135e-10, 3.19056755e-11/

      data coef_j(1:51, 31) /
     & 1.37799601e-02, 1.37800274e-02,
     & 1.37800778e-02, 1.37801704e-02, 1.37803386e-02, 1.37806330e-02,
     & 1.37811376e-02, 1.37820629e-02, 1.37837454e-02, 1.37866904e-02,
     & 1.37917405e-02, 1.38010045e-02, 1.38178664e-02, 1.38474308e-02,
     & 1.38982799e-02, 1.39920548e-02, 1.41210902e-02, 1.43171872e-02,
     & 1.46961074e-02, 1.52270987e-02, 1.59778404e-02, 1.70905750e-02,
     & 1.87061758e-02, 2.11789725e-02, 2.50716330e-02, 3.10732443e-02,
     & 3.92866830e-02, 5.24646620e-02, 7.45948113e-02, 1.44921051e-01,
     & 2.88593785e-01, 1.37313825e-01, 6.35764233e-02, 3.58313861e-02,
     & 2.14391887e-02, 1.33813743e-02, 7.44208358e-03, 3.73383866e-03,
     & 1.70148693e-03, 6.64396181e-04, 1.98863645e-04, 6.17263113e-05,
     & 1.96498532e-05, 6.37293608e-06, 2.09672615e-06, 4.03929604e-07,
     & 7.93103504e-08, 1.58017896e-08, 3.18509708e-09, 3.81879489e-10,
     & 4.63921497e-11/

      data coef_j(1:51, 32) /
     & 1.10836014e-02, 1.10836534e-02, 1.10836924e-02,
     & 1.10837639e-02, 1.10838939e-02, 1.10841214e-02, 1.10845115e-02,
     & 1.10852267e-02, 1.10865271e-02, 1.10888033e-02, 1.10927065e-02,
     & 1.10998664e-02, 1.11128976e-02, 1.11357432e-02, 1.11750288e-02,
     & 1.12474539e-02, 1.13470595e-02, 1.14983165e-02, 1.17902001e-02,
     & 1.21983629e-02, 1.27737467e-02, 1.36229698e-02, 1.48484789e-02,
     & 1.67074935e-02, 1.95944437e-02, 2.39552086e-02, 2.97559740e-02,
     & 3.86981296e-02, 5.32679383e-02, 8.05305435e-02, 1.50256705e-01,
     & 3.10028330e-01, 1.43359269e-01, 5.79514507e-02, 3.41359790e-02,
     & 2.07888262e-02, 1.13072749e-02, 5.57200234e-03, 2.50380051e-03,
     & 9.66527996e-04, 2.86446076e-04, 8.83262559e-05, 2.79849270e-05,
     & 9.04393983e-06, 2.96723592e-06, 5.69809173e-07, 1.11611337e-07,
     & 2.21956566e-08, 4.46712809e-09, 5.34730172e-10, 6.48783939e-11/

      data coef_j(1:51, 33) /
     & 8.05345169e-03, 8.05348793e-03, 8.05351512e-03, 8.05356495e-03,
     & 8.05365555e-03, 8.05381410e-03, 8.05408592e-03, 8.05458428e-03,
     & 8.05549048e-03, 8.05707659e-03, 8.05979643e-03, 8.06478541e-03,
     & 8.07386492e-03, 8.08978091e-03, 8.11714522e-03, 8.16757597e-03,
     & 8.23689741e-03, 8.34208772e-03, 8.54480995e-03, 8.82771009e-03,
     & 9.22537505e-03, 9.80990903e-03, 1.06485399e-02, 1.19099547e-02,
     & 1.38442261e-02, 1.67118796e-02, 2.04310483e-02, 2.59634497e-02,
     & 3.44927631e-02, 4.92459652e-02, 7.17143792e-02, 1.53427244e-01,
     & 3.26514616e-01, 1.45742764e-01, 6.13074379e-02, 3.60649980e-02,
     & 1.88174151e-02, 8.98850140e-03, 3.94809638e-03, 1.49719844e-03,
     & 4.37206538e-04, 1.33525468e-04, 4.20207143e-05, 1.35118440e-05,
     & 4.41591581e-06, 8.44260219e-07, 1.64822984e-07, 3.26934692e-08,
     & 6.56642250e-09, 7.84314417e-10, 9.49969176e-11/

      data coef_j(1:51, 34) /
     & 6.23899214e-03,
     & 6.23901934e-03, 6.23903975e-03, 6.23907716e-03, 6.23914519e-03,
     & 6.23926423e-03, 6.23946831e-03, 6.23984247e-03, 6.24052283e-03,
     & 6.24171365e-03, 6.24375562e-03, 6.24750111e-03, 6.25431728e-03,
     & 6.26626487e-03, 6.28680368e-03, 6.32464677e-03, 6.37664705e-03,
     & 6.45551338e-03, 6.60736780e-03, 6.81898452e-03, 7.11587158e-03,
     & 7.55107120e-03, 8.17302480e-03, 9.10333057e-03, 1.05183121e-02,
     & 1.25918504e-02, 1.52408962e-02, 1.91039365e-02, 2.48947623e-02,
     & 3.45085256e-02, 4.88880645e-02, 7.83625393e-02, 1.60081234e-01,
     & 3.50633150e-01, 1.58268933e-01, 6.90948156e-02, 3.39747806e-02,
     & 1.55020551e-02, 6.60454805e-03, 2.44930269e-03, 7.02699419e-04,
     & 2.12238793e-04, 6.62828734e-05, 2.11942671e-05, 6.89702873e-06,
     & 1.31225577e-06, 2.55272620e-07, 5.04948874e-08, 1.01195427e-08,
     & 1.20591545e-09, 1.45796071e-10/

      data coef_j(1:51, 35) /
     & 3.86282087e-03, 3.86283723e-03,
     & 3.86284949e-03, 3.86287197e-03, 3.86291285e-03, 3.86298440e-03,
     & 3.86310704e-03, 3.86333191e-03, 3.86374079e-03, 3.86445644e-03,
     & 3.86568359e-03, 3.86793446e-03, 3.87203051e-03, 3.87920970e-03,
     & 3.89154986e-03, 3.91428208e-03, 3.94550854e-03, 3.99284634e-03,
     & 4.08391991e-03, 4.21067527e-03, 4.38819654e-03, 4.64778015e-03,
     & 5.01747072e-03, 5.56772337e-03, 6.39868712e-03, 7.60416847e-03,
     & 9.12449772e-03, 1.13047814e-02, 1.44977905e-02, 1.96168808e-02,
     & 2.68935185e-02, 4.08571483e-02, 6.32813848e-02, 1.50625472e-01,
     & 3.39572661e-01, 1.48214255e-01, 4.93249265e-02, 2.20913623e-02,
     & 9.10755191e-03, 3.29653797e-03, 9.28076731e-04, 2.77064012e-04,
     & 8.58448019e-05, 2.72917711e-05, 8.84251505e-06, 1.67419032e-06,
     & 3.24506159e-07, 6.40121059e-08, 1.28003362e-08, 1.52186166e-09,
     & 1.83661677e-10/

      data coef_j(1:51, 36) /
     & 2.98227701e-03, 2.98228934e-03, 2.98229858e-03,
     & 2.98231552e-03, 2.98234633e-03, 2.98240025e-03, 2.98249267e-03,
     & 2.98266213e-03, 2.98297027e-03, 2.98350958e-03, 2.98443436e-03,
     & 2.98613057e-03, 2.98921719e-03, 2.99462686e-03, 3.00392462e-03,
     & 3.02104965e-03, 3.04456802e-03, 3.08020825e-03, 3.14873479e-03,
     & 3.24401847e-03, 3.37728863e-03, 3.57180598e-03, 3.84811665e-03,
     & 4.25788866e-03, 4.87350005e-03, 5.76017191e-03, 6.86845602e-03,
     & 8.44008711e-03, 1.07075673e-02, 1.42674113e-02, 1.91878660e-02,
     & 2.82561307e-02, 4.26527329e-02, 7.23734657e-02, 1.66053959e-01,
     & 3.70990169e-01, 1.46316337e-01, 4.02698622e-02, 1.66654154e-02,
     & 5.85171602e-03, 1.60804097e-03, 4.73088651e-04, 1.45155849e-04,
     & 4.58264260e-05, 1.47698909e-05, 2.78019464e-06, 5.36593606e-07,
     & 1.05505562e-07, 2.10437806e-08, 2.49526484e-09, 3.00506659e-10/

      data coef_j(1:51, 37) /
     & 2.45811060e-03, 2.45812052e-03, 2.45812796e-03, 2.45814160e-03,
     & 2.45816641e-03, 2.45820983e-03, 2.45828425e-03, 2.45842070e-03,
     & 2.45866882e-03, 2.45910309e-03, 2.45984773e-03, 2.46121352e-03,
     & 2.46369880e-03, 2.46805437e-03, 2.47553979e-03, 2.48932478e-03,
     & 2.50825196e-03, 2.53692541e-03, 2.59202576e-03, 2.66857395e-03,
     & 2.77551117e-03, 2.93133156e-03, 3.15215542e-03, 3.47856747e-03,
     & 3.96667106e-03, 4.66523184e-03, 5.53158490e-03, 6.74834637e-03,
     & 8.48186839e-03, 1.11569348e-02, 1.47721717e-02, 2.12195479e-02,
     & 3.09773686e-02, 5.02403557e-02, 8.47913144e-02, 1.81191796e-01,
     & 4.35742447e-01, 1.57375913e-01, 3.77857605e-02, 1.33290042e-02,
     & 3.53494257e-03, 1.01785399e-03, 3.07954790e-04, 9.62707958e-05,
     & 3.08028604e-05, 5.75212773e-06, 1.10383599e-06, 2.16097638e-07,
     & 4.29560091e-08, 5.07558302e-09, 6.09587246e-10/

      data coef_j(1:51, 38) /
     & 1.59316061e-03,
     & 1.59316688e-03, 1.59317159e-03, 1.59318020e-03, 1.59319588e-03,
     & 1.59322330e-03, 1.59327032e-03, 1.59335653e-03, 1.59351327e-03,
     & 1.59378762e-03, 1.59425804e-03, 1.59512085e-03, 1.59669084e-03,
     & 1.59944218e-03, 1.60417019e-03, 1.61287589e-03, 1.62482628e-03,
     & 1.64292426e-03, 1.67768186e-03, 1.72592489e-03, 1.79323589e-03,
     & 1.89114489e-03, 2.02956203e-03, 2.23347321e-03, 2.53694524e-03,
     & 2.96846806e-03, 3.49945234e-03, 4.23810820e-03, 5.27764154e-03,
     & 6.85566320e-03, 8.94419422e-03, 1.25605380e-02, 1.78083775e-02,
     & 2.75074712e-02, 4.34318231e-02, 6.94569400e-02, 1.80213811e-01,
     & 4.82085871e-01, 1.59332134e-01, 2.91552500e-02, 7.98884103e-03,
     & 2.23838591e-03, 6.64530586e-04, 2.04998005e-04, 6.49535002e-05,
     & 1.20017500e-05, 2.28581804e-06, 4.44969147e-07, 8.80630041e-08,
     & 1.03581918e-08, 1.23968941e-09/

      data coef_j(1:51, 39) /
     & 9.19353157e-04, 9.19356689e-04,
     & 9.19359338e-04, 9.19364194e-04, 9.19373024e-04, 9.19388477e-04,
     & 9.19414967e-04, 9.19463536e-04, 9.19551848e-04, 9.19706417e-04,
     & 9.19971453e-04, 9.20457558e-04, 9.21342061e-04, 9.22892046e-04,
     & 9.25555409e-04, 9.30458801e-04, 9.37188357e-04, 9.47376741e-04,
     & 9.66933632e-04, 9.94056491e-04, 1.03185805e-03, 1.08675904e-03,
     & 1.16420985e-03, 1.27797277e-03, 1.44658757e-03, 1.68503220e-03,
     & 1.97650300e-03, 2.37877954e-03, 2.93932288e-03, 3.77931108e-03,
     & 4.87354280e-03, 6.72829417e-03, 9.34450658e-03, 1.39897358e-02,
     & 2.12049787e-02, 3.26161983e-02, 5.81647722e-02, 1.80163994e-01,
     & 5.28527592e-01, 1.49377768e-01, 1.15278749e-02, 4.59015236e-03,
     & 1.38941231e-03, 4.28560071e-04, 1.35327423e-04, 2.48657102e-05,
     & 4.71313230e-06, 9.13883767e-07, 1.80285390e-07, 2.11329971e-08,
     & 2.52237838e-09/

      data coef_j(1:51, 40) /
     & 4.97079819e-04, 4.97081685e-04, 4.97083085e-04,
     & 4.97085651e-04, 4.97090317e-04, 4.97098482e-04, 4.97112480e-04,
     & 4.97138144e-04, 4.97184809e-04, 4.97266483e-04, 4.97406528e-04,
     & 4.97663382e-04, 4.98130737e-04, 4.98949688e-04, 5.00356809e-04,
     & 5.02947077e-04, 5.06501374e-04, 5.11881044e-04, 5.22202627e-04,
     & 5.36506913e-04, 5.56423251e-04, 5.85308796e-04, 6.25981209e-04,
     & 6.85566078e-04, 7.73559903e-04, 8.97395372e-04, 1.04790530e-03,
     & 1.25423681e-03, 1.53936087e-03, 1.96212602e-03, 2.50592714e-03,
     & 3.41272196e-03, 4.66542054e-03, 6.82886520e-03, 1.00691970e-02,
     & 1.49757493e-02, 2.58195605e-02, 5.09895938e-02, 1.82142093e-01,
     & 6.01661913e-01, 1.50263934e-01, 1.58288354e-02, 5.51672004e-03,
     & 1.65384948e-03, 5.09052680e-04, 9.09755702e-05, 1.69166221e-05,
     & 3.23492020e-06, 6.31498526e-07, 7.32461577e-08, 8.67314425e-09/

      data coef_j(1:51, 41) /
     & 1.99897580e-04, 1.99898314e-04, 1.99898865e-04, 1.99899876e-04,
     & 1.99901713e-04, 1.99904927e-04, 1.99910438e-04, 1.99920541e-04,
     & 1.99938912e-04, 1.99971066e-04, 2.00026199e-04, 2.00127316e-04,
     & 2.00311299e-04, 2.00633683e-04, 2.01187569e-04, 2.02207070e-04,
     & 2.03605772e-04, 2.05722301e-04, 2.09781451e-04, 2.15403274e-04,
     & 2.23223904e-04, 2.34552788e-04, 2.50477950e-04, 2.73755006e-04,
     & 3.08022036e-04, 3.56046650e-04, 4.14131503e-04, 4.93307223e-04,
     & 6.01961696e-04, 7.61675253e-04, 9.65031926e-04, 1.29979774e-03,
     & 1.75495195e-03, 2.52510301e-03, 3.64908014e-03, 5.29931482e-03,
     & 8.79054164e-03, 1.67919264e-02, 3.55701775e-02, 1.66679011e-01,
     & 6.41873216e-01, 1.58308363e-01, 1.77085013e-02, 6.03634025e-03,
     & 1.80763238e-03, 3.11806787e-04, 5.65880157e-05, 1.06367396e-05,
     & 2.05017106e-06, 2.34827287e-07, 2.75486735e-08/

      data coef_j(1:51, 42) /
     & 5.28818603e-05,
     & 5.28820503e-05, 5.28821928e-05, 5.28824541e-05, 5.28829291e-05,
     & 5.28837603e-05, 5.28851853e-05, 5.28877980e-05, 5.28925486e-05,
     & 5.29008632e-05, 5.29151200e-05, 5.29412674e-05, 5.29888416e-05,
     & 5.30722006e-05, 5.32154105e-05, 5.34789783e-05, 5.38405186e-05,
     & 5.43874713e-05, 5.54359909e-05, 5.68872122e-05, 5.89042335e-05,
     & 6.18224554e-05, 6.59176796e-05, 7.18895835e-05, 8.06530367e-05,
     & 9.28834987e-05, 1.07603823e-04, 1.27556001e-04, 1.54750330e-04,
     & 1.94387175e-04, 2.44363059e-04, 3.25638255e-04, 4.34517962e-04,
     & 6.15363065e-04, 8.73306817e-04, 1.24206340e-03, 1.99323939e-03,
     & 3.61586646e-03, 7.36911421e-03, 1.44074720e-02, 1.54062035e-01,
     & 6.33582634e-01, 1.56936612e-01, 1.73635032e-02, 5.93902443e-03,
     & 9.83219604e-04, 1.72819923e-04, 3.17727814e-05, 6.02693095e-06,
     & 6.79784799e-07, 7.88643246e-08/

      data coef_j(1:51, 43) /
     & 1.82807098e-05, 1.82807748e-05,
     & 1.82808235e-05, 1.82809129e-05, 1.82810753e-05, 1.82813597e-05,
     & 1.82818471e-05, 1.82827408e-05, 1.82843657e-05, 1.82872098e-05,
     & 1.82920863e-05, 1.83010300e-05, 1.83173026e-05, 1.83458148e-05,
     & 1.83947973e-05, 1.84849417e-05, 1.86085855e-05, 1.87956191e-05,
     & 1.91541013e-05, 1.96501256e-05, 2.03392769e-05, 2.13358157e-05,
     & 2.27332848e-05, 2.47691753e-05, 2.77527835e-05, 3.19096345e-05,
     & 3.69028889e-05, 4.36557463e-05, 5.28355375e-05, 6.61732519e-05,
     & 8.29308083e-05, 1.10069853e-04, 1.46252852e-04, 2.06020283e-04,
     & 2.90747496e-04, 4.11136869e-04, 6.54752904e-04, 1.17836636e-03,
     & 2.40810384e-03, 5.96195264e-03, 1.74408453e-02, 1.57214854e-01,
     & 6.34718148e-01, 1.61430142e-01, 2.17755787e-02, 3.57256008e-03,
     & 5.94303637e-04, 1.05566722e-04, 1.95664621e-05, 2.16074217e-06,
     & 2.47019226e-07/

      data coef_j(1:51, 44) /
     & 5.90993561e-06, 5.90995641e-06, 5.90997201e-06,
     & 5.91000062e-06, 5.91005263e-06, 5.91014365e-06, 5.91029968e-06,
     & 5.91058575e-06, 5.91110593e-06, 5.91201634e-06, 5.91357737e-06,
     & 5.91644034e-06, 5.92164933e-06, 5.93077618e-06, 5.94645523e-06,
     & 5.97530877e-06, 6.01488215e-06, 6.07473829e-06, 6.18944359e-06,
     & 6.34811763e-06, 6.56849358e-06, 6.88701017e-06, 7.33337610e-06,
     & 7.98307221e-06, 8.93403352e-06, 1.02568260e-05, 1.18428606e-05,
     & 1.39833327e-05, 1.68858911e-05, 2.10905717e-05, 2.63556257e-05,
     & 3.48481793e-05, 4.61177632e-05, 6.46293202e-05, 9.07018097e-05,
     & 1.27489574e-04, 2.01259458e-04, 3.57792631e-04, 7.19204235e-04,
     & 1.74517798e-03, 5.82749688e-03, 1.69669437e-02, 1.55348802e-01,
     & 6.23701145e-01, 1.43451922e-01,-3.48187206e-03, 6.67723197e-04,
     & 1.48321433e-04, 2.94977261e-05, 3.38985522e-06, 3.95018156e-07/

      data coef_j(1:51, 45) /
     & 2.25483171e-06, 2.25483958e-06, 2.25484548e-06, 2.25485630e-06,
     & 2.25487597e-06, 2.25491039e-06, 2.25496940e-06, 2.25507759e-06,
     & 2.25527431e-06, 2.25561862e-06, 2.25620898e-06, 2.25729171e-06,
     & 2.25926164e-06, 2.26271320e-06, 2.26864252e-06, 2.27955359e-06,
     & 2.29451754e-06, 2.31714912e-06, 2.36051277e-06, 2.42048496e-06,
     & 2.50375229e-06, 2.62405024e-06, 2.79253700e-06, 3.03758083e-06,
     & 3.39586922e-06, 3.89356224e-06, 4.48935327e-06, 5.29197530e-06,
     & 6.37805708e-06, 7.94736586e-06, 9.90683609e-06, 1.30567384e-05,
     & 1.72201765e-05, 2.40272738e-05, 3.35631444e-05, 4.69403195e-05,
     & 7.35652864e-05, 1.29461927e-04, 2.56623835e-04, 6.10103049e-04,
     & 1.98016924e-03, 6.61162926e-03, 1.97409592e-02, 1.66436919e-01,
     & 6.81084221e-01, 1.26300556e-01,-4.66188632e-04, 8.88047485e-04,
     & 1.80476316e-04, 2.02474567e-05, 2.30519837e-06/

      data coef_j(1:51, 46) /
     & 7.66178811e-07,
     & 7.66181465e-07, 7.66183455e-07, 7.66187103e-07, 7.66193736e-07,
     & 7.66205345e-07, 7.66225246e-07, 7.66261733e-07, 7.66328077e-07,
     & 7.66444192e-07, 7.66643289e-07, 7.67008435e-07, 7.67672784e-07,
     & 7.68836788e-07, 7.70836355e-07, 7.74515824e-07, 7.79561768e-07,
     & 7.87192752e-07, 8.01812442e-07, 8.22027724e-07, 8.50088062e-07,
     & 8.90613005e-07, 9.47343962e-07, 1.02979830e-06, 1.15025096e-06,
     & 1.31737793e-06, 1.51718372e-06, 1.78595349e-06, 2.14900946e-06,
     & 2.67250263e-06, 3.32462241e-06, 4.37002472e-06, 5.74739251e-06,
     & 7.99091094e-06, 1.11202703e-05, 1.54900517e-05, 2.41361492e-05,
     & 4.21368915e-05, 8.26176837e-05, 1.93279196e-04, 6.10921593e-04,
     & 1.98181686e-03, 6.63905502e-03, 2.11121110e-02, 1.47647210e-01,
     & 7.41809339e-01, 1.33790339e-01, 5.01487182e-04, 1.04568765e-03,
     & 1.20288596e-04, 1.34382316e-05/

      data coef_j(1:51, 47) /
     & 8.57088619e-08, 8.57091533e-08,
     & 8.57093719e-08, 8.57097727e-08, 8.57105014e-08, 8.57117766e-08,
     & 8.57139628e-08, 8.57179708e-08, 8.57252587e-08, 8.57380140e-08,
     & 8.57598846e-08, 8.57999953e-08, 8.58729720e-08, 8.60008310e-08,
     & 8.62204624e-08, 8.66245821e-08, 8.71787179e-08, 8.80165955e-08,
     & 8.96213548e-08, 9.18393144e-08, 9.49161071e-08, 9.93558454e-08,
     & 1.05563824e-07, 1.14572453e-07, 1.27704487e-07, 1.45874719e-07,
     & 1.67529054e-07, 1.96553240e-07, 2.35594091e-07, 2.91602843e-07,
     & 3.60979046e-07, 4.71448010e-07, 6.15864232e-07, 8.48941386e-07,
     & 1.17061211e-06, 1.61468936e-06, 2.48048153e-06, 4.24535627e-06,
     & 8.09776422e-06, 1.81670384e-05, 5.32821613e-05, 1.54559697e-04,
     & 4.28901135e-04, 9.55275430e-04,-3.76288900e-03, 1.28758626e-01,
     & 7.30925185e-01, 1.35950613e-01, 3.50239034e-03, 7.39490102e-04,
     & 7.93709107e-05/

      data coef_j(1:51, 48) /
     & 2.06439653e-08, 2.06440354e-08, 2.06440880e-08,
     & 2.06441844e-08, 2.06443596e-08, 2.06446663e-08, 2.06451920e-08,
     & 2.06461559e-08, 2.06479084e-08, 2.06509758e-08, 2.06562352e-08,
     & 2.06658810e-08, 2.06834303e-08, 2.07141775e-08, 2.07669938e-08,
     & 2.08641748e-08, 2.09974301e-08, 2.11989155e-08, 2.15848062e-08,
     & 2.21181348e-08, 2.28579477e-08, 2.39254208e-08, 2.54179319e-08,
     & 2.75835584e-08, 3.07400247e-08, 3.51068015e-08, 4.03100133e-08,
     & 4.72828475e-08, 5.66603189e-08, 7.01107091e-08, 8.67680891e-08,
     & 1.13287218e-07, 1.47950783e-07, 2.03890911e-07, 2.81096290e-07,
     & 3.87699179e-07, 5.95628575e-07, 1.01992811e-06, 1.94824133e-06,
     & 4.38748695e-06, 1.30179794e-05, 3.88485065e-05, 1.16417342e-04,
     & 3.47961738e-04, 1.00762108e-03, 2.73564972e-04, 1.32228865e-01,
     & 7.23792490e-01, 1.23663010e-01,-9.29885927e-03,-1.23193146e-04/

      data coef_j(1:51, 49) /
     & 4.56812442e-09, 4.56813986e-09, 4.56815144e-09, 4.56817267e-09,
     & 4.56821126e-09, 4.56827881e-09, 4.56839460e-09, 4.56860689e-09,
     & 4.56899290e-09, 4.56966849e-09, 4.57082688e-09, 4.57295137e-09,
     & 4.57681661e-09, 4.58358866e-09, 4.59522127e-09, 4.61662468e-09,
     & 4.64597233e-09, 4.69034496e-09, 4.77532270e-09, 4.89275528e-09,
     & 5.05562923e-09, 5.29059184e-09, 5.61901966e-09, 6.09538996e-09,
     & 6.78936667e-09, 7.74882510e-09, 8.89123179e-09, 1.04209298e-08,
     & 1.24762158e-08, 1.54208935e-08, 1.90632192e-08, 2.48537007e-08,
     & 3.24104017e-08, 4.45831096e-08, 6.13489390e-08, 8.44499966e-08,
     & 1.29392210e-07, 2.20783682e-07, 4.19850164e-07, 9.39902731e-07,
     & 2.76543520e-06, 8.18408673e-06, 2.43693695e-05, 7.29812344e-05,
     & 2.19196312e-04, 1.10653705e-03, 8.66793851e-04, 1.36464968e-01,
     & 7.60579030e-01, 1.08315064e-01,-7.14703041e-03/

      data coef_j(1:51, 50) /
     & 9.32044857e-10,
     & 9.32047995e-10, 9.32050349e-10, 9.32054664e-10, 9.32062510e-10,
     & 9.32076240e-10, 9.32099779e-10, 9.32142934e-10, 9.32221403e-10,
     & 9.32358740e-10, 9.32594222e-10, 9.33026093e-10, 9.33811827e-10,
     & 9.35188459e-10, 9.37553133e-10, 9.41903948e-10, 9.47869514e-10,
     & 9.56888953e-10, 9.74161074e-10, 9.98027764e-10, 1.03112604e-09,
     & 1.07886618e-09, 1.14558244e-09, 1.24232345e-09, 1.38320084e-09,
     & 1.57787370e-09, 1.80953521e-09, 2.11953699e-09, 2.53574655e-09,
     & 3.13154835e-09, 3.86780770e-09, 5.03700719e-09, 6.56094325e-09,
     & 9.01229904e-09, 1.23833002e-08, 1.70205327e-08, 2.60240567e-08,
     & 4.42839754e-08, 8.39202093e-08, 1.86998871e-07, 5.46516447e-07,
     & 1.60572524e-06, 4.74612620e-06, 1.41224686e-05, 4.23264569e-05,
     & 2.22304979e-04, 1.15531950e-03, 2.47181017e-03, 1.20822713e-01,
     & 8.05702466e-01, 1.42819359e-02/

      data coef_j(1:51, 51) /
     & 3.98573562e-11, 3.98574863e-11,
     & 3.98575839e-11, 3.98577629e-11, 3.98580882e-11, 3.98586575e-11,
     & 3.98596336e-11, 3.98614230e-11, 3.98646767e-11, 3.98703713e-11,
     & 3.98801355e-11, 3.98980426e-11, 3.99306215e-11, 3.99876986e-11,
     & 4.00857347e-11, 4.02660912e-11, 4.05133383e-11, 4.08870542e-11,
     & 4.16023791e-11, 4.25901010e-11, 4.39585256e-11, 4.59296366e-11,
     & 4.86791694e-11, 5.26561680e-11, 5.84281163e-11, 6.63696338e-11,
     & 7.57734928e-11, 8.82877885e-11, 1.04980903e-10, 1.28693399e-10,
     & 1.57746760e-10, 2.03423868e-10, 2.62280518e-10, 3.55705134e-10,
     & 4.82255574e-10, 6.53598367e-10, 9.79681374e-10, 1.62290687e-09,
     & 2.96780567e-09, 6.28523094e-09, 1.69238753e-08, 4.47834951e-08,
     & 1.15119869e-07, 2.80762913e-07, 6.12865151e-07, 7.29517944e-07,
     &-1.85281351e-05,-3.70871197e-04,-9.85146118e-03, 9.43947195e-02,
     & 9.92892411e-01/

!---------------------------- josh EXECUTION ---------------------------

      abtot_nu(1:ndepth) = a_cont(1:ndepth) + a_line(1:ndepth) +
     &                     sigma_c(1:ndepth) + sigma_l(1:ndepth)
      alpha_nu(1:ndepth) = (sigma_c(1:ndepth) + sigma_l(1:ndepth)) /
     &                      abtot_nu(1:ndepth)
      snubar(1:ndepth) = (a_cont(1:ndepth) * s_cont(1:ndepth) +
     &                    a_line(1:ndepth) * s_line(1:ndepth)) /
     &                   (a_cont(1:ndepth) + a_line(1:ndepth))

      call integ(rhodz(1:ndepth), abtot_nu(1:ndepth),
     &           taunu(1:ndepth), abtot_nu(1) * rhodz(1))

!.... THE SECTION FOR NO SCATTERING THAT WAS HERE HAS BEEN REMOVED
!.... BECAUSE THE SOLUTION ALWAYS INCLUDES SCATTERING

      maxj = 0
      if(taunu(1) .gt. xtau8(nxtau)) maxj = 1

      if(taunu(1) .le. xtau8(nxtau)) then ! NOT OPTICALLY THICK
!!!!     maxj = map1(taunu(1:ndepth), snubar(1:ndepth), 
         maxj = map_cs(taunu(1:ndepth), snubar(1:ndepth),
     &               xtau8(1:nxtau), xsbar8(1:nxtau))

!!!!     maxj = map1(taunu(1:ndepth), alpha_nu(1:ndepth),  
         maxj = map_cs(taunu(1:ndepth), alpha_nu(1:ndepth),
     &               xtau8(1:nxtau), xalpha8(1:nxtau))

         do l = 1, nxtau

!.... IN CASE OF BAD INTERPOLATION

            xalpha(l) = max(xalpha8(l), 0.0d0)
!!!!        xsbar(l)  = max(xsbar8(l), 1.0d-38)
            xsbar(l)  = max(xsbar8(l), real_tiny)

            if(xtau8(l) .lt. taunu(1)) then
!!!!           xsbar(l)  = max(snubar(1), 1.0d-38)
               xsbar(l)  = max(snubar(1), real_tiny)
               xalpha(l) = max(alpha_nu(1), 0.0d0)
            end if

            xs(l) = xsbar(l)
            diag(l) = 1.0 - xalpha(l) * coef_j(l,l)
            xsbar(l) = (1.0 - xalpha(l)) * xsbar(l)
         end do

         l = 1

         do
            if_err = .false.

            do k = nxtau, 1, -1
               delxs = sum(coef_j(k,1:nxtau) * xs(1:nxtau))
               delxs = (delxs * xalpha(k) + xsbar(k) - xs(k)) / diag(k)
               if(abs(delxs/xs(k)) .gt. 0.00001d0) if_err = .true.
!!!!           xs(k) = max(xs(k) + delxs, 1.0e-38)
!.... tiny(1.0) = 2^(-126) = 1.175E-38 = SMALLEST r4_type
               xs(k) = max(xs(k) + delxs, tiny(1.0))
            end do

            if(.not. if_err) exit
            l = l + 1
            if(l .gt. nxtau) exit ! THE LIMIT IS ARBITRARY
         end do

!!!!     xs8(1:nxtau) = real(xs(1:nxtau), r8_type)
         xs8(1:nxtau) = xs(1:nxtau)

!!!!     mdummy = map1(xtau8(1:nxtau), xs8(1:nxtau), 
         mdummy = map_cs(xtau8(1:nxtau), xs8(1:nxtau),
     &                 taunu(1:maxj),  snu(1:maxj))
      end if ! TAUNU(1) .LE. XTAU8(NXTAU)

      if(maxj .lt. ndepth) then
         maxj1 = maxj+1
         if(maxj .eq. 1) maxj1 = 1

         snu(maxj1:ndepth) = snubar(maxj1:ndepth)

         m = max(maxj-1, 1)
         error = 1.0d0
         l = 1

         do
            if_neg = .false.                                   ! ATLAS12
            j = m                                              ! ATLAS12

            do                                                 ! ATLAS12

               if(snu(j) .le. 0.0d0) then                      ! ATLAS12
                  if_neg = .true.                              ! ATLAS12
                  snubar(m:ndepth) = bnu(m:ndepth)             ! ATLAS12
                  snu(m:ndepth) = bnu(m:ndepth)                ! ATLAS12
                  exit
               end if                                          ! ATLAS12

               j = j + 1                                       ! ATLAS12
               if(j .gt. ndepth) exit                          ! ATLAS12
            end do                                             ! ATLAS12

            call deriv(taunu(m:ndepth), snu(m:ndepth), hnu(m:ndepth))

            j = m                                              ! ATLAS12

            do                                                 ! ATLAS12

               if(hnu(j) .le. 0.0d0) then                      ! ATLAS12
                  if_neg = .true.                              ! ATLAS12
                  snubar(m:ndepth) = bnu(m:ndepth)             ! ATLAS12
                  snu(m:ndepth) = bnu(m:ndepth)                ! ATLAS12
                  call deriv(taunu(m:ndepth), snu(m:ndepth),   ! ATLAS12
     &                       hnu(m:ndepth))                    ! ATLAS12
                  exit
               end if                                          ! ATLAS12

               j = j + 1                                       ! ATLAS12
               if(j .gt. ndepth) exit                          ! ATLAS12
            end do                                             ! ATLAS12

            hnu(m:ndepth) = hnu(m:ndepth) * third

            call deriv(taunu(maxj:ndepth), hnu(maxj:ndepth),
     &                 jmins_nu(maxj:ndepth))

            if(if_neg) jmins_nu(maxj1:ndepth) = 0.0d0
            jnu(maxj1:ndepth) = jmins_nu(maxj1:ndepth)+snu(maxj1:ndepth)

            do j = maxj1, ndepth
               snew = (1.0 - alpha_nu(j)) * snubar(j) +
     &                 alpha_nu(j) * jnu(j)
               error = abs(snew - snu(j)) / snew + error
               snu(j) = snew
            end do

            if(error .lt. 1.0d-5) exit                      ! FROM BOB'S
            l = l + 1
            if(l .gt. nxtau) exit ! LIMIT ON l IS ARBITRARY
         end do

      end if ! MAXJ .LT. NDEPTH

      if(maxj .eq. 1) then
         knu(1) = jnu(1) * third
      else

         do l = 1, nxtau
            xjs(l) = -xs(l) + sum(coef_j(l,1:nxtau) * xs(1:nxtau))
            xjs8(l) = xjs(l)
            xh(l) = sum(coef_h(l,1:nxtau) * xs(1:nxtau))
            xh8(l) = real(xh(l), r8_type)
         end do

!!!!     mdummy = map1(xtau8(1:nxtau), xjs8(1:nxtau), 
         mdummy = map_cs(xtau8(1:nxtau), xjs8(1:nxtau),
     &                 taunu(1:maxj),  jmins_nu(1:maxj))
!!!!     mdummy = map1(xtau8(1:nxtau), xh8(1:nxtau), 
         mdummy = map_cs(xtau8(1:nxtau), xh8(1:nxtau),
     &                 taunu(1:maxj),  hnu(1:maxj))

         knu(1) = sum(ck(1:nxtau) * xs(1:ndepth))

!!!!     snu(1:maxj) = max(snu(1:maxj), 1.0d-38)                !ATLAS12
         snu(1:maxj) = max(snu(1:maxj), real_tiny)              !ATLAS12
!!!!     jnu(1:maxj) = max(jmins_nu(1:maxj) + snu(1:maxj), 1.0d-38)!A12
         jnu(1:maxj) = max(jmins_nu(1:maxj) + snu(1:maxj), real_tiny)!A12
      end if

!.... SECTION TO COMPUTE SURFACE INTENSITIES
!!!!  ASSUMES REAL*8 AND LARGE EXPONENT RANGE

      if(if_int(iter)) then

         if(taunu(1) .le. xtau8(nxtau)) then ! OPTICALLY THIN
            call josh_parcoe(xs8(1:nxtau), xtau8(1:nxtau))
            ctwo(1:nxtau) = c(1:nxtau) * 2.0
            b2ct(1:nxtau) = b(1:nxtau) + ctwo(1:nxtau) * xtau8(1:nxtau)
!.... REPLACED 2019 APR
!!!!        forall(j=1:nxtau-1) b2ct1(j) = b(j) + ctwo(j) * xtau8(j+1)
            b2ct1(1:nxtau-1) = b(1:nxtau-1) +
     &                         ctwo(1:nxtau-1) * xtau8(2:nxtau)

            extau(1:nxtau,1:n_mu) = 0.0d0

            do i_mu = 1, n_mu

               do j = 1, nxtau
                  tangle = xtau8(j) / surf_mu(i_mu)
                  if(tangle .lt. 300.0d0) extau(j, i_mu) = exp(-tangle)
               end do

            end do

            surf_int(1:n_mu) = 0.0d0

            do i_mu = 1, n_mu
               j = 1

               do
                  if(extau(j, i_mu) .eq. 0.0d0) exit
                  surf_int(i_mu) = surf_int(i_mu) + extau(j, i_mu) *
     &                             (xs8(j) + (b2ct(j) + ctwo(j) *
     &                              surf_mu(i_mu)) * surf_mu(i_mu))
                  if(j .eq. nxtau) exit
                  surf_int(i_mu) = surf_int(i_mu) - extau(j+1, i_mu) *
     &                             (xs8(j+1) + (b2ct1(j) + ctwo(j) *
     &                              surf_mu(i_mu)) * surf_mu(i_mu))
                  j = j + 1
               end do

            end do

         else ! OPTICALLY THICK
            call josh_parcoe(snu(1:ndepth), taunu(1:ndepth))
            ctwo(1:ndepth) = c(1:ndepth) * 2.0
            b2ct(1:ndepth) = b(1:ndepth) +
     &                       ctwo(1:ndepth) * taunu(1:ndepth)
!.... REPLACED 2019 APR
!!!!        forall(j=1:ndepth-1) b2ct1(j) = b(j) + ctwo(j) * taunu(j+1)

!!!!        do concurrent(j = 1:ndepth-1)
!!!!           b2ct1(j) = b(j) + ctwo(j) * taunu(j+1)
!!!!        end do

            b2ct1(1:ndepth-1) = b(1:ndepth-1) +
     &                          ctwo(1:ndepth-1) * taunu(2:ndepth)

            do i_mu = 1, n_mu
               old = 1.0
               sum_mu = 0.0
               j = 1

               do
                  tangle = taunu(j+1) / surf_mu(i_mu)
                  new = exp(-tangle)
                  d = tangle - taunu(j) / surf_mu(i_mu)

                  if(d .gt. 0.03) then
                     sum_mu = sum_mu +
     &                        old * (snu(j) + (b2ct(j) +
     &                               ctwo(j) * surf_mu(i_mu)) *
     &                               surf_mu(i_mu)) -
     &                        new * (snu(j+1) + (b2ct1(j) +
     &                               ctwo(j) * surf_mu(i_mu)) *
     &                               surf_mu(i_mu))

                     if(tangle .ge. 300.0) then
                        surf_int(i_mu) = sum_mu
                        exit
                     end if

                  else
                     ddddd = 1.0
                     if(d .gt. 0.001) ddddd = ((((d/9.0 + 1.0) * d/8.0 +
     &                                     1.0) * d / 7.0 + 1.0) * d /
     &                                     6.0 + 1.0) * d / 5.0 + 1.0
                     sum_mu = sum_mu +
     &                        new * (snu(j) + (snu(j) +
     &                                         b2ct(j) * surf_mu(i_mu) +
     &                                         (snu(j) + (b2ct(j) +
     &                                         ctwo(j) * surf_mu(i_mu))*
     &                                         surf_mu(i_mu)) *
     &                                         (ddddd * d / 4.0 + 1.0) *
     &                                          d / 3.0) * d / 2.0) * d
                  end if

                  old = new
                  j = j + 1

                  if(j .eq. ndepth) then
                     surf_int(i_mu) = sum_mu +
     &                                old * (snu(ndepth) +
     &                                      (b2ct(ndepth) +
     &                                       ctwo(ndepth) *
     &                                       surf_mu(i_mu)) *
     &                                       surf_mu(i_mu))
                     exit
                  end if

               end do ! J = 1, NDEPTH

            end do ! I_MU = 1, N_MU

         end if ! TAUNU(1) .LE. XTAU8(NXTAU)

      end if ! IF_INT(ITER) .EQ. .TRUE.

      contains ! INTERNAL SUBROUTINE -----------------------------------

         subroutine josh_parcoe(f, x)

!.... NOW ALWAYS IN DOUBLE PRECISION

!------------------------ josh_parcoe ARGUMENTS ------------------------

         real(re_type) :: f(:)
         real(re_type) :: x(:)

!------------------------ josh_parcoe VARIABLES ------------------------

         integer(in_type) :: j
         integer(in_type) :: n

         real(re_type)    :: d
         real(re_type)    :: wt

!------------------------ josh_parcoe EXECUTION ------------------------

         n = size(x)

         c(1) = 0.0d0
         b(1) = (f(2) - f(1)) / (x(2) - x(1))
         a(1) = f(1) - x(1) * b(1)

         c(n) = 0.0d0
         b(n) = (f(n) - f(n-1)) / (x(n) - x(n-1))
         a(n) = f(n) - x(n) * b(n)

         if(n .gt. 2) then
            c(2) = 0.0d0
            b(2) = (f(3) - f(2)) / (x(3) - x(2))
            a(2) = f(2) - x(2) * b(2)

            if(n .gt. 3) then
               c(3) = 0.0d0
               b(3) = (f(4) - f(3)) / (x(4) - x(3))
               a(3) = f(3) - x(3) * b(3)
            end if

            do j = 4, n-1
               d  = (f(j) - f(j-1)) / (x(j) - x(j-1))
               c(j) = f(j+1) / ((x(j+1) - x(j)) * (x(j+1) - x(j-1))) -
     &                f(j) / ((x(j) - x(j-1)) * (x(j+1) - x(j))) +
     &                f(j-1) / ((x(j) - x(j-1)) * (x(j+1) - x(j-1)))
               b(j) = d - (x(j) + x(j-1)) * c(j)
               a(j) = f(j-1) - x(j-1) * d + x(j) * x(j-1) * c(j)
            end do

            do j = 4, n-1

               if(c(j) .ne. 0.0d0) then
                  wt = abs(c(j+1)) / (abs(c(j+1)) + abs(c(j)))
                  a(j) = a(j+1) + wt * (a(j) - a(j+1))
                  b(j) = b(j+1) + wt * (b(j) - b(j+1))
                  c(j) = c(j+1) + wt * (c(j) - c(j+1))
               end if

            end do

            a(n-1) = a(n)
            b(n-1) = b(n)
            c(n-1) = c(n)
         end if

         end subroutine josh_parcoe

!.... END CONTAINS

      end subroutine josh

!****************** E N D  S U B R O U T I N E  J O S H ****************

      subroutine josh_f ! FEAUTRIER METHOD

!.... 2019 APR - REPLACED forall BY do concurrent
!.... 2014 MAY - CHANGED module_atlas_dimensions TO module_code_dimensions
!.... 2014 APR - RENAMED rhox TO rhodz FOR RHO * DZ
!.... 2011 JUN - MADE IF_INT AND IF_SFLUX ARRAYS DIMENSION MAX_ITER
!.... 2007 DEC - CHANGED jmins TO jmins_nu
!....          - CHANGED abtot TO abtot_nu, alpha TO alpha_nu
!.... 2007 JUN - REPLACED ifsurf NUMBER BY LOGICALS if_sflux AND if_int
!.... 2007 MAR - CHANGED nrhox TO ndepth
!....          - MADE subroutine meudon INTERNAL
!.... 2007 JAN - CHANGED maxd TO max_d
!.... 1994 FEB - IF TAUNU(1) .LT. 0.2 SOLVE THE TRANSFER EQUATION
!....            IF TAUNU(1) .GE. 0.2, SET FLUX = BB

      use abtot_vars                 ! abtot_nu, alpha_nu
      use atmosphere_parameters, only: ndepth
      use code_dimensions,       only: max_d
      use freq_vars,             only: bnu
      use gaussian_parameters        ! nmu3/4/8 mu3/4/8 wt3/4/8
      use if_vars,               only: if_int
      use intensity_vars,        only: n_mu, surf_int, surf_mu
      use iter_vars,             only: iter
      use rad_vars                   ! hnu, jmins_nu, jnu, knu, snu,
                                     ! taunu
      use rhodz_var                  ! rhodz
      use total_opacity,         only: a_cont, a_line, sigma_c, sigma_l
      use var_types

      implicit none

!--------------------------- INTERFACE BLOCK ---------------------------

      interface

         subroutine deriv(x, f, dfdx)
         use var_types
         real(re_type), intent(out) :: dfdx(:)
         real(re_type), intent(in)  :: f(:)
         real(re_type), intent(in)  :: x(:)
         end subroutine deriv

         subroutine integ(x, f, fint, start)
         use var_types
         real(re_type), intent(in)  :: f(:)
         real(re_type), intent(out) :: fint(:)
         real(re_type), intent(in)  :: start
         real(re_type), intent(in)  :: x(:)
         end subroutine integ

      end interface

!--------------------------- josh_f CONSTANT ---------------------------

      real(re_type), parameter :: third = 1.0d0 / 3.0d0

!-------------------------- josh_f VARIABLES ---------------------------

      integer(in_type) :: i_mu ! ANGLE INDEX
      integer(in_type) :: j    ! DEPTH INDEX

      real(re_type) :: ddtaunu(max_d)
      real(re_type) :: dtaudm(max_d)
      real(re_type) :: dtaunu(max_d)
      real(re_type) :: extau(max_d)
      real(re_type) :: slope
      real(re_type) :: therm(max_d)

!-------------------------- josh_f EXECUTION ---------------------------

!.... TOTAL OPACITY AT THIS FREQUENCY
      abtot_nu(1:ndepth) = a_cont(1:ndepth) + a_line(1:ndepth) +
     &                     sigma_c(1:ndepth) + sigma_l(1:ndepth)

!.... SCATTERING / TOTAL OPACITY AT THIS FREQUENCY
      alpha_nu(1:ndepth) = (sigma_c(1:ndepth) + sigma_l(1:ndepth)) /
     &                     abtot_nu(1:ndepth)

!.... THERMAL EMISSION AT THIS FREQUENCY
      therm(1:ndepth) = (1.0d0 - alpha_nu(1:ndepth)) * bnu(1:ndepth)

!.... VERTICAL OPTICAL DEPTH FOR THIS FREQUENCY & ODF STEP
      call integ(rhodz(1:ndepth), abtot_nu(1:ndepth),
     &           taunu(1:ndepth), abtot_nu(1) * rhodz(1))

!.... INITIALIZE hnu TO THE DIFFUSION APPROXIMATION

      call deriv(taunu(1:ndepth), bnu(1:ndepth) * third, hnu(1:ndepth))

      if(taunu(1) .gt. 20.0d0) then ! SKIP IF OPTICALLY THICK - 20 = BOB'S
         jnu(1:ndepth) = bnu(1:ndepth)
         knu(1:ndepth) = bnu(1:ndepth) * third
         snu(1:ndepth) = bnu(1:ndepth)
         jmins_nu(1:ndepth) = 0.0d0
         surf_int(1:n_mu) = bnu(1)

      else

!.... dtaunu IS BETWEEN J AND J+1, CENTERED AT J+1/2

!.... REPLACED 2019 APR
!!!!     forall(j = 1:ndepth-1) dtaunu(j) = 0.5d0 *
!!!! &                          (abtot_nu(j) + abtot_nu(j+1)) *
!!!! &                          (rhodz(j+1) - rhodz(j))

         do concurrent(j = 1:ndepth-1)
            dtaunu(j) = 0.5d0 * (abtot_nu(j) + abtot_nu(j+1)) *
     &                          (rhodz(j+1) - rhodz(j))
         end do

!!!! FOLLOWING LINE IS SLIGHTLY CLOSER TO JOSH RESULT, BUT LESS SMOOTH
         dtaunu(1) = taunu(1)
         dtaunu(ndepth) = dtaunu(ndepth-1)
!!!!     dtaunu(ndepth) = taunu(ndepth) - taunu(ndepth-1)

!!!!     forall(j = 2:ndepth-1) ddtaunu(j) = 2.0d0 /
!!!! &                                      (dtaunu(j-1) + dtaunu(j))

!.... THIS ddtaunu GIVES jnu AND hnu SLIGHTLY CLOSER TO BOB'S josh

         call deriv(rhodz(1:ndepth), taunu(1:ndepth), dtaudm(1:ndepth))

!.... REPLACED 2019 APR
!!!!     forall(j = 2:ndepth-1) ddtaunu(j) = 1.0d0 /
!!!! &    (dtaudm(j) * 0.5d0 * (rhodz(j+1) - rhodz(j-1)))

         do concurrent(j = 2:ndepth-1)
            ddtaunu(j) = 1.0d0 / (dtaudm(j) *
     &                            0.5d0 * (rhodz(j+1) - rhodz(j-1)))
         end do

         call meudon

!!!!     snu(1:ndepth) = therm(1:ndepth) +
!!!! &                   alpha_nu(1:ndepth) * jnu(1:ndepth)
!!!!     jmins_nu(1:ndepth) = jnu(1:ndepth) - snu(1:ndepth)
      end if

      contains ! INTERNAL SUBROUTINE -----------------------------------

         subroutine meudon

!--------------------------- INTERFACE BLOCK ---------------------------

         interface

            subroutine matinv(a)
            use var_types
            real(re_type), intent(inout) :: a(:, :)
            end subroutine matinv

         end interface

!-------------------------- meudon CONSTANTS ---------------------------

!.... ASSIGN THE NUMBER OF ANGLES DESIRED

         integer(in_type), parameter :: num_mu = nmu3
!!!!     integer(in_type), parameter :: num_mu = nmu4
!!!!     integer(in_type), parameter :: num_mu = nmu8

!-------------------------- meudon VARIABLES ---------------------------

         integer(in_type) :: im

         logical, save :: first = .true.

         real(re_type)       :: aa(num_mu)
         real(re_type)       :: bb(num_mu, num_mu)
         real(re_type)       :: bbinv(num_mu, num_mu)
         real(re_type)       :: cc(num_mu)
         real(re_type)       :: dd(num_mu, num_mu, max_d)
         real(re_type)       :: dtinv(max_d)
         real(re_type)       :: factor
         real(re_type)       :: factor_mu
         real(re_type)       :: h_mu(max_d, num_mu)
         real(re_type)       :: j_mu(max_d, num_mu)
         real(re_type), save :: mu(num_mu)
         real(re_type), save :: mu2(num_mu)
         real(re_type)       :: psi(num_mu, max_d)
         real(re_type)       :: q(num_mu)
         real(re_type), save :: wt(num_mu)
         real(re_type), save :: wtmu(num_mu)
         real(re_type), save :: wtmu2(num_mu)

!-------------------------- meudon EXECUTION ---------------------------

         if(first) then ! SET UP mu AND wt FOR THE DESIRED nmu

            if(num_mu .eq. 3) then
               mu(1:num_mu) = mu3(1:num_mu)
               wt(1:num_mu) = wt3(1:num_mu)
            else if(num_mu .eq. 4) then
               mu(1:num_mu) = mu4(1:num_mu)
               wt(1:num_mu) = wt4(1:num_mu)
            else if(num_mu .eq. 8) then
               mu(1:num_mu) = mu8(1:num_mu)
               wt(1:num_mu) = wt8(1:num_mu)
            end if

            mu2(1:num_mu) = mu(1:num_mu) * mu(1:num_mu)
            wtmu(1:num_mu) = wt(1:num_mu) * mu(1:num_mu)
            wtmu2(1:num_mu) = wt(1:num_mu) * mu2(1:num_mu)
            first = .false.
         end if

         dtinv(1:ndepth-1) = 1.0d0 / dtaunu(1:ndepth-1)

!.... UPPER BOUNDARY.  SECOND-ORDER CONDITION

         aa(1:num_mu) = 0.0d0
         cc(1:num_mu) = mu(1:num_mu) * dtinv(1)
         factor = 0.5d0 * dtaunu(1)

         do im = 1, num_mu
            factor_mu = factor / mu(im)
            q(im) = therm(1) * factor_mu
            bb(im, 1:num_mu) = -factor_mu * alpha_nu(1) * wt(1:num_mu)
            bb(im, im) = bb(im, im) + 1.0d0 + cc(im) + factor_mu
         end do

         bbinv(1:num_mu, 1:num_mu) = bb(1:num_mu, 1:num_mu)
         call matinv(bbinv(1:num_mu, 1:num_mu))

         do im = 1, num_mu
            psi(im, 1) = sum(bbinv(im, 1:num_mu) * q(1:num_mu))
            dd(im, 1:num_mu, 1) = bbinv(im, 1:num_mu) * cc(1:num_mu)
         end do

!.... NORMAL DEPTH POINTS

         do j = 2, ndepth - 1
            aa(1:num_mu) = mu2(1:num_mu) * dtinv(j-1) * ddtaunu(j)
            cc(1:num_mu) = mu2(1:num_mu) * dtinv(j) * ddtaunu(j)
            q(1:num_mu) = therm(j) + aa(1:num_mu) * psi(1:num_mu, j-1)

            do im = 1, num_mu
               bb(im, 1:num_mu) = -wt(1:num_mu) * alpha_nu(j)
               bb(im, im) = bb(im, im) + 1.0d0 + aa(im) + cc(im)
            end do

            do im = 1, num_mu
               bb(im, 1:num_mu) = bb(im, 1:num_mu) -
     &                           aa(im) * dd(im, 1:num_mu, j-1)
            end do

            bbinv(1:num_mu, 1:num_mu) = bb(1:num_mu, 1:num_mu)
            call matinv(bbinv(1:num_mu, 1:num_mu))

            do im = 1, num_mu
               psi(im, j) = sum(bbinv(im, 1:num_mu) * q(1:num_mu))
               dd(im, 1:num_mu, j) = bbinv(im, 1:num_mu) * cc(1:num_mu)
            end do

         end do

!.... LOWER BOUNDARY.  SECOND-ORDER CONDITION

         aa(1:num_mu) = mu(1:num_mu) * dtinv(ndepth-1)
         factor = 0.5d0 * dtaunu(ndepth-1)

         do im = 1, num_mu
            factor_mu = factor / mu(im)
            q(im) = bnu(ndepth) + mu(im) * hnu(ndepth) +
     &                            factor_mu * therm(ndepth) +
     &                            aa(im) * psi(im, ndepth - 1)
            bb(im, 1:num_mu) = -factor_mu * alpha_nu(ndepth) *
     &                          wt(1:num_mu)
            bb(im, im) = bb(im, im) + 1.0d0 + aa(im) + factor_mu
         end do

         do im = 1, num_mu
            bb(im, 1:num_mu) = bb(im, 1:num_mu) -
     &                         aa(im) * dd(im, 1:num_mu, ndepth-1)
         end do

         bbinv(1:num_mu, 1:num_mu) = bb(1:num_mu, 1:num_mu)
         call matinv(bbinv(1:num_mu, 1:num_mu))

         do im = 1, num_mu
            psi(im, ndepth) = sum(bbinv(im, 1:num_mu) * q(1:num_mu))
            j_mu(ndepth, im) = psi(im, ndepth)
         end do

!.... BACK SUBSTITUTION

         do j = ndepth-1, 1, -1

            do im = 1, num_mu
               j_mu(j, im) = sum(dd(im, 1:num_mu, j) *
     &                           j_mu(j+1, 1:num_mu)) + psi(im, j)
            end do

         end do

         jnu(1) = sum(wt(1:num_mu) * j_mu(1, 1:num_mu))     ! MM: 83.44
         hnu(1) = sum(wtmu(1:num_mu) * j_mu(1, 1:num_mu))   ! MM: 83.45
!!!! &                / (1.0d0 + 0.5d0 * dtaunu(1) / mu(1:num_mu)))
         knu(1) = sum(wtmu2(1:num_mu) * j_mu(1, 1:num_mu))  ! MM: 83.46

!.... REPLACED 2019 APR
!!!!     forall(j = 2:ndepth-1)
         do concurrent(j = 2:ndepth-1)
            jnu(j) = sum(wt(1:num_mu) * j_mu(j, 1:num_mu))
            knu(j) = sum(wtmu2(1:num_mu) * j_mu(j, 1:num_mu))
         end do
!!!!     end forall

!.... h_mu IS ALONG THE RAY, SO CONVERT RADIAL taunu TO ALONG THE RAY

         do im = 1, num_mu
            call deriv(taunu(1:ndepth)/mu(im), j_mu(1:ndepth, im),
     &                 h_mu(1:ndepth, im))
         end do

!.... REPLACED 2019 APR
!!!!     forall(j = 2:ndepth-1) hnu(j) = sum(wtmu(1:num_mu) *
!!!! &                                       h_mu(j, 1:num_mu))

         do concurrent(j = 2:ndepth-1)
            hnu(j) = sum(wtmu(1:num_mu) * h_mu(j, 1:num_mu))
         end do

         jnu(ndepth) = sum(wt(1:num_mu) * j_mu(ndepth, 1:num_mu))
         knu(ndepth) = jnu(ndepth) * third ! EDDINGTON APPROXIMATION
         snu(1:ndepth) = therm(1:ndepth) +
     &                   alpha_nu(1:ndepth) * jnu(1:ndepth)
         jmins_nu(1:ndepth) = jnu(1:ndepth) - snu(1:ndepth)

         if(if_int(iter)) then ! THE SURFACE INTENSITY

!.... THIS IS THE ANALYTIC INTEGRATION OF THE SOURCE FUNCTION,
!.....REPRESENTED BY (A + B * TAU), FROM TAU(J) TO TAU(J+1).
!.... THIS AGREES WITH BOB'S JOSH TO ABOUT 0.2%

!.... NOTE: n_mu = NUMBER OF INTENSITIES SPECIFIED DURING READIN
!....            .NE. THE LOCAL CONSTANT num_mu

            surf_int(1:n_mu) = 0.0d0

            do i_mu = 1, n_mu
               extau(1:ndepth) = exp(-taunu(1:ndepth) / surf_mu(i_mu))

               do j = 1, ndepth-1
                  slope = (snu(j+1) - snu(j)) / (taunu(j+1) - taunu(j))
                  surf_int(i_mu) = surf_int(i_mu) +
     &                             extau(j) * (snu(j) +
     &                                         slope * surf_mu(i_mu)) -
     &                             extau(j+1) * (snu(j) + slope *
     &                                           (surf_mu(i_mu) +
     &                                           taunu(j+1) - taunu(j)))
               end do

            end do

         end if ! IF_INT(ITER) .EQ. .TRUE.

         end subroutine meudon

!.... END CONTAINS

      end subroutine josh_f

!*************** E N D  S U B R O U T I N E  J O S H _ F ***************

      subroutine josh_r ! RYBICKI'S VERSION OF FEAUTRIER METHOD
!....                     AS DESCRIBED BY MIHALAS AND HUMMER (1974) BUT
!....                     USING SOME NOTATION FROM MIHALAS & MIHALAS (FRH)

!.... 2014 MAY - CHANGED module_atlas_dimensions TO module_code_dimensions
!.... 2014 APR - RENAMED rhox TO rhodz FOR RHO * DZ
!.... 2011 JUN - MADE IF_INT AND IF_SFLUX ARRAYS DIMENSION MAX_ITER
!.... 2007 DEC - CHANGED jmins TO jmins_nu
!....          - CHANGED abtot TO abtot_nu, alpha TO alpha_nu
!.... 2007 JUN - REPLACED ifsurf NUMBER BY LOGICALS if_sflux AND if_int
!.... 2007 MAR - CHANGED nrhox TO ndepth
!....          - MADE subroutine rybicki INTERNAL
!.... 2007 JAN - CHANGED maxd TO max_d

      use abtot_vars                 ! abtot_nu, alpha_nu
      use atmosphere_parameters, only: ndepth
      use code_dimensions,       only: max_d, max_mu
      use freq_vars,             only: bnu
      use gaussian_parameters        ! nmu3/4/8 mu3/4/8 wt3/4/8
      use if_vars,               only: if_int
      use intensity_vars,        only: n_mu, surf_int, surf_mu
      use iter_vars,             only: iter
      use rad_vars                   ! hnu, jmins_nu, jnu, knu, snu,
                                     ! taunu
      use rhodz_var                  ! rhodz
      use total_opacity,         only: a_cont, a_line, sigma_c, sigma_l
      use var_types

      implicit none

!--------------------------- INTERFACE BLOCK ---------------------------

      interface

         subroutine deriv(x, f, dfdx)
         use var_types
         real(re_type), intent(out) :: dfdx(:)
         real(re_type), intent(in)  :: f(:)
         real(re_type), intent(in)  :: x(:)
         end subroutine deriv

         function expi(n, x) result(exp_i) ! BOB'S EXP INTEGRAL
         use var_types
         integer(in_type), intent(in) :: n
         real(re_type)                :: exp_i
         real(re_type),    intent(in) :: x
         end function expi

         subroutine integ(x, f, fint, start)
         use var_types
         real(re_type), intent(in)  :: f(:)
         real(re_type), intent(out) :: fint(:)
         real(re_type), intent(in)  :: start
         real(re_type), intent(in)  :: x(:)
         end subroutine integ

      end interface

!-------------------------- josh_r CONSTANTS ---------------------------

!.... ASSIGN THE NUMBER OF ANGLES DESIRED

      integer(in_type), parameter :: num_mu = nmu3
!!!!  integer(in_type), parameter :: num_mu = nmu4
!!!!  integer(in_type), parameter :: num_mu = nmu8

      real(re_type),    parameter :: third = 1.0d0 / 3.0d0

!-------------------------- josh_r VARIABLES ---------------------------

      integer(in_type) :: i_mu ! ANGLE INDEX
!.... j IS LOCAL TO do concurrent
!!!!  integer(in_type) :: j    ! DEPTH INDEX

      logical, save :: first = .true.

      real(re_type)       :: ddtaunu(max_d)
      real(re_type)       :: dtaudm(max_d)
      real(re_type)       :: dtaunu(max_d)
      real(re_type)       :: extau(max_d)
      real(re_type), save :: mu(num_mu)
      real(re_type)       :: slope
      real(re_type)       :: therm(max_d)
      real(re_type), save :: wt_mu(num_mu)

!-------------------------- josh_r EXECUTION ---------------------------

      if(first) then ! SET UP mu AND wt FOR THE DESIRED nmu

         if(num_mu .eq. 3) then
            mu(1:num_mu) = mu3(1:num_mu)
            wt_mu(1:num_mu) = wt3(1:num_mu)
         else if(num_mu .eq. 4) then
            mu(1:num_mu) = mu4(1:num_mu)
            wt_mu(1:num_mu) = wt4(1:num_mu)
         else if(num_mu .eq. 8) then
            mu(1:num_mu) = mu8(1:num_mu)
            wt_mu(1:num_mu) = wt8(1:num_mu)
         end if

         first = .false.
      end if

!.... VARIABLES THAT DEPEND ON FREQUENCY AND/OR ODF STEP

!.... TOTAL OPACITY AT THIS FREQUENCY
      abtot_nu(1:ndepth) = a_cont(1:ndepth) + a_line(1:ndepth) +
     &                     sigma_c(1:ndepth) + sigma_l(1:ndepth)

!.... SCATTERING / TOTAL OPACITY AT THIS FREQUENCY
      alpha_nu(1:ndepth) = (sigma_c(1:ndepth) + sigma_l(1:ndepth)) /
     &                     abtot_nu(1:ndepth)

!.... THERMAL EMISSION
      therm(1:ndepth) = (1.0d0 - alpha_nu(1:ndepth)) * bnu(1:ndepth)

!.... VERTICAL OPTICAL DEPTH FOR THIS FREQUENCY & ODF STEP
      call integ(rhodz(1:ndepth), abtot_nu(1:ndepth),
     &           taunu(1:ndepth), abtot_nu(1) * rhodz(1))

!.... INITIALIZE hnu TO THE DIFFUSION APPROXIMATION

      call deriv(taunu(1:ndepth), bnu(1:ndepth) * third, hnu(1:ndepth))

      if(taunu(1) .gt. 20.0d0) then ! SKIP IF OPTICALLY THICK - 20 = BOB'S
         jnu(1:ndepth) = bnu(1:ndepth)
         knu(1:ndepth) = bnu(1:ndepth) * third
         snu(1:ndepth) = bnu(1:ndepth)
         jmins_nu(1:ndepth) = 0.0d0
         surf_int(1:n_mu) = bnu(1)

      else ! LOOP OVER ALL ANGLES

!.... VERTICAL dtaunu IS BETWEEN j AND j+1, CENTERED AT j+1/2
!.... REPLACED 2019 APR
!!!!     forall(j = 1:ndepth-1) dtaunu(j) = 0.5d0 *
!!!! &                          (abtot_nu(j) + abtot_nu(j+1)) *
!!!! &                          (rhodz(j+1) - rhodz(j))

         do concurrent(integer(in_type) :: j = 1:ndepth-1)
            dtaunu(j) = 0.5d0 * (abtot_nu(j) + abtot_nu(j+1)) *
     &                          (rhodz(j+1) - rhodz(j))
         end do

         dtaunu(1) = taunu(1) ! SEEMS SLIGHTLY CLOSER TO BOB'S josh
         dtaunu(ndepth) = dtaunu(ndepth-1)
!!!!     dtaunu(ndepth) = taunu(ndepth) - taunu(ndepth-1)

!!!!     forall(j = 2:ndepth-1) ddtaunu(j) = 2.0d0 /
!!!! &                                      (dtaunu(j-1) + dtaunu(j))

!.... THIS ddtaunu GIVES jnu AND hnu SLIGHTLY CLOSER TO BOB'S josh

         call deriv(rhodz(1:ndepth), taunu(1:ndepth), dtaudm(1:ndepth))
!.... REPLACED 2019 APR
!!!!     forall(j = 2:ndepth-1) ddtaunu(j) = 1.0d0 /
!!!! &    (dtaudm(j) * 0.5d0 * (rhodz(j+1) - rhodz(j-1)))

         do concurrent(integer(in_type) :: j = 2:ndepth-1)
            ddtaunu(j) = 1.0d0 / (dtaudm(j) *
     &                            0.5d0 * (rhodz(j+1) - rhodz(j-1)))
         end do

         do i_mu = 1, num_mu ! AT EACH ANGLE DO ALL DEPTHS 
            call rybicki
         end do

         snu(1:ndepth) = therm(1:ndepth) +
     &                   alpha_nu(1:ndepth) * jnu(1:ndepth)
         jmins_nu(1:ndepth) = jnu(1:ndepth) - snu(1:ndepth)
      end if ! TEST ON TAUNU(1) .GT. 1.0

      contains ! INTERNAL SUBROUTINE -----------------------------------

         subroutine rybicki

!--------------------------- INTERFACE BLOCK ---------------------------

         interface

            subroutine matinv(a)
            use var_types
            real(re_type), intent(inout) :: a(:, :)
            end subroutine matinv

            subroutine tridag(dl, d, du, r, u) 
            use var_types
            real(re_type), intent(in)  :: d(1:)
            real(re_type), intent(in)  :: dl(2:)
            real(re_type), intent(in)  :: du(1:)
            real(re_type), intent(in)  :: r(1:)
            real(re_type), intent(out) :: u(1:)
            end subroutine tridag

         end interface

!-------------------------- rybicki VARIABLES --------------------------

         integer(in_type) :: ij
         integer(in_type) :: im
         integer(in_type) :: j

         real(re_type)       :: arr_t(max_d, max_d)
         real(re_type)       :: arr_t1(max_d, max_d)
         real(re_type), save :: arr_t1u(max_d, max_d, max_mu)
         real(re_type)       :: arr_u(max_d, max_d)
         real(re_type)       :: arr_v(max_d, max_d)
         real(re_type), save :: arr_w(max_d, max_d)
         real(re_type)       :: arr_w1(max_d, max_d)
         real(re_type)       :: d(max_d)
         real(re_type)       :: dl(max_d)
         real(re_type)       :: du(max_d)
         real(re_type)       :: factor
         real(re_type)       :: factor1
         real(re_type)       :: factor2
         real(re_type)       :: h_mu(max_d, max_mu)
         real(re_type)       :: j_mu(max_d, max_mu)
         real(re_type)       :: mu2
         real(re_type), save :: mu_val(max_mu)
         real(re_type)       :: unit_matrix(max_d, max_d)
         real(re_type)       :: vec_k(max_d)
         real(re_type), save :: vec_q(max_d)
         real(re_type), save :: vec_t1k(max_d, max_mu)
         real(re_type), save :: wt(max_mu)
         real(re_type), save :: wtmu(max_mu)
         real(re_type), save :: wtmu2(max_mu)

!-------------------------- rybicki EXECUTION --------------------------

         if(i_mu .eq. 1) then ! RESET FOR EACH ANGLE
            arr_t1u(:, :, :) = 0.0d0
            arr_w(:, :) = 0.0d0
!.... REPLACED 2019 APR
!!!!        forall(j = 1:ndepth) arr_w(j, j) = 1.0d0 ! DIAGONAL

            do concurrent(j = 1:ndepth)
               arr_w(j, j) = 1.0d0 ! DIAGONAL
            end do

            vec_t1k(:, :) = 0.0d0
            vec_q(:) = 0.0d0
            unit_matrix(:, :) = 0.0d0
!.... REPLACED 2019 APR
!!!!        forall(j = 1:ndepth) unit_matrix(j, j) = 1.0d0 ! UNIT MATRIX DIAG

            do concurrent(j = 1:ndepth)
               unit_matrix(j, j) = 1.0d0 ! UNIT MATRIX DIAGONAL
            end do

         end if

!.... TO BE SAFE, INITIALIZE THESE FOR EACH ANGLE

         arr_t(:, :) = 0.0d0
         arr_u(:, :) = 0.0d0
         arr_v(:, :) = 0.0d0
         vec_k(:) = 0.0d0

!.... STORE ANGLES, WEIGHTS AND THEIR PRODUCTS

         mu_val(i_mu) = mu(i_mu)
         wt(i_mu) = wt_mu(i_mu)
         wtmu(i_mu) = wt(i_mu) * mu_val(i_mu)
         wtmu2(i_mu) = wtmu(i_mu) * mu_val(i_mu)
         mu2 = mu_val(i_mu) * mu_val(i_mu)

!.... UPPER BOUNDARY.  SECOND-ORDER DIFFERENCE EQUATION

         factor1 = 0.5d0 * dtaunu(1) / mu_val(i_mu)
         factor2 = mu_val(i_mu) / dtaunu(1)
         arr_t(1, 1) = 1.0d0 + factor1 + factor2
         arr_t(1, 2) = -factor2
         arr_u(1, 1) = -factor1 * alpha_nu(1) ! SCATTERING TERM
         arr_v(1, 1) = wt(i_mu)
         vec_k(1) = factor1 * therm(1)        ! THERMAL TERM

!.... BODY OF THE ATMOSPHERE.  DIFFERENCE EQUATIONS FOR NOW

         do j = 2, ndepth-1
            factor = mu2 * ddtaunu(j)
            factor1 = factor / dtaunu(j-1)
            factor2 = factor / dtaunu(j)
            arr_t(j, j-1) = -factor1
            arr_t(j, j) = 1.0d0 + factor1 + factor2
            arr_t(j, j+1) = -factor2
            arr_u(j, j) = -alpha_nu(j)        ! SCATTERING TERM
            arr_v(j, j) = wt(i_mu)
            vec_k(j) = therm(j)               ! THERMAL TERM
         end do

!.... LOWER BOUNDARY.  SECOND-ORDER DIFFUSION APPROXIMATION

         factor1 = 0.5d0 * dtaunu(ndepth) / mu_val(i_mu)
         factor2 = mu_val(i_mu) / dtaunu(ndepth)
         arr_t(ndepth, ndepth-1) = -factor2
         arr_t(ndepth, ndepth) = 1.0d0 + factor1 + factor2
         arr_u(ndepth, ndepth) = -factor1 * alpha_nu(ndepth)! SCATTERING
         arr_v(ndepth, ndepth) = wt(i_mu)
         vec_k(ndepth) = factor1 * therm(ndepth) +
     &                   bnu(ndepth) + mu_val(i_mu) * hnu(ndepth) ! THERMAL

!.... INVERT arr_t

!.... REPLACED 2019 APR
!!!!     forall(j = 2:ndepth)   dl(j) = arr_t(j, j-1)
!!!!     forall(j = 1:ndepth)   d(j)  = arr_t(j, j)
!!!!     forall(j = 1:ndepth-1) du(j) = arr_t(j, j+1)

         do concurrent(j = 2:ndepth)
            dl(j) = arr_t(j, j-1)
         end do

         do concurrent(j = 1:ndepth)
            d(j) = arr_t(j, j)
         end do

         do concurrent(j = 1:ndepth-1)
            du(j) = arr_t(j, j+1)
         end do

         do j = 1, ndepth
            call tridag(dl(2:ndepth), d(1:ndepth),
     &                  du(1:ndepth-1), unit_matrix(1:ndepth, j),
     &                  arr_t1(1:ndepth, j))
         end do

!.... USE THAT arr_u IS DIAGONAL IN arr_t1 * arr_u. KEEP EACH RAY

         do ij = 1, ndepth
!.... REPLACED 2019 APR
!!!!        forall(j = 1:ndepth) arr_t1u(ij, j, i_mu) = arr_t1(ij, j) *
!!!! &                                                  arr_u(j, j)

            do concurrent(j = 1:ndepth)
               arr_t1u(ij, j, i_mu) = arr_t1(ij, j) * arr_u(j, j)
            end do

         end do

!.... USE THAT vec_k IS A VECTOR IN arr_t1 * vec_k. KEEP EACH RAY

!.... REPLACED 2019 APR
!!!!     forall(j = 1:ndepth) vec_t1k(j, i_mu) = sum(arr_t1(j, 1:ndepth)
!!!! &                                           * vec_k(1:ndepth))

         do concurrent(j = 1:ndepth)
            vec_t1k(j, i_mu) = sum(arr_t1(j, 1:ndepth) *
     &                             vec_k(1:ndepth))
         end do

!.... USE THAT arr_v IS DIAGONAL TO ACCUMULATE arr_w

!.... REPLACED 2019 APR
!!!!     forall(ij = 1:ndepth) arr_w(ij, 1:ndepth) = arr_w(ij, 1:ndepth)
!!!! &                         + arr_v(ij, ij)
!!!! &                         * arr_t1u(ij, 1:ndepth, i_mu)

         do concurrent(ij = 1:ndepth)
            arr_w(ij, 1:ndepth) = arr_w(ij, 1:ndepth) + arr_v(ij, ij) *
     &                            arr_t1u(ij, 1:ndepth, i_mu)
         end do

!.... USE THAT arr_v IS DIAGONAL TO ACCUMULATE vec_q

!.... REPLACED 2019 APR
!!!!     forall(j = 1:ndepth) vec_q(j) = vec_q(j) + arr_v(j, j)
!!!! &                                            * vec_t1k(j, i_mu)

         do concurrent(j = 1:ndepth)
            vec_q(j) = vec_q(j) + arr_v(j, j) * vec_t1k(j, i_mu)
         end do

         if(i_mu .eq. num_mu) then  ! HAVE COMPLETED ALL THE ANGLES

!.... INVERT arr_w

            arr_w1(1:ndepth, 1:ndepth) = arr_w(1:ndepth, 1:ndepth)
            call matinv(arr_w1(1:ndepth, 1:ndepth))

!.... SOLVE FOR jnu AT EACH DEPTH

!.... REPLACED 2019 APR
!!!!        forall(j = 1:ndepth) jnu(j) = sum(arr_w1(j, 1:ndepth) *
!!!! &                                        vec_q(1:ndepth))

            do concurrent(j = 1:ndepth)
               jnu(j) = sum(arr_w1(j, 1:ndepth) * vec_q(1:ndepth))
            end do

!.... USE jnu TO COMPTUE j_mu VALUES AT EACH ANGLE AND DEPTH

            do im = 1, num_mu
!.... REPLACED 2019 APR
!!!!           forall(j = 1:ndepth) j_mu(j, im) = vec_t1k(j, im) -
!!!! &                sum(arr_t1u(j, 1:ndepth, im) * jnu(1:ndepth))

               do concurrent(j = 1:ndepth)
                  j_mu(j, im) = vec_t1k(j, im) -
     &                          sum(arr_t1u(j, 1:ndepth, im) *
     &                              jnu(1:ndepth))
               end do

            end do

!.... USE j_mu TO COMPUTE hnu AND knu

            hnu(1) = sum(wtmu(1:num_mu) * j_mu(1, 1:num_mu))! MM: 83.44
!!!! &                   / (1.0d0 +
!!!! &                      0.5d0 * dtaunu(1) / mu_val(1:num_mu)))
            knu(1) = sum(wtmu2(1:num_mu) * j_mu(1, 1:num_mu))!MM: 83.46

!.... REPLACED 2019 APR
!!!!        forall(j = 2:ndepth-1) knu(j) = sum(wtmu2(1:num_mu) *
!!!! &                                          j_mu(j, 1:num_mu))

            do concurrent(j = 2:ndepth-1)
               knu(j) = sum(wtmu2(1:num_mu) * j_mu(j, 1:num_mu))
            end do

!.... h_mu IS ALONG THE RAY, SO CONVERT RADIAL taunu TO ALONG THE RAY

            do im = 1, num_mu
               call deriv(taunu(1:ndepth)/mu(im), j_mu(1:ndepth, im),
     &                    h_mu(1:ndepth, im))
             end do

!.... REPLACED 2019 APR
!!!!        forall(j = 2:ndepth-1) hnu(j) = sum(wtmu(1:num_mu) *
!!!! &                                          h_mu(j, 1:num_mu))

            do concurrent(j = 2:ndepth-1)
               hnu(j) = sum(wtmu(1:num_mu) * h_mu(j, 1:num_mu))
            end do

            knu(ndepth) = jnu(ndepth) * third ! EDDINGTON APPROXIMATION

            if(if_int(iter)) then

!.... THIS IS THE ANALYTIC INTEGRATION OF THE SOURCE FUNCTION,
!.....REPRESENTED BY (A + B * TAU), FROM TAU(J) TO TAU(J+1).
!.... THIS AGREES WITH BOB'S JOSH TO ABOUT 0.2%

!.... NOTE: n_mu = NUMBER OF INTENSITIES SPECIFIED DURING READIN
!....            .NE. THE LOCAL CONSTANT num_mu

               surf_int(1:n_mu) = 0.0d0

               do i_mu = 1, n_mu
                  extau(1:ndepth) = exp(-taunu(1:ndepth)/surf_mu(i_mu))

                  do j = 1, ndepth-1
                     slope = (snu(j+1) - snu(j)) /
     &                       (taunu(j+1) - taunu(j))
                     surf_int(i_mu) = surf_int(i_mu) +
     &                                extau(j) * (snu(j) + slope *
     &                                            surf_mu(i_mu)) -
     &                                extau(j+1) * (snu(j) + slope *
     &                                             (surf_mu(i_mu) +
     &                                              taunu(j+1) -
     &                                              taunu(j)))
                  end do

               end do

            end if ! IF_INT(ITER) .EQ. .TRUE.

         end if ! I_MU .EQ. NUM_MU

         end subroutine rybicki

!.... END CONTAINS

      end subroutine josh_r

!**************** E N D  S U B R O U T I N E  J O S H _ R **************
